angular.module('bagu.controllers')
  //房间列表
  .controller('RoomChooseCtrl', function($scope, $stateParams, $ionicPopover, $ionicHistory, $state, $http, _, $Cache, $Window,
    $Rent, $Room, $Building, $Product, $Invoice, $PriceLog, $RoomValue, $Analysis, DATALIST, EVENTS) {

    /***保存url参数***/
    //设置楼房ID
    $scope.buildingId = $stateParams.buildingId;
    $scope.SELECTTEMPL = $Room.getChooseTemplate();

    /***右侧功能菜单***/
    //房间颜色定义
    //enum:{0:'未出租',1:'已出租，未选择',2:'选择'}
    var ROOM_COLORS = {
      0: 'roomcantselected',
      1: 'roomunselected',
      2: 'roomselected'
    };

    /***工具函数***/
    //根据房间状态,判断是否属于某个选择模板
    function isTemplateStatus(template, status) {
      return !template.status || template.status.length === 0 || _.contains(template.status, status);
    }
    //更新选择房间
    function setChooseRooms() {
      //保存选择的房间数据
      var roomids = [];
      _.forEach($scope.rooms, function(room) {
        if (room.selected) {
          roomids.push(room._id);
        }
      });
      $Room.setChooseRooms($scope.buildingId, roomids);
    }
    //设置房间选择状态
    //submit:标示是否即时更新房间选择数据
    function setRoomStatus(room, selected, submit) {
      if (room.disable || room.selected === selected) {
        return;
      }
      submit = _.isEmpty(submit) ? true : submit;
      room.selected = selected;
      room.class = room.selected ? ROOM_COLORS[2] : ROOM_COLORS[1];
      //设置选择模板的数据统计
      _.forEach($scope.SELECTTEMPL, function(template) {
        if (isTemplateStatus(template, room.roomstatus)) {
          template.selectcount += selected ? 1 : -1;
          template.selected = template.selectcount === template.count;
        }
      });
      if (submit) {
        setChooseRooms();
      }
    }
    //设置房间列表
    function setRooms(buildingId, rooms) {
      $scope.rooms = [];
      var selectRooms = $Room.getChooseRooms(buildingId);
      //清空选择模板的数据统计
      _.forEach($scope.SELECTTEMPL, function(template) {
        template.count = 0;
      });
      //设置房间状态
      _.forEach(rooms, function(room) {
        //是否未出租,自住或未设置?
        var disable = !(room.roomstatus >= DATALIST.ROOM_STATUS.ORDER && room.roomstatus < DATALIST.ROOM_STATUS.NULL);
        //是否已选择?
        var selectedId = _.find(selectRooms, function(item) {
          return item === room._id;
        });
        var selected = !disable && selectedId === room._id;
        //样式
        var classname = disable ? ROOM_COLORS[0] : selected ? ROOM_COLORS[2] : ROOM_COLORS[1];
        //设置选择模板的数据统计
        _.forEach($scope.SELECTTEMPL, function(template) {
          if (!disable && isTemplateStatus(template, room.roomstatus)) {
            template.count++;
          }
        });
        $scope.rooms.push({
          _id: room._id,
          code: room.code,
          roomstatus: room.roomstatus,
          floor: room.floor,
          disable: disable,
          selected: selected,
          class: classname
        });
      });
      //整理楼层及房间
      $scope.floorrooms = $Room.groupRooms(buildingId, $scope.rooms);
    }

    /***接口函数***/
    //依据模板选择
    $scope.doSelect = function(templ) {
      var selected = !templ.selected;
      _.forEach($scope.rooms, function(room) {
        if (isTemplateStatus(templ, room.roomstatus)) {
          setRoomStatus(room, selected, false);
        }
      });
      //更新服务对象
      setChooseRooms();
    };
    //房间点击
    $scope.doRoomClick = function(room) {
      setRoomStatus(room, !room.selected);
      if (!room.selected) {
        $scope.SELECTTEMPL.dALL.selected = false;
      } else {
        var allselect = false;
        var unselectroom = _.find($scope.rooms, function(room) {
          return !room.selected && !room.disable;
        });
        $scope.SELECTTEMPL.dALL.selected = !unselectroom;
      }
    };

    /***远程请求***/
    $scope.getBuildingRooms = function(buildingId, isRefresh) {
      isRefresh = isRefresh || false;
      $Room.getBuildingRooms(buildingId, isRefresh).then(function(roomlist) {
          setRooms(buildingId, roomlist);
        })
        .finally(function() {
          // Stop the ion-refresher from spinning
          if (isRefresh) {
            $scope.$broadcast('scroll.refreshComplete');
          }
        });
    };

    /***初始化数据***/
    function refreshData() {
      //获取楼房信息
      $Building.getBuilding($scope.buildingId).then(function(building) {
        $scope.title = building.name;
        $scope.building = building;

        //获取房间列表
        $scope.getBuildingRooms($scope.buildingId);
      });
    }

    /***监听事件***/
    //退出时清理菜单对象
    $scope.$on('$destroy', function() {

    });
    //返回到缓存页面
    $scope.$on('$ionicView.enter', function(e) {
      refreshData();
    });
  });
