angular.module('ngCordova', [
  'ngCordova.plugins'
]);


angular.module('ngCordova.plugins', [
  'ngCordova.plugins.ble',
  'ngCordova.plugins.bluetoothSerial',
  'ngCordova.plugins.media',
  'ngCordova.plugins.toast',
  'ngCordova.plugins.camera',
  'ngCordova.plugins.file',
  'ngCordova.plugins.fileTransfer',
  'ngCordova.plugins.appVersion',
  'ngCordova.plugins.inAppBrowser'
]);

//#### Begin Individual Plugin Code ####
//  install   :   cordova plugin add https://github.com/don/cordova-plugin-ble-central.git
//  link      :   https://github.com/don/cordova-plugin-ble-central

/* globals ble: true */
angular.module('ngCordova.plugins.ble', [])

.factory('$cordovaBLE', ['$q', '$timeout', '$log', function ($q, $timeout, $log) {

	return {
		scan: function (services, seconds) {
			var q = $q.defer();

			ble.startScan(services, function (result) {
				q.notify(result);
			}, function (error) {
				q.reject(error);
			});

			$timeout(function () {
				ble.stopScan(function () {
					q.resolve();
				}, function (error) {
					q.reject(error);
				});
			}, seconds * 1000);

			return q.promise;
		},

		startScan: function (services, callback, errorCallback) {
			return ble.startScan(services, callback, errorCallback);
		},

		stopScan: function () {
			var q = $q.defer();
			ble.stopScan(function () {
				q.resolve();
			}, function (error) {
				q.reject(error);
			});
			return q.promise;
		},

		connect: function (deviceID) {
			var q = $q.defer();
			ble.connect(deviceID, function (result) {
				q.resolve(result);
			}, function (error) {
				q.reject(error);
			});
			return q.promise;
		},

		disconnect: function (deviceID) {
			var q = $q.defer();
			ble.disconnect(deviceID, function (result) {
				q.resolve(result);
			}, function (error) {
				q.reject(error);
			});
			return q.promise;
		},

		read: function (deviceID, serviceUUID, characteristicUUID) {
			var q = $q.defer();
			ble.read(deviceID, serviceUUID, characteristicUUID, function (result) {
				q.resolve(result);
			}, function (error) {
				q.reject(error);
			});
			return q.promise;
		},

		write: function (deviceID, serviceUUID, characteristicUUID, data) {
			var q = $q.defer();
			ble.write(deviceID, serviceUUID, characteristicUUID, data, function (result) {
				q.resolve(result);
			}, function (error) {
				q.reject(error);
			});
			return q.promise;
		},

		writeWithoutResponse: function (deviceID, serviceUUID, characteristicUUID, data) {
			var q = $q.defer();
			ble.writeWithoutResponse(deviceID, serviceUUID, characteristicUUID, data, function (result) {
				q.resolve(result);
			}, function (error) {
				q.reject(error);
			});
			return q.promise;
		},

		writeCommand: function (deviceID, serviceUUID, characteristicUUID, data) {
			$log.warning('writeCommand is deprecated, use writeWithoutResponse');
			return this.writeWithoutResponse(deviceID, serviceUUID, characteristicUUID, data);
		},

		startNotification: function (deviceID, serviceUUID, characteristicUUID, callback, errorCallback) {
			return ble.startNotification(deviceID, serviceUUID, characteristicUUID, callback, errorCallback);
		},

		stopNotification: function (deviceID, serviceUUID, characteristicUUID) {
			var q = $q.defer();
			ble.stopNotification(deviceID, serviceUUID, characteristicUUID, function (result) {
				q.resolve(result);
			}, function (error) {
				q.reject(error);
			});
			return q.promise;
		},

		isConnected: function (deviceID) {
			var q = $q.defer();
			ble.isConnected(deviceID, function (result) {
				q.resolve(result);
			}, function (error) {
				q.reject(error);
			});
			return q.promise;
		},

		enable: function () {
			var q = $q.defer();
			ble.enable(function (result) {
				q.resolve(result);
			}, function (error) {
				q.reject(error);
			});
			return q.promise;
		},

		isEnabled: function () {
			var q = $q.defer();
			ble.isEnabled(function (result) {
				q.resolve(result);
			}, function (error) {
				q.reject(error);
			});
			return q.promise;
		}
	};
  }]);

// install   :     cordova plugin add https://github.com/don/BluetoothSerial.git
// link      :     https://github.com/don/BluetoothSerial

angular.module('ngCordova.plugins.bluetoothSerial', [])

.factory('$cordovaBluetoothSerial', ['$q', '$window', function ($q, $window) {

	return {
		connect: function (address) {
			var q = $q.defer();
			var disconnectionPromise = $q.defer();
			var isConnected = false;
			$window.bluetoothSerial.connect(address, function () {
				isConnected = true;
				q.resolve(disconnectionPromise);
			}, function (error) {
				if (isConnected === false) {
					disconnectionPromise.reject(error);
				}
				q.reject(error);
			});
			return q.promise;
		},

		// not supported on iOS
		connectInsecure: function (address) {
			var q = $q.defer();
			$window.bluetoothSerial.connectInsecure(address, function () {
				q.resolve();
			}, function (error) {
				q.reject(error);
			});
			return q.promise;
		},

		disconnect: function () {
			var q = $q.defer();
			$window.bluetoothSerial.disconnect(function () {
				q.resolve();
			}, function (error) {
				q.reject(error);
			});
			return q.promise;
		},

		list: function () {
			var q = $q.defer();
			$window.bluetoothSerial.list(function (data) {
				q.resolve(data);
			}, function (error) {
				q.reject(error);
			});
			return q.promise;
		},

		discoverUnpaired: function () {
			var q = $q.defer();
			$window.bluetoothSerial.discoverUnpaired(function (data) {
				q.resolve(data);
			}, function (error) {
				q.reject(error);
			});
			return q.promise;
		},

		setDeviceDiscoveredListener: function () {
			var q = $q.defer();
			$window.bluetoothSerial.setDeviceDiscoveredListener(function (data) {
				q.notify(data);
			});
			return q.promise;
		},

		clearDeviceDiscoveredListener: function () {
			$window.bluetoothSerial.clearDeviceDiscoveredListener();
		},

		showBluetoothSettings: function () {
			var q = $q.defer();
			$window.bluetoothSerial.showBluetoothSettings(function () {
				q.resolve();
			}, function (error) {
				q.reject(error);
			});
			return q.promise;
		},

		isEnabled: function () {
			var q = $q.defer();
			$window.bluetoothSerial.isEnabled(function () {
				q.resolve();
			}, function () {
				q.reject();
			});
			return q.promise;
		},

		enable: function () {
			var q = $q.defer();
			$window.bluetoothSerial.enable(function () {
				q.resolve();
			}, function () {
				q.reject();
			});
			return q.promise;
		},

		isConnected: function () {
			var q = $q.defer();
			$window.bluetoothSerial.isConnected(function () {
				q.resolve();
			}, function () {
				q.reject();
			});
			return q.promise;
		},

		available: function () {
			var q = $q.defer();
			$window.bluetoothSerial.available(function (data) {
				q.resolve(data);
			}, function (error) {
				q.reject(error);
			});
			return q.promise;
		},

		read: function () {
			var q = $q.defer();
			$window.bluetoothSerial.read(function (data) {
				q.resolve(data);
			}, function (error) {
				q.reject(error);
			});
			return q.promise;
		},

		readUntil: function (delimiter) {
			var q = $q.defer();
			$window.bluetoothSerial.readUntil(delimiter, function (data) {
				q.resolve(data);
			}, function (error) {
				q.reject(error);
			});
			return q.promise;
		},

		write: function (data) {
			var q = $q.defer();
			$window.bluetoothSerial.write(data, function () {
				q.resolve();
			}, function (error) {
				q.reject(error);
			});
			return q.promise;
		},

		subscribe: function (delimiter) {
			var q = $q.defer();
			$window.bluetoothSerial.subscribe(delimiter, function (data) {
				q.notify(data);
			}, function (error) {
				q.reject(error);
			});
			return q.promise;
		},

		subscribeRawData: function () {
			var q = $q.defer();
			$window.bluetoothSerial.subscribeRawData(function (data) {
				q.notify(data);
			}, function (error) {
				q.reject(error);
			});
			return q.promise;
		},

		unsubscribe: function () {
			var q = $q.defer();
			$window.bluetoothSerial.unsubscribe(function () {
				q.resolve();
			}, function (error) {
				q.reject(error);
			});
			return q.promise;
		},

		unsubscribeRawData: function () {
			var q = $q.defer();
			$window.bluetoothSerial.unsubscribeRawData(function () {
				q.resolve();
			}, function (error) {
				q.reject(error);
			});
			return q.promise;
		},

		clear: function () {
			var q = $q.defer();
			$window.bluetoothSerial.clear(function () {
				q.resolve();
			}, function (error) {
				q.reject(error);
			});
			return q.promise;
		},

		readRSSI: function () {
			var q = $q.defer();
			$window.bluetoothSerial.readRSSI(function (data) {
				q.resolve(data);
			}, function (error) {
				q.reject(error);
			});
			return q.promise;
		}
	};
}]);
// install   :      cordova plugin add cordova-plugin-media
// link      :      https://github.com/apache/cordova-plugin-media

/* globals Media: true */
angular.module('ngCordova.plugins.media', [])

.service('WebMedia', ['$q', '$interval', function ($q, $interval) {
		var MEDIA_NONE = 0;
		var MEDIA_STARTING = 1;
		var MEDIA_RUNNING = 2;
		var MEDIA_PAUSED = 3;
		var MEDIA_STOPPED = 4;

		function WebMedia(src, option) {
			option = option || {};
			this.q = this.mediaStatus = null;
			this.mediaPosition = -1;
			this.mediaTimer = null;
			this.mediaDuration = -1;
			this.media = document.createElement('audio');
			this.media.autoplay = false; //不自动播放
			this.media.loop = false; //不循环播放
			this.media.src  =  src;
			var me = this;
			//播放完成
			this.media.onended = function (success) {
				me.clearTimer();
				me.resetValues();
				if (me.q) {
					me.q.notify({
						status: MEDIA_STOPPED
					});
					me.q.resolve(success);
				}
			};
			//加载出错?
			this.media.onerror = function (err) {
				me.clearTimer();
				me.resetValues();
				if (me.q) {
					me.q.reject(err);
				}
			};
			//取消加载?
			this.media.onabort = function (err) {
				me.clearTimer();
				me.resetValues();
				if (me.q) {
					me.q.reject(err);
				}
			};
			//开始播放?
			this.media.onplay = function () {
				if (me.q) {
					me.q.notify({
						status: MEDIA_STARTING
					});
				}
			};
			//暂停?
			this.media.onpause = function () {
				if (me.q) {
					me.q.notify({
						status: MEDIA_PAUSED
					});
				}
			};
		}

		WebMedia.prototype.setTimer = function () {
			var me = this;
			var media = me.media;
			if (angular.isDefined(me.mediaTimer)) {
				return;
			}

			me.mediaTimer = $interval(function () {
				if (me.mediaDuration < 0) {
					me.mediaDuration = media.duration;
					if (me.q && me.mediaDuration > 0) {
						me.q.notify({
							duration: me.mediaDuration
						});
					}
				}
				me.mediaPosition = media.currentTime >= 0 ? media.currentTime : -1;
				if (me.q) {
					me.q.notify({
						position: me.mediaPosition
					});
				}

			}, 1000);
		};

		WebMedia.prototype.clearTimer = function () {
			if (angular.isDefined(this.mediaTimer)) {
				$interval.cancel(this.mediaTimer);
				this.mediaTimer = undefined;
			}
		};

		WebMedia.prototype.resetValues = function () {
			this.mediaPosition = -1;
			this.mediaDuration = -1;
		};

		// iOS quirks :
		// -  myMedia.play({ numberOfLoops: 2 }) -> looping
		// -  myMedia.play({ playAudioWhenScreenIsLocked : false })
		WebMedia.prototype.play = function (options) {
			this.q = $q.defer();

			if (typeof options !== 'object') {
				options = {};
			}

			this.media.play(options);

			this.setTimer();

			return this.q.promise;
		};

		WebMedia.prototype.pause = function () {
			this.clearTimer();
			this.media.pause();
		};

		WebMedia.prototype.stop = function () {
			this.clearTimer();
			this.media.currentTime = -1;
			this.media.pause();
			if (this.q) {
				this.q.notify({
					status: Media.MEDIA_STOPPED
				});
				this.q.resolve();
			}
		};

		WebMedia.prototype.release = function () {
			delete this.media;
			this.media = undefined;
		};

		WebMedia.prototype.seekTo = function (timing) {
			this.media.currentTime = timing;
		};

		WebMedia.prototype.setVolume = function (volume) {
			this.media.volume = volume;
		};

		WebMedia.prototype.startRecord = function () {
			if (this.q) {
				this.q.notify({
					status: -1
				});
				this.q.reject('不支持该操作');
			}
		};

		WebMedia.prototype.stopRecord = function () {
			if (this.q) {
				this.q.notify({
					status: -1
				});
				this.q.reject('不支持该操作');
			}
		};

		WebMedia.prototype.currentTime = function () {
			return this.media.currentTime;
		};

		WebMedia.prototype.getDuration = function () {
			return this.media.duration;
		};

		return WebMedia;

	  }])
	.service('NewMedia', ['$q', '$interval', function ($q, $interval) {

		function NewMedia(src) {
			this.q = this.q2 = this.q3 = this.mediaStatus = null;
			this.mediaPosition = -1;
			this.mediaTimer = null;
			this.mediaDuration = -1;
			var me = this;
			this.media = new Media(src,
				function (success) {
					me.clearTimer();
					me.resetValues();
					if (me.q) {
						me.q.resolve(success);
					}
				},
				function (error) {
					me.clearTimer();
					me.resetValues();
					if (me.q) {
						me.q.reject(error);
					}
				},
				function (status) {
					me.mediaStatus = status;
					if (me.q) {
						me.q.notify({
							status: me.mediaStatus
						});
					}
				});
		}

		NewMedia.prototype.setTimer = function () {
			var me = this;
			var media = me.media;
			if (angular.isDefined(me.mediaTimer)) {
				return;
			}

			me.mediaTimer = $interval(function () {
				if (me.mediaDuration < 0) {
					me.mediaDuration = media.getDuration();
					if (me.q && me.mediaDuration > 0) {
						me.q.notify({
							duration: me.mediaDuration
						});
					}
				}

				media.getCurrentPosition(
					// success callback
					function (position) {
						if (position > -1) {
							me.mediaPosition = position;
						}
					},
					// error callback
					function (e) {
						console.log('Error getting pos=' + e);
					});

				if (me.q) {
					me.q.notify({
						position: me.mediaPosition
					});
				}

			}, 1000);
		};

		NewMedia.prototype.clearTimer = function () {
			if (angular.isDefined(this.mediaTimer)) {
				$interval.cancel(this.mediaTimer);
				this.mediaTimer = undefined;
			}
		};

		NewMedia.prototype.resetValues = function () {
			this.mediaPosition = -1;
			this.mediaDuration = -1;
		};

		// iOS quirks :
		// -  myMedia.play({ numberOfLoops: 2 }) -> looping
		// -  myMedia.play({ playAudioWhenScreenIsLocked : false })
		NewMedia.prototype.play = function (options) {
			this.q = $q.defer();

			if (typeof options !== 'object') {
				options = {};
			}

			this.media.play(options);

			this.setTimer();

			return this.q.promise;
		};

		NewMedia.prototype.pause = function () {
			this.clearTimer();
			this.media.pause();
		};

		NewMedia.prototype.stop = function () {
			this.media.stop();
		};

		NewMedia.prototype.release = function () {
			this.media.release();
			this.media = undefined;
		};

		NewMedia.prototype.seekTo = function (timing) {
			this.media.seekTo(timing);
		};

		NewMedia.prototype.setVolume = function (volume) {
			this.media.setVolume(volume);
		};

		NewMedia.prototype.startRecord = function () {
			this.media.startRecord();
		};

		NewMedia.prototype.stopRecord = function () {
			this.media.stopRecord();
		};

		NewMedia.prototype.currentTime = function () {
			var me = this;
			me.q2 = $q.defer();
			this.media.getCurrentPosition(function (position) {
				me.q2.resolve(position);
			});
			return me.q2.promise;
		};

		NewMedia.prototype.getDuration = function () {
			var me = this;
			me.q3 = $q.defer();
			this.media.getDuration(function (duration) {
				me.q3.resolve(duration);
			});
			return me.q3.promise;
		};

		return NewMedia;

  }])
	.factory('$cordovaMedia', ['WebMedia', 'NewMedia', function (WebMedia, NewMedia) {
		return {
			newMedia: function (src) {
				//判断当前平台
				if (window.cordova && Media) {
					return new NewMedia(src);
				} else {
					return new WebMedia(src);
				}
			}
		};
  }]);
// install   :      cordova plugin add https://github.com/EddyVerbruggen/Toast-PhoneGap-Plugin.git
// link      :      https://github.com/EddyVerbruggen/Toast-PhoneGap-Plugin

angular.module('ngCordova.plugins.toast', [])

.factory('$cordovaToast', ['$q', '$window', function ($q, $window) {

	return {
		showShortTop: function (message) {
			var q = $q.defer();
			$window.plugins.toast.showShortTop(message, function (response) {
				q.resolve(response);
			}, function (error) {
				q.reject(error);
			});
			return q.promise;
		},

		showShortCenter: function (message) {
			var q = $q.defer();
			$window.plugins.toast.showShortCenter(message, function (response) {
				q.resolve(response);
			}, function (error) {
				q.reject(error);
			});
			return q.promise;
		},

		showShortBottom: function (message) {
			var q = $q.defer();
			$window.plugins.toast.showShortBottom(message, function (response) {
				q.resolve(response);
			}, function (error) {
				q.reject(error);
			});
			return q.promise;
		},

		showLongTop: function (message) {
			var q = $q.defer();
			$window.plugins.toast.showLongTop(message, function (response) {
				q.resolve(response);
			}, function (error) {
				q.reject(error);
			});
			return q.promise;
		},

		showLongCenter: function (message) {
			var q = $q.defer();
			$window.plugins.toast.showLongCenter(message, function (response) {
				q.resolve(response);
			}, function (error) {
				q.reject(error);
			});
			return q.promise;
		},

		showLongBottom: function (message) {
			var q = $q.defer();
			$window.plugins.toast.showLongBottom(message, function (response) {
				q.resolve(response);
			}, function (error) {
				q.reject(error);
			});
			return q.promise;
		},

		show: function (message, duration, position) {
			var q = $q.defer();
			$window.plugins.toast.show(message, duration, position, function (response) {
				q.resolve(response);
			}, function (error) {
				q.reject(error);
			});
			return q.promise;
		},

		hide: function () {
			var q = $q.defer();
			try {
				$window.plugins.toast.hide();
				q.resolve();
			} catch (error) {
				q.reject(error && error.message);
			}
			return q.promise;
		}
	};

}]);


// install   :   cordova plugin add cordova-plugin-camera
// link      :   https://github.com/apache/cordova-plugin-camera

angular.module('ngCordova.plugins.camera', [])

.factory('$cordovaCamera', ['$q', function ($q) {

	return {
		getPicture: function (options) {
			var q = $q.defer();

			if (!navigator.camera) {
				q.resolve(null);
				return q.promise;
			}

			navigator.camera.getPicture(function (imageData) {
				q.resolve(imageData);
			}, function (err) {
				q.reject(err);
			}, options);

			return q.promise;
		},

		cleanup: function () {
			var q = $q.defer();

			navigator.camera.cleanup(function () {
				q.resolve();
			}, function (err) {
				q.reject(err);
			});

			return q.promise;
		}
	};
}]);
// install   :     cordova plugin add cordova-plugin-file-transfer
// link      :     https://github.com/apache/cordova-plugin-file-transfer

/* globals FileTransfer: true */
angular.module('ngCordova.plugins.fileTransfer', [])
	//纯WEB版本的文件上传
	.factory('$cordovaWebFileUpload', ['$q', '$timeout', function ($q, $timeout) {

		var idCounter = 0;

		var transfers = {};
		return {
			upload: function (server, file, options, trustAllHosts) {
				var q = $q.defer();

				options = options || {};
				var uri = (options && options.encodeURI === false) ? server : encodeURI(server);

				var fileKey = options.fileKey || 'file';
				var fileName = options.fileName || 'image.jpg';
				var mimeType = options.mimeType || 'image/jpeg';
				var params = options.params || {};
				var withCredentials = options.withCredentials || false;
				// var chunkedMode = !!options.chunkedMode; // Not supported
				var headers = options.headers || {};
				var httpMethod = options.httpMethod && options.httpMethod.toUpperCase() === 'PUT' ? 'PUT' : 'POST';

				var that = this;
				var xhr = transfers[this._id] = new XMLHttpRequest();
				xhr.withCredentials = withCredentials;

				if (options && options.timeout !== undefined && options.timeout !== null) {
					$timeout(function () {
						xhr.abort();
					}, options.timeout);
					options.timeout = null;
				}

				this.onprogress = function (progress) {
					q.notify(progress);
				};

				q.promise.abort = function () {
					xhr.abort();
				}; {

					var fail = function (code, status, response) {
						if (transfers[this._id]) {
							delete transfers[this._id];
						}
						q.reject(code);
					};
					var reader = new FileReader();
					reader.onloadend = function () {
						var blob = new Blob([this.result], {
							type: mimeType
						});

						// Prepare form data to send to server
						var fd = new FormData();
						fd.append(fileKey, blob, fileName);
						for (var prop in params) {
							if (params.hasOwnProperty(prop)) {
								fd.append(prop, params[prop]);
							}
						}

						xhr.open(httpMethod, server);

						// Fill XHR headers
						for (var header in headers) {
							if (headers.hasOwnProperty(header)) {
								xhr.setRequestHeader(header, headers[header]);
							}
						}

						xhr.onload = function () {
							if (this.status === 200) {
								var result = {}; // jshint ignore:line
								result.bytesSent = blob.size;
								result.responseCode = this.status;
								result.response = this.response;
								delete transfers[that._id];
								q.resolve(result);
							} else if (this.status === 404) {
								fail(1, this.status, this.response);
							} else {
								fail(2, this.status, this.response);
							}
						};

						xhr.ontimeout = function () {
							fail(3, this.status, this.response);
						};

						xhr.onerror = function () {
							fail(4, this.status, this.response);
						};

						xhr.onabort = function () {
							fail(5, this.status, this.response);
						};

						xhr.upload.onprogress = function (e) {
							if (that.onprogress) {
								that.onprogress(e);
							}
						};

						xhr.send(fd);
						// Special case when transfer already aborted, but XHR isn't sent.
						// In this case XHR won't fire an abort event, so we need to check if transfers record
						// isn't deleted by filetransfer.abort and if so, call XHR's abort method again
						if (!transfers[that._id]) {
							xhr.abort();
						}
					};
					reader.readAsArrayBuffer(file);
				}
				return q.promise;
			}
		};
}])
	.factory('$cordovaFileTransfer', ['$q', '$timeout', function ($q, $timeout) {
		return {
			download: function (source, filePath, options, trustAllHosts) {
				var q = $q.defer();
				var ft = new FileTransfer();
				var uri = (options && options.encodeURI === false) ? source : encodeURI(source);

				if (options && options.timeout !== undefined && options.timeout !== null) {
					$timeout(function () {
						ft.abort();
					}, options.timeout);
					options.timeout = null;
				}

				ft.onprogress = function (progress) {
					q.notify(progress);
				};

				q.promise.abort = function () {
					ft.abort();
				};

				ft.download(uri, filePath, q.resolve, q.reject, trustAllHosts, options);
				return q.promise;
			},

			upload: function (server, filePath, options, trustAllHosts) {
				var q = $q.defer();
				var ft = new FileTransfer();
				var uri = (options && options.encodeURI === false) ? server : encodeURI(server);

				if (options && options.timeout !== undefined && options.timeout !== null) {
					$timeout(function () {
						ft.abort();
					}, options.timeout);
					options.timeout = null;
				}

				ft.onprogress = function (progress) {
					q.notify(progress);
				};

				q.promise.abort = function () {
					ft.abort();
				};

				ft.upload(filePath, uri, q.resolve, q.reject, options, trustAllHosts);
				return q.promise;
			}
		};
}]);

//#### Begin Individual Plugin Code ####

// install   :     cordova plugin add cordova-plugin-file
// link      :     https://github.com/apache/cordova-plugin-file

angular.module('ngCordova.plugins.file', [])

.constant('$cordovaFileError', {
	1: 'NOT_FOUND_ERR',
	2: 'SECURITY_ERR',
	3: 'ABORT_ERR',
	4: 'NOT_READABLE_ERR',
	5: 'ENCODING_ERR',
	6: 'NO_MODIFICATION_ALLOWED_ERR',
	7: 'INVALID_STATE_ERR',
	8: 'SYNTAX_ERR',
	9: 'INVALID_MODIFICATION_ERR',
	10: 'QUOTA_EXCEEDED_ERR',
	11: 'TYPE_MISMATCH_ERR',
	12: 'PATH_EXISTS_ERR'
})

.provider('$cordovaFile', [function () {

	this.$get = ['$q', '$window', '$cordovaFileError', function ($q, $window, $cordovaFileError) {

		return {

			getFreeDiskSpace: function () {
				var q = $q.defer();
				cordova.exec(function (result) {
					q.resolve(result);
				}, function (error) {
					q.reject(error);
				}, 'File', 'getFreeDiskSpace', []);
				return q.promise;
			},

			checkDir: function (path, dir) {
				var q = $q.defer();

				if ((/^\//.test(dir))) {
					q.reject('directory cannot start with \/');
				}

				try {
					var directory = path + dir;
					$window.resolveLocalFileSystemURL(directory, function (fileSystem) {
						if (fileSystem.isDirectory === true) {
							q.resolve(fileSystem);
						} else {
							q.reject({
								code: 13,
								message: 'input is not a directory'
							});
						}
					}, function (error) {
						error.message = $cordovaFileError[error.code];
						q.reject(error);
					});
				} catch (err) {
					err.message = $cordovaFileError[err.code];
					q.reject(err);
				}

				return q.promise;
			},

			checkFile: function (path, file) {
				var q = $q.defer();

				if ((/^\//.test(file))) {
					q.reject('directory cannot start with \/');
				}

				try {
					var directory = path + file;
					$window.resolveLocalFileSystemURL(directory, function (fileSystem) {
						if (fileSystem.isFile === true) {
							q.resolve(fileSystem);
						} else {
							q.reject({
								code: 13,
								message: 'input is not a file'
							});
						}
					}, function (error) {
						error.message = $cordovaFileError[error.code];
						q.reject(error);
					});
				} catch (err) {
					err.message = $cordovaFileError[err.code];
					q.reject(err);
				}

				return q.promise;
			},

			createDir: function (path, dirName, replaceBool) {
				var q = $q.defer();

				if ((/^\//.test(dirName))) {
					q.reject('directory cannot start with \/');
				}

				replaceBool = replaceBool ? false : true;

				var options = {
					create: true,
					exclusive: replaceBool
				};

				try {
					$window.resolveLocalFileSystemURL(path, function (fileSystem) {
						fileSystem.getDirectory(dirName, options, function (result) {
							q.resolve(result);
						}, function (error) {
							error.message = $cordovaFileError[error.code];
							q.reject(error);
						});
					}, function (err) {
						err.message = $cordovaFileError[err.code];
						q.reject(err);
					});
				} catch (e) {
					e.message = $cordovaFileError[e.code];
					q.reject(e);
				}

				return q.promise;
			},

			createFile: function (path, fileName, replaceBool) {
				var q = $q.defer();

				if ((/^\//.test(fileName))) {
					q.reject('file-name cannot start with \/');
				}

				replaceBool = replaceBool ? false : true;

				var options = {
					create: true,
					exclusive: replaceBool
				};

				try {
					$window.resolveLocalFileSystemURL(path, function (fileSystem) {
						fileSystem.getFile(fileName, options, function (result) {
							q.resolve(result);
						}, function (error) {
							error.message = $cordovaFileError[error.code];
							q.reject(error);
						});
					}, function (err) {
						err.message = $cordovaFileError[err.code];
						q.reject(err);
					});
				} catch (e) {
					e.message = $cordovaFileError[e.code];
					q.reject(e);
				}
				return q.promise;
			},

			removeDir: function (path, dirName) {
				var q = $q.defer();

				if ((/^\//.test(dirName))) {
					q.reject('file-name cannot start with \/');
				}

				try {
					$window.resolveLocalFileSystemURL(path, function (fileSystem) {
						fileSystem.getDirectory(dirName, {
							create: false
						}, function (dirEntry) {
							dirEntry.remove(function () {
								q.resolve({
									success: true,
									fileRemoved: dirEntry
								});
							}, function (error) {
								error.message = $cordovaFileError[error.code];
								q.reject(error);
							});
						}, function (err) {
							err.message = $cordovaFileError[err.code];
							q.reject(err);
						});
					}, function (er) {
						er.message = $cordovaFileError[er.code];
						q.reject(er);
					});
				} catch (e) {
					e.message = $cordovaFileError[e.code];
					q.reject(e);
				}
				return q.promise;
			},

			removeFile: function (path, fileName) {
				var q = $q.defer();

				if ((/^\//.test(fileName))) {
					q.reject('file-name cannot start with \/');
				}

				try {
					$window.resolveLocalFileSystemURL(path, function (fileSystem) {
						fileSystem.getFile(fileName, {
							create: false
						}, function (fileEntry) {
							fileEntry.remove(function () {
								q.resolve({
									success: true,
									fileRemoved: fileEntry
								});
							}, function (error) {
								error.message = $cordovaFileError[error.code];
								q.reject(error);
							});
						}, function (err) {
							err.message = $cordovaFileError[err.code];
							q.reject(err);
						});
					}, function (er) {
						er.message = $cordovaFileError[er.code];
						q.reject(er);
					});
				} catch (e) {
					e.message = $cordovaFileError[e.code];
					q.reject(e);
				}
				return q.promise;
			},

			removeRecursively: function (path, dirName) {
				var q = $q.defer();

				if ((/^\//.test(dirName))) {
					q.reject('file-name cannot start with \/');
				}

				try {
					$window.resolveLocalFileSystemURL(path, function (fileSystem) {
						fileSystem.getDirectory(dirName, {
							create: false
						}, function (dirEntry) {
							dirEntry.removeRecursively(function () {
								q.resolve({
									success: true,
									fileRemoved: dirEntry
								});
							}, function (error) {
								error.message = $cordovaFileError[error.code];
								q.reject(error);
							});
						}, function (err) {
							err.message = $cordovaFileError[err.code];
							q.reject(err);
						});
					}, function (er) {
						er.message = $cordovaFileError[er.code];
						q.reject(er);
					});
				} catch (e) {
					e.message = $cordovaFileError[e.code];
					q.reject(e);
				}
				return q.promise;
			},

			writeFile: function (path, fileName, text, replaceBool) {
				var q = $q.defer();

				if ((/^\//.test(fileName))) {
					q.reject('file-name cannot start with \/');
				}

				replaceBool = replaceBool ? false : true;

				var options = {
					create: true,
					exclusive: replaceBool
				};

				try {
					$window.resolveLocalFileSystemURL(path, function (fileSystem) {
						fileSystem.getFile(fileName, options, function (fileEntry) {
							fileEntry.createWriter(function (writer) {
								if (options.append === true) {
									writer.seek(writer.length);
								}

								if (options.truncate) {
									writer.truncate(options.truncate);
								}

								writer.onwriteend = function (evt) {
									if (this.error) {
										q.reject(this.error);
									} else {
										q.resolve(evt);
									}
								};

								writer.write(text);

								q.promise.abort = function () {
									writer.abort();
								};
							});
						}, function (error) {
							error.message = $cordovaFileError[error.code];
							q.reject(error);
						});
					}, function (err) {
						err.message = $cordovaFileError[err.code];
						q.reject(err);
					});
				} catch (e) {
					e.message = $cordovaFileError[e.code];
					q.reject(e);
				}

				return q.promise;
			},

			writeExistingFile: function (path, fileName, text) {
				var q = $q.defer();

				if ((/^\//.test(fileName))) {
					q.reject('file-name cannot start with \/');
				}

				try {
					$window.resolveLocalFileSystemURL(path, function (fileSystem) {
						fileSystem.getFile(fileName, {
							create: false
						}, function (fileEntry) {
							fileEntry.createWriter(function (writer) {
								writer.seek(writer.length);

								writer.onwriteend = function (evt) {
									if (this.error) {
										q.reject(this.error);
									} else {
										q.resolve(evt);
									}
								};

								writer.write(text);

								q.promise.abort = function () {
									writer.abort();
								};
							});
						}, function (error) {
							error.message = $cordovaFileError[error.code];
							q.reject(error);
						});
					}, function (err) {
						err.message = $cordovaFileError[err.code];
						q.reject(err);
					});
				} catch (e) {
					e.message = $cordovaFileError[e.code];
					q.reject(e);
				}

				return q.promise;
			},

			readAsText: function (path, file) {
				var q = $q.defer();

				if ((/^\//.test(file))) {
					q.reject('file-name cannot start with \/');
				}

				try {
					$window.resolveLocalFileSystemURL(path, function (fileSystem) {
						fileSystem.getFile(file, {
							create: false
						}, function (fileEntry) {
							fileEntry.file(function (fileData) {
								var reader = new FileReader();

								reader.onloadend = function (evt) {
									if (evt.target.result !== undefined || evt.target.result !== null) {
										q.resolve(evt.target.result);
									} else if (evt.target.error !== undefined || evt.target.error !== null) {
										q.reject(evt.target.error);
									} else {
										q.reject({
											code: null,
											message: 'READER_ONLOADEND_ERR'
										});
									}
								};

								reader.readAsText(fileData);
							});
						}, function (error) {
							error.message = $cordovaFileError[error.code];
							q.reject(error);
						});
					}, function (err) {
						err.message = $cordovaFileError[err.code];
						q.reject(err);
					});
				} catch (e) {
					e.message = $cordovaFileError[e.code];
					q.reject(e);
				}

				return q.promise;
			},

			readAsDataURL: function (path, file) {
				var q = $q.defer();

				if ((/^\//.test(file))) {
					q.reject('file-name cannot start with \/');
				}

				try {
					$window.resolveLocalFileSystemURL(path, function (fileSystem) {
						fileSystem.getFile(file, {
							create: false
						}, function (fileEntry) {
							fileEntry.file(function (fileData) {
								var reader = new FileReader();
								reader.onloadend = function (evt) {
									if (evt.target.result !== undefined || evt.target.result !== null) {
										q.resolve(evt.target.result);
									} else if (evt.target.error !== undefined || evt.target.error !== null) {
										q.reject(evt.target.error);
									} else {
										q.reject({
											code: null,
											message: 'READER_ONLOADEND_ERR'
										});
									}
								};
								reader.readAsDataURL(fileData);
							});
						}, function (error) {
							error.message = $cordovaFileError[error.code];
							q.reject(error);
						});
					}, function (err) {
						err.message = $cordovaFileError[err.code];
						q.reject(err);
					});
				} catch (e) {
					e.message = $cordovaFileError[e.code];
					q.reject(e);
				}

				return q.promise;
			},

			readAsBinaryString: function (path, file) {
				var q = $q.defer();

				if ((/^\//.test(file))) {
					q.reject('file-name cannot start with \/');
				}

				try {
					$window.resolveLocalFileSystemURL(path, function (fileSystem) {
						fileSystem.getFile(file, {
							create: false
						}, function (fileEntry) {
							fileEntry.file(function (fileData) {
								var reader = new FileReader();
								reader.onloadend = function (evt) {
									if (evt.target.result !== undefined || evt.target.result !== null) {
										q.resolve(evt.target.result);
									} else if (evt.target.error !== undefined || evt.target.error !== null) {
										q.reject(evt.target.error);
									} else {
										q.reject({
											code: null,
											message: 'READER_ONLOADEND_ERR'
										});
									}
								};
								reader.readAsBinaryString(fileData);
							});
						}, function (error) {
							error.message = $cordovaFileError[error.code];
							q.reject(error);
						});
					}, function (err) {
						err.message = $cordovaFileError[err.code];
						q.reject(err);
					});
				} catch (e) {
					e.message = $cordovaFileError[e.code];
					q.reject(e);
				}

				return q.promise;
			},

			readAsArrayBuffer: function (path, file) {
				var q = $q.defer();

				if ((/^\//.test(file))) {
					q.reject('file-name cannot start with \/');
				}

				try {
					$window.resolveLocalFileSystemURL(path, function (fileSystem) {
						fileSystem.getFile(file, {
							create: false
						}, function (fileEntry) {
							fileEntry.file(function (fileData) {
								var reader = new FileReader();
								reader.onloadend = function (evt) {
									if (evt.target.result !== undefined || evt.target.result !== null) {
										q.resolve(evt.target.result);
									} else if (evt.target.error !== undefined || evt.target.error !== null) {
										q.reject(evt.target.error);
									} else {
										q.reject({
											code: null,
											message: 'READER_ONLOADEND_ERR'
										});
									}
								};
								reader.readAsArrayBuffer(fileData);
							});
						}, function (error) {
							error.message = $cordovaFileError[error.code];
							q.reject(error);
						});
					}, function (err) {
						err.message = $cordovaFileError[err.code];
						q.reject(err);
					});
				} catch (e) {
					e.message = $cordovaFileError[e.code];
					q.reject(e);
				}

				return q.promise;
			},

			moveFile: function (path, fileName, newPath, newFileName) {
				var q = $q.defer();

				newFileName = newFileName || fileName;

				if ((/^\//.test(fileName)) || (/^\//.test(newFileName))) {
					q.reject('file-name cannot start with \/');
				}

				try {
					$window.resolveLocalFileSystemURL(path, function (fileSystem) {
						fileSystem.getFile(fileName, {
							create: false
						}, function (fileEntry) {
							$window.resolveLocalFileSystemURL(newPath, function (newFileEntry) {
								fileEntry.moveTo(newFileEntry, newFileName, function (result) {
									q.resolve(result);
								}, function (error) {
									q.reject(error);
								});
							}, function (err) {
								q.reject(err);
							});
						}, function (err) {
							q.reject(err);
						});
					}, function (er) {
						q.reject(er);
					});
				} catch (e) {
					q.reject(e);
				}
				return q.promise;
			},

			moveDir: function (path, dirName, newPath, newDirName) {
				var q = $q.defer();

				newDirName = newDirName || dirName;

				if (/^\//.test(dirName) || (/^\//.test(newDirName))) {
					q.reject('file-name cannot start with \/');
				}

				try {
					$window.resolveLocalFileSystemURL(path, function (fileSystem) {
						fileSystem.getDirectory(dirName, {
							create: false
						}, function (dirEntry) {
							$window.resolveLocalFileSystemURL(newPath, function (newDirEntry) {
								dirEntry.moveTo(newDirEntry, newDirName, function (result) {
									q.resolve(result);
								}, function (error) {
									q.reject(error);
								});
							}, function (erro) {
								q.reject(erro);
							});
						}, function (err) {
							q.reject(err);
						});
					}, function (er) {
						q.reject(er);
					});
				} catch (e) {
					q.reject(e);
				}
				return q.promise;
			},

			copyDir: function (path, dirName, newPath, newDirName) {
				var q = $q.defer();

				newDirName = newDirName || dirName;

				if (/^\//.test(dirName) || (/^\//.test(newDirName))) {
					q.reject('file-name cannot start with \/');
				}

				try {
					$window.resolveLocalFileSystemURL(path, function (fileSystem) {
						fileSystem.getDirectory(dirName, {
							create: false,
							exclusive: false
						}, function (dirEntry) {

							$window.resolveLocalFileSystemURL(newPath, function (newDirEntry) {
								dirEntry.copyTo(newDirEntry, newDirName, function (result) {
									q.resolve(result);
								}, function (error) {
									error.message = $cordovaFileError[error.code];
									q.reject(error);
								});
							}, function (erro) {
								erro.message = $cordovaFileError[erro.code];
								q.reject(erro);
							});
						}, function (err) {
							err.message = $cordovaFileError[err.code];
							q.reject(err);
						});
					}, function (er) {
						er.message = $cordovaFileError[er.code];
						q.reject(er);
					});
				} catch (e) {
					e.message = $cordovaFileError[e.code];
					q.reject(e);
				}
				return q.promise;
			},

			copyFile: function (path, fileName, newPath, newFileName) {
				var q = $q.defer();

				newFileName = newFileName || fileName;

				if ((/^\//.test(fileName))) {
					q.reject('file-name cannot start with \/');
				}

				try {
					$window.resolveLocalFileSystemURL(path, function (fileSystem) {
						fileSystem.getFile(fileName, {
							create: false,
							exclusive: false
						}, function (fileEntry) {

							$window.resolveLocalFileSystemURL(newPath, function (newFileEntry) {
								fileEntry.copyTo(newFileEntry, newFileName, function (result) {
									q.resolve(result);
								}, function (error) {
									error.message = $cordovaFileError[error.code];
									q.reject(error);
								});
							}, function (erro) {
								erro.message = $cordovaFileError[erro.code];
								q.reject(erro);
							});
						}, function (err) {
							err.message = $cordovaFileError[err.code];
							q.reject(err);
						});
					}, function (er) {
						er.message = $cordovaFileError[er.code];
						q.reject(er);
					});
				} catch (e) {
					e.message = $cordovaFileError[e.code];
					q.reject(e);
				}
				return q.promise;
			},

			readFileMetadata: function (path, file) {
				var q = $q.defer();

				if ((/^\//.test(file))) {
					q.reject('directory cannot start with \/');
				}

				try {
					var directory = path + file;
					$window.resolveLocalFileSystemURL(directory, function (fileEntry) {
						fileEntry.file(function (result) {
							q.resolve(result);
						}, function (error) {
							error.message = $cordovaFileError[error.code];
							q.reject(error);
						});
					}, function (err) {
						err.message = $cordovaFileError[err.code];
						q.reject(err);
					});
				} catch (e) {
					e.message = $cordovaFileError[e.code];
					q.reject(e);
				}

				return q.promise;
			}

			/*
			 listFiles: function (path, dir) {

			 },

			 listDir: function (path, dirName) {
			 var q = $q.defer();

			 try {
			 $window.resolveLocalFileSystemURL(path, function (fileSystem) {
			 fileSystem.getDirectory(dirName, options, function (parent) {
			 var reader = parent.createReader();
			 reader.readEntries(function (entries) {
			 q.resolve(entries);
			 }, function () {
			 q.reject('DIR_READ_ERROR : ' + path + dirName);
			 });
			 }, function (error) {
			 error.message = $cordovaFileError[error.code];
			 q.reject(error);
			 });
			 }, function (err) {
			 err.message = $cordovaFileError[err.code];
			 q.reject(err);
			 });
			 } catch (e) {
			 e.message = $cordovaFileError[e.code];
			 q.reject(e);
			 }

			 return q.promise;
			 },

			 */
		};

  }];
}]);

// install   :     cordova plugin add https://github.com/whiteoctober/cordova-plugin-app-version.git
// link      :     https://github.com/whiteoctober/cordova-plugin-app-version

angular.module('ngCordova.plugins.appVersion', [])

.factory('$cordovaAppVersion', ['$q', function ($q) {

	return {
		getAppName: function () {
			var q = $q.defer();
			cordova.getAppVersion.getAppName(function (name) {
				q.resolve(name);
			});

			return q.promise;
		},

		getPackageName: function () {
			var q = $q.defer();
			cordova.getAppVersion.getPackageName(function (pack) {
				q.resolve(pack);
			});

			return q.promise;
		},

		getVersionNumber: function () {
			var q = $q.defer();
			cordova.getAppVersion.getVersionNumber(function (version) {
				q.resolve(version);
			});

			return q.promise;
		},

		getVersionCode: function () {
			var q = $q.defer();
			cordova.getAppVersion.getVersionCode(function (code) {
				q.resolve(code);
			});

			return q.promise;
		}
	};
  }]);

// install   :     cordova plugin add cordova-plugin-inappbrowser
// link      :     https://github.com/apache/cordova-plugin-inappbrowser

angular.module('ngCordova.plugins.inAppBrowser', [])

.provider('$cordovaInAppBrowser', [function () {

	var ref;
	var defaultOptions = this.defaultOptions = {};

	this.setDefaultOptions = function (config) {
		defaultOptions = angular.extend(defaultOptions, config);
	};

	this.$get = ['$rootScope', '$q', '$window', '$timeout', function ($rootScope, $q, $window, $timeout) {
		return {
			open: function (url, target, requestOptions) {
				var q = $q.defer();

				if (requestOptions && !angular.isObject(requestOptions)) {
					q.reject('options must be an object');
					return q.promise;
				}

				var options = angular.extend({}, defaultOptions, requestOptions);

				var opt = [];
				angular.forEach(options, function (value, key) {
					opt.push(key + '=' + value);
				});
				var optionsString = opt.join();

				ref = $window.open(url, target, optionsString);

				ref.addEventListener('loadstart', function (event) {
					$timeout(function () {
						$rootScope.$broadcast('$cordovaInAppBrowser:loadstart', event);
					});
				}, false);

				ref.addEventListener('loadstop', function (event) {
					q.resolve(event);
					$timeout(function () {
						$rootScope.$broadcast('$cordovaInAppBrowser:loadstop', event);
					});
				}, false);

				ref.addEventListener('loaderror', function (event) {
					q.reject(event);
					$timeout(function () {
						$rootScope.$broadcast('$cordovaInAppBrowser:loaderror', event);
					});
				}, false);

				ref.addEventListener('exit', function (event) {
					$timeout(function () {
						$rootScope.$broadcast('$cordovaInAppBrowser:exit', event);
					});
				}, false);

				return q.promise;
			},

			close: function () {
				ref.close();
				ref = null;
			},

			show: function () {
				ref.show();
			},

			executeScript: function (details) {
				var q = $q.defer();

				ref.executeScript(details, function (result) {
					q.resolve(result);
				});

				return q.promise;
			},

			insertCSS: function (details) {
				var q = $q.defer();

				ref.insertCSS(details, function (result) {
					q.resolve(result);
				});

				return q.promise;
			}
		};
      }];
    }]);
