angular.module('bagu')
  .provider('$Cache', function() {
    var service = {
      tablekey: undefined,
      tableprefix: undefined,
      $get: function($rootScope, $q, EVENTS, _) {

        //.service('$Cache', function($q) {
        var LOCAL_CACHE_TABLES_KEY = service.tablekey || '_CacheTables';
        var LOCAL_CACHE_TABLE_PREFIX = service.tableprefix || '_Cache_';
        var data = {};

        //缓存刷新(清空)回调函数:每个表限制只保留一个回调函数
        var refreshCallBack = [];

        //检查表名是否已经写入表清单中
        function fixTableNames(table) {
          //注册表名
          var existstable = false;
          angular.forEach(data[LOCAL_CACHE_TABLES_KEY], function(name) {
            if (name === table) {
              existstable = true;
              return false;
            }
          });
          if (existstable !== true) {
            if (!data[LOCAL_CACHE_TABLES_KEY]) {
              data[LOCAL_CACHE_TABLES_KEY] = [];
            }
            data[LOCAL_CACHE_TABLES_KEY].push(table);
          }
        }

        //调用刷新钩子,table为空,则刷新所有钩子
        function callCacheRefreshCallBack(table) {
          _.forEach(refreshCallBack, function(item) {
            if (_.isEmpty(table) || item.table === table) {
              //增加缓存节点
              if (!data[LOCAL_CACHE_TABLE_PREFIX + table]) {
                data[LOCAL_CACHE_TABLE_PREFIX + table] = {};
              }
              item.fn(data[LOCAL_CACHE_TABLE_PREFIX + table]);
            }
          });
        }

        //注册缓存表刷新缓存钩子
        function registerTableCacheRefreshCallback(table, fn) {
          if (!_.isFunction(fn)) {
            return false;
          }
          //注册表名
          var existsfn = false;
          angular.forEach(refreshCallBack, function(item) {
            if (item.table === table && item.fn === fn) {
              existsfn = true;
              return false;
            }
          });
          if (existsfn !== true) {
            return refreshCallBack.push({
              table: table,
              fn: fn
            });
          }
        }

        //注册缓存清理事件钩子
        //(全局事件),任何一个表缓存清理,都会触发
        function registerCacheClearCallback(fn) {
          if (!_.isFunction(fn)) {
            return false;
          }
          return refreshCallBack.push({
            table: '',
            fn: fn
          });
        }

        //加载缓存
        function loadAll() {
          try {
            data[LOCAL_CACHE_TABLES_KEY] = angular.fromJson(window.localStorage.getItem(LOCAL_CACHE_TABLES_KEY));
          } catch (err) {
            data[LOCAL_CACHE_TABLES_KEY] = [];
          }
          // data[LOCAL_CACHE_TABLES_KEY] = angular.fromJson(window.localStorage.getItem(LOCAL_CACHE_TABLES_KEY));
          // if(!data[LOCAL_CACHE_TABLES_KEY]){
          //   data[LOCAL_CACHE_TABLES_KEY] = [];
          // }
          angular.forEach(data[LOCAL_CACHE_TABLES_KEY], function(table) {
            var itemdata = null;
            try {
              itemdata = angular.fromJson(window.localStorage.getItem(LOCAL_CACHE_TABLE_PREFIX + table));
            } catch (err) {
              itemdata = [];
            }
            //如果是数组,则将其转换为对象
            var tabledata = {};
            if (angular.isArray(itemdata)) {
              angular.forEach(itemdata, function(tablerecord, index) {
                var id = tablerecord._id || index;
                tabledata[id] = tablerecord;
              });
            } else {
              tabledata = itemdata;
            }
            data[LOCAL_CACHE_TABLE_PREFIX + table] = tabledata;
          });
        }
        //保存表致缓存
        function storeTable(table) {
          fixTableNames(table);
          if (data[LOCAL_CACHE_TABLES_KEY]) {
            window.localStorage.setItem(LOCAL_CACHE_TABLES_KEY, angular.toJson(data[LOCAL_CACHE_TABLES_KEY]));
          }
          if (data[LOCAL_CACHE_TABLE_PREFIX + table]) {
            window.localStorage.setItem(LOCAL_CACHE_TABLE_PREFIX + table, angular.toJson(data[LOCAL_CACHE_TABLE_PREFIX + table]));
          }
        }
        //保存所有缓存
        function storeAll() {
          angular.forEach(data, function(value, key) {
            fixTableNames(key);
            if (value) {
              window.localStorage.setItem(key, angular.toJson(value));
            }
          });
        }
        //清除表缓存
        function destroyTable(table) {
          window.localStorage.removeItem(LOCAL_CACHE_TABLE_PREFIX + table);
        }
        //清除所有缓存
        function destroyAll() {
          angular.forEach(data, function(value, key) {
            window.localStorage.removeItem(key);
          });
          window.localStorage.removeItem(LOCAL_CACHE_TABLES_KEY);
          data = {};
          data[LOCAL_CACHE_TABLES_KEY] = [];
        }
        //获取缓存表
        function getTabel(table, fn) {
          //增加缓存节点
          if (!data[LOCAL_CACHE_TABLE_PREFIX + table]) {
            data[LOCAL_CACHE_TABLE_PREFIX + table] = {};
          }
          //注册缓存刷新钩子
          if (_.isFunction(fn)) {
            registerTableCacheRefreshCallback(table, fn);
          }
          //返回
          return data[LOCAL_CACHE_TABLE_PREFIX + table];
        }

        //加载
        var load = function() {
          loadAll();
        };
        //增加单条记录到缓存
        var addRecord = function(table, id, record) {
          var cachetable = getTabel(table);
          if (!id && record._id) {
            id = record._id;
          }
          if (!id) {
            id = cachetable.length;
          }
          cachetable[id] = record;
        };
        //增加对象到缓存
        var add = function(table, records) {
          if (!angular.isArray(records)) {
            records = [records];
          }
          angular.forEach(records, function(record) {
            addRecord(table, record._id, record);
          });

        };
        //设置(替换)缓存
        var set = function(table, tabledata) {
          //add(table, tabledata);
          data[LOCAL_CACHE_TABLE_PREFIX + table] = tabledata;
          storeTable(table);
          //调用刷新钩子
          callCacheRefreshCallBack(table);
          return data[LOCAL_CACHE_TABLE_PREFIX + table];
        };
        //获取缓存表
        var get = function(table, fn) {
          return getTabel(table, fn);
        };
        //获取缓存记录
        var getRecord = function(table, id) {
          return getTabel(table)[id];
        };
        //保存
        var saveTable = function(table) {
          storeTable(table);
        };
        //保存
        var save = function() {
          storeAll();
        };
        //清除缓存
        var clear = function() {
          destroyAll();
          //调用刷新钩子
          // $rootScope.$broadcast(EVENTS.CACHE.CLEAR);
          //调用刷新钩子
          callCacheRefreshCallBack('');
        };
        //清空缓存表
        var clearTable = function(table) {
          destroyTable(table);
          data[LOCAL_CACHE_TABLE_PREFIX + table] = {};
          //调用刷新钩子
          callCacheRefreshCallBack(table);
          //返回
          return data[LOCAL_CACHE_TABLE_PREFIX + table];
        };

        load();

        return {
          load: load,
          add: add,
          addRecord: addRecord,
          set: set,
          get: get,
          getRecord: getRecord,
          save: save,
          saveTable: saveTable,
          clear: clear,
          clearTable: clearTable,
          registerCacheClearCallback: registerCacheClearCallback
        };
      }
    };

    return service;
  });
