angular.module('bagu')
  .provider('$Date', function() {
    return {
      $get: function() {
        //是否正确日期对象
        var isValidDate = function(d) {
          if (Object.prototype.toString.call(d) !== '[object Date]')
            return false;
          return !isNaN(d.getTime());
        };

        //获取当前日期所在月份的总天数
        var getMonthDays = function(d1) {
          return new Date(d1.getFullYear(), d1.getMonth() + 1, 0).getDate();
        };

        //当前日期是否为当月的最后一天
        var isLastMonthDay = function(d1) {
          return d1.getDate() === getMonthDays(d1);
        };

        //修正日期,去掉时间内容
        var fixShortDate = function(d1) {
          return new Date(d1.getFullYear(), d1.getMonth(), d1.getDate());
        };

        //修正日期大小顺序,第一个变量是小日期,第二个变量是大日期
        //返回:如果顺序正常,则返回1.否则返回-1,同时调换两个变量引用,变量名称不变.
        var fixDateIndex = function(d1, d2) {
          //规整日期
          fixShortDate(d1);
          fixShortDate(d2);
          if (d1 > d2) {
            var d3 = new Date(d1);
            d1.setFullYear(d2.getFullYear(), d2.getMonth(), d2.getDate());
            d2.setFullYear(d3.getFullYear(), d3.getMonth(), d3.getDate());
            return -1;
          } else {
            return 1;
          }
        };

        /**
         * 计算日期相差
         */
        var dayDiff = function(d1, d2) {
          var t2 = d2.getTime();
          var t1 = d1.getTime();

          return parseInt((t2 - t1) / (24 * 3600 * 1000));
        };

        var weekDiff = function(d1, d2) {
          var t2 = d2.getTime();
          var t1 = d1.getTime();

          return parseInt((t2 - t1) / (24 * 3600 * 1000 * 7));
        };

        var monthDiff = function(d1, d2) {
          var d1Y = d1.getFullYear();
          var d2Y = d2.getFullYear();
          var d1M = d1.getMonth();
          var d2M = d2.getMonth();
          var diff = (d2M + 12 * d2Y) - (d1M + 12 * d1Y);
          //开始月份天数大于结束月份天数?
          if (d2.getDate() + 1 < d1.getDate() && !isLastMonthDay(d2) && diff > 0) {
            diff = diff - 1;
          }
          return diff;
        };

        var yearDiff = function(d1, d2) {
          return d2.getFullYear() - d1.getFullYear();
        };

        /**
         * 日期增加函数
         */
        var addHours = function(d1, hours) {
          var t1 = d1.getTime();

          return new Date(t1 + hours * 3600 * 1000);
        };

        var addDays = function(d1, days) {
          var t1 = d1.getTime();
          d1.setTime(t1 + days * 24 * 3600 * 1000);
          return d1;
        };

        var addWeeks = function(d1, weeks) {
          var t1 = d1.getTime();
          d1.setTime(t1 + weeks * 24 * 3600 * 1000 * 7);
          return d1;
        };

        var addMonths = function(d1, months) {
          var currentMonth = d1.getMonth();
          d1.setMonth(d1.getMonth() + months);

          if (d1.getMonth() !== ((currentMonth + months) % 12)) {
            d1.setDate(0);
          }
          return d1;
        };

        var addYears = function(d1, years) {
          d1.setFullYear(d1.getFullYear() + years);
          return d1;
        };

        //月份与日期差
        //特殊考虑按自然月收租的情况,
        //如:2月22日收至3月31日,租金算为1月多7天
        //多余的7天为2月份的
        var monthdayDiff = function(d1, d2) {
          //避免修改原值
          var newd1 = new Date(d1);
          var newd2 = new Date(d2);
          //修正起始和截止日期顺序,起始日期始终小于结束月份
          var fixindex = fixDateIndex(newd1, newd2);
          var month = monthDiff(newd1, newd2);
          var day = dayDiff(addMonths(new Date(newd1), month), newd2);
          var maxdays = getMonthDays(newd2);
          var d1maxdays = getMonthDays(newd1);
          var d1day = newd1.getDate();
          var d2day = newd2.getDate();

          //整月
          if (d1day === 1 && isLastMonthDay(newd2)) {
            month = month + 1;
            day = -1;
          } else
          //判断月中到下个月月底的情况,如:2016-02-18 到 2016-3-31,
          //应该算第二个月为一个整月,剩余天数为前一个月的2月18日至2月29日的天数
          if (month >= 1 && day >= 0 && isLastMonthDay(newd2)) {
            day = dayDiff(new Date(newd1), new Date(newd1.getFullYear(), newd1.getMonth() + 1, 0));
            maxdays = getMonthDays(newd1);
          } else
          //哪个月剩余天数占比多,多余天数就按哪月算,如'20150210','20150302',2月份占比较多,剩余天数为28天
          if (d2day < 15 && day + 1 > d2day && d1maxdays - d1day > d2day) {
            maxdays = getMonthDays(newd1);
          }

          return {
            month: fixindex * month,
            day: fixindex * (day + 1),
            maxdays: maxdays
          };
        };
        /**
         * 根据指定日期,获取指定收租截止日期
         * 如果有设置收租日,则为收租日前一天(确保相隔日期大于15天),否则为开始日期增加一个月
         */
        var getRentEndDay = function(d1, rentday, mindiffday) {
          mindiffday = mindiffday || 15;
          if (isValidDate(rentday)) {
            rentday = rentday.getDate();
          }
          var rday = rentday || 0,
            bday = new Date(d1);
          var result = addMonths(bday, 1); //一个月后
          //如果设置了收租日,则将结束日期设为收租日
          if (rday !== 0) {
            result.setDate(rday);
          }
          //如果时间差小于等于15天,则增加一个月.
          var diffday = dayDiff(d1, result);
          if (diffday <= mindiffday) {
            result = addMonths(result, 1);
          }
          //提前一天
          result = addDays(result, -1);
          //返回
          return result;
        };

        //获取两个时间之间时长的文字描述,自动忽略日期顺序的问题
        //如:1个月整,1个月零20天,30天
        var getDateRangeString = function(d1, d2) {
          //避免修改原值
          var newd1 = new Date(d1);
          var newd2 = new Date(d2);
          fixDateIndex(newd1, newd2);
          var mddiff = monthdayDiff(newd1, newd2);
          var result = '';
          if (mddiff.month !== 0 && mddiff.day === 0) {
            result = mddiff.month.toString() + '个月整';
          } else if (mddiff.month !== 0 && mddiff.day !== 0) {
            result = mddiff.month.toString() + '个月零' + mddiff.day.toString() + '天';
          } else if (mddiff.month === 0 && mddiff.day !== 0) {
            result = mddiff.day.toString() + '天';
          } else {
            result = '0天';
          }
          return result;
        };


        //获取中国日期:如2016年3月8日
        var getChinaDateString = function(date) {
          if(!angular.isDate( date )){
            date = new Date(date);
          }
          return date.getFullYear() + '年' + (date.getMonth() + 1) + '月' + date.getDate() + '日';
        };

        //日期对象格式化
        var format = function(date,fmt) {
          var o = {
            'M+': date.getMonth() + 1, //月份
            'd+': date.getDate(), //日
            'h+': date.getHours(), //小时
            'm+': date.getMinutes(), //分
            's+': date.getSeconds(), //秒
            'q+': Math.floor((date.getMonth() + 3) / 3), //季度
            'S': date.getMilliseconds() //毫秒
          };
          if (/(y+)/.test(fmt))
            fmt = fmt.replace(RegExp.$1, (date.getFullYear() + '').substr(4 - RegExp.$1.length));
          for (var k in o)
            if (new RegExp('(' + k + ')').test(fmt))
              fmt = fmt.replace(RegExp.$1, (RegExp.$1.length === 1) ? (o[k]) : (('00' + o[k]).substr(('' + o[k]).length)));
          return fmt;
        };


        return {
          dayDiff: dayDiff,
          weekDiff: weekDiff,
          monthDiff: monthDiff,
          monthdayDiff: monthdayDiff,
          yearDiff: yearDiff,
          addDays: addDays,
          addWeeks: addWeeks,
          addMonths: addMonths,
          addYears: addYears,
          isLastMonthDay: isLastMonthDay,
          getMonthDays: getMonthDays,
          getRentEndDay: getRentEndDay,
          isValidDate: isValidDate,
          fixShortDate: fixShortDate,
          getDateRangeString: getDateRangeString,
          getChinaDateString: getChinaDateString,
          format: format
        };
      }
    };
  });
