angular.module('bagu')
  /********
  缓存月度统计(只保存最近月份),格式:{"buildingID":{money,nofee,...}...}
  ********/
  .service('$Analysis', function($q, $http, $Window, $Date, $Cache, $Invoice, DATALIST, _) {
    /****对象定义****/
    //表名
    var TABLENAME = 'analysis';
    //数据实体对象
    var DATA = null;
    //服务对象
    var service = {};

    /****工具函数****/
    //获取缓存数据
    function loadCacheData() {
      //缓存对象的引用
      DATA = $Cache.get(TABLENAME, function(cachetable) {
        DATA = $Cache.get(TABLENAME);
      });
      return DATA;
    }
    //保存数据到缓存
    function save2Cache() {
      //保存结果到缓存
      $Cache.saveTable(TABLENAME);
    }
    //保存单个楼房统计数据到缓存
    function saveAnalysis2Cache(building, year, month, analysis) {
      var buildingId = building._id ? building._id : building;
      //整理统计数据(按金额倒排序)
      analysis.paytypes = _.sortBy( analysis.paytypes, function(item) { return item.money * -1; });
      analysis.products = _.sortBy( analysis.products, function(item) { return item.money * -1; });
      //清除原有楼房的房间数据
      DATA[buildingId] = DATA[buildingId] || {};
      DATA[buildingId][year] = DATA[buildingId][year] || {};
      DATA[buildingId][year][month] = analysis;
      //写入缓存
      save2Cache();
    }

    //从缓存中获取楼房统计对象
    function getCacheAnalysis(building, year, month) {
      var buildingId = building._id ? building._id : building;
      return DATA[buildingId] && DATA[buildingId][year] && DATA[buildingId][year][month] ? DATA[buildingId][year][month] : null;
    }

    //根据租金单时间,动态计算当前的缴费月份
    function getBuildingCurrentAnalysisMonthly(building, fromServer) {
      var buildingId = building._id ? building._id : building;
      return $Invoice.getBuildingLastInvoice(buildingId, fromServer).then(function(invoices) {
        var rentdate = building && building.settings && building.settings.rentdate ? building.settings.rentdate : 1;
        var before = building && building.settings && building.settings.before ? building.settings.before : 2;
        var currentdate = $Date.addDays(new Date(), before);
        if (currentdate.getDate() < rentdate) {
          $Date.addMonths(currentdate, -1);
        }
        var year = currentdate.getFullYear();
        var month = currentdate.getMonth() + 1;
        //汇总统计
        if (invoices) {
          var monthcount = _.countBy(invoices, function(item) {
            var begindate = item.begindate ? item.begindate : currentdate;
            return begindate.getFullYear().toString() + (begindate.getMonth() + 1).toString();
          });
          var monthlystr = '',
            maxcount = 0;
          _.forEach(monthcount, function(value, key) {
            if (value > maxcount) {
              maxcount = value;
              monthlystr = key;
            }
          });
          if (monthlystr !== '') {
            year = parseInt(monthlystr.substring(0, 4));
            month = parseInt(monthlystr.substring(4));
          }
          //console.log(monthcount,year,month);
        }
        return {
          year: year,
          month: month
        };
      }, function(err) {
        return $q.reject(err);
      });
    }

    /****服务方法****/

    service.getBuildingCurrentAnalysisMonthly = getBuildingCurrentAnalysisMonthly;

    //获取指定楼房的统计数据
    //building:楼房(_id,或对象)
    //fromServer:是否强制从服务器刷新数据
    //返回值:
    // {
    //  "money":1000
    //  ....
    // }
    var getBuildingAnalysis = function(building, year, month, fromServer) {
      fromServer = fromServer || false;
      //返回promise
      return $q(function(resolve, reject) {
        if (!building) {
          return reject('楼房参数错误!');
        }
        var buildingId = building._id ? building._id : building;
        var cacheanalysis = getCacheAnalysis(buildingId, year, month);
        if (fromServer !== true && !_.isEmpty(cacheanalysis)) {
          //返回缓存数据
          return resolve(cacheanalysis);
        } else {
          $Window.showLoading();
          //无缓存数据或强制刷新,则从远程取
          $http.put('/monthly/' + buildingId + '/' + year.toString() + '/' + month.toString())
            .success(function(response) {
              //保存到缓存
              saveAnalysis2Cache(buildingId, year, month, response);
              //回调
              resolve(getCacheAnalysis(buildingId, year, month));
            })
            .error(function(err) {
              reject(err);
            })
            .finally(function() {
              $Window.hideLoading();
            });
        }
      });
    };
    service.getBuildingAnalysis = getBuildingAnalysis;

    //获取当前时间的楼房统计数据
    service.getBuildingCurrentAnalysis = function(building, fromServer) {
      // var rentdate = building && building.settings && building.settings.rentdate ? building.settings.rentdate : 1;
      // var before = building && building.settings && building.settings.before ? building.settings.before : 2;
      // var currentdate = $Date.addDays(new Date(),before);
      // if(currentdate.getDate() < rentdate){
      //   $Date.addMonths(currentdate,-1);
      // }
      // var year = currentdate.getFullYear();
      // var month = currentdate.getMonth() + 1;
      //从缓存中获取租金单计算当前月份
      return getBuildingCurrentAnalysisMonthly(building, false).then(function(monthly) {
        return getBuildingAnalysis(building, monthly.year, monthly.month, fromServer);
      }, function(err) {
        return $q.reject(err);
      });
    };

    //加载缓存
    if (!DATA) {
      loadCacheData();
    }

    return service;
  });
