angular.module('bagu')

/***
Audio service
语音服务,调用Media播放语音
***/
.service('$Audio', function ($q, $rootScope, $http, $ionicModal, $timeout, $ionicPlatform, _, $Mustache,
		SERVER_SETTINGS, AuthService, $cordovaMedia, LOCAL, $Date, $Local, DATALIST) {

    /***全局常量***/
    //普通话
    var LANGUAGE_CN = 'cn';
    //粤语
    var LANGUAGE_YY = 'yy';


    /***全局变量***/
    //语言定义
    //其中语音文件位置采用mustache模板系统计算
		/* 模板定义,会自动附上:
		    localFileSystem(本地目录系统,包含常用目录)
		    token(当前登录令牌)
		    server(server设置对象)
		    language(语言定义)
        audio(语音定义)
		  */
    var languageConfig = {
      //普通话
      cn: {
        name: '普通话',
        code: LANGUAGE_CN,
        //默认音频文件位置
        path: '/audio/{{{language.code}}}/{{{audio.code}}}.wav',
        //苹果音频文件位置
        iosPath: '/audio/{{{language.code}}}/{{{audio.code}}}.wav',
        //安卓音频文件位置
        androidPath: '/android_asset/www/audio/{{{language.code}}}/{{{audio.code}}}.wav',
        //windows音频文件位置
        winPath: '/audio/{{{language.code}}}/{{{audio.code}}}.wav'
      },
      //粤语
      yy: {
        name: '粤语',
        code: LANGUAGE_YY,
        //默认音频文件位置
        path: '/audio/{{{language.code}}}/{{{audio.code}}}.wav',
        //苹果音频文件位置
        iosPath: '/audio/{{{language.code}}}/{{{audio.code}}}.wav',
        //安卓音频文件位置
        androidPath: '/android_asset/www/audio/{{{language.code}}}/{{{audio.code}}}.wav',
        //windows音频文件位置
        winPath: '/audio/{{{language.code}}}/{{{audio.code}}}.wav'
      },
    };
    //音频文件定义:分类/语音
    var audioConfig = {
      //提示音,编码由10XX开头,如:1001
      beep: [
        {text:'di', code:1000}, //提示
        {text:'dong', code:1001}, //警告
        {text:'bong', code:1002}, //错误
        {text:'dingdong', code:1003}, //通知
      ],
      //数字,编码由11XX开头,如:1101
      number: [
        {text:['0','零'], code:1100},
        {text:['1','壹'], code:1101},
        {text:['2','贰'], code:1102},
        {text:['3','叁'], code:1103},
        {text:['4','肆'], code:1104},
        {text:['5','伍'], code:1105},
        {text:['6','陆'], code:1106},
        {text:['7','柒'], code:1107},
        {text:['8','捌'], code:1108},
        {text:['9','玖'], code:1109},
        {text:'10', code:1110},
        {text:'11', code:1111},
        {text:'12', code:1112},
      ],
      //常用字,编码由12XX开头,如:1201
      cnChar: [
        {text:'房', code:1200},
        {text:'水', code:1201},
        {text:'电', code:1202},
        {text:'表', code:1203},
        {text:'年', code:1204},
        {text:'月', code:1205},
        {text:'日', code:1206},
        {text:'号', code:1207},
        {text:'元', code:1208},
        {text:'费', code:1209},
        {text:' ', code:1210},
        {text:'不', code:1211},
        {text:'分', code:1212},
        {text:'角', code:1213},
        {text:'点', code:1214},
        {text:'拾', code:1215},
        {text:'佰', code:1216},
        {text:'仟', code:1217},
        {text:'万', code:1218},
        {text:'亿', code:1219},
        {text:'方', code:1220},
        {text:'度', code:1221},
      ],
      //常用词,编码由13XX开头,如:1301
      cnWord: [
        {text:'保存', code:1300},
        {text:'底数', code:1301},
        {text:'房间', code:1302},
        {text:'费用', code:1303},
        {text:'清除', code:1304},
        {text:'取消', code:1305},
        {text:'删除', code:1306},
        {text:'上一间', code:1307},
        {text:'下一间', code:1308},
        {text:'是否', code:1309},
        {text:'用量', code:1310},
        {text:'用量不合理', code:1311},
        {text:'用量偏多', code:1312},
        {text:'用量偏少', code:1313},
        {text:'上次用量', code:1314},
      ],
    };
    //多媒体对象
    var _audios = {};
    var _player = null;
		//本地设置
		var localSettings = $Local.get();

    /***工具函数***/
		//将字符串解析成字符数组'123' => ['1','2','3']
    function string2Array(str){
      var texts = [];
      for (var i=0; i < str.length; i++) {
        texts.push(str[i]);
      }
      return texts;
    }
		//是否设置不使用语音? CUR_LANGUAGE === ''
		function isUnUseAudio(){
			return localSettings && localSettings.AUDIO && localSettings.AUDIO.CUR_LANGUAGE === '';
		}
		//获取语言的本地设置值,如果未设置则默认普通话
		function getLanguage(){
			if(localSettings && localSettings.AUDIO){
				return localSettings.AUDIO.CUR_LANGUAGE || localSettings.AUDIO.DEFAULT_LANGUAGE;
			}else{
				return LANGUAGE_CN;
			}
		}

    /***自定义方法***/

    //根据文本,查找对应编码
    var getCodeByText = function(text){
      //console.log(text);
      var thisAudio = null;
      _.forEach(audioConfig, function(audioType){
        thisAudio = _.find(audioType, function(item){
          if(_.isArray(item.text)){
            return _.findIndex(item.text, function(subtext){
              return _.isString(subtext) && subtext.toLowerCase() === text.toLowerCase();
            }) >= 0;
          }else if(_.isString(item.text)){
            return item.text.toLowerCase() === text.toLowerCase();
          }else{
            return item.text && item.text.toString().toLowerCase() === text.toLowerCase();
          }
        });
        //如果找到,则返回
        if(thisAudio){
          return false;
        }
      });
      return thisAudio ? thisAudio.code : null;
    };

    //根据文本(数组),查找对应编码(数组)
    var getCodesByTexts = function(texts){
      var result = [];
      if(!_.isArray(texts)){
        if(!_.isString(texts)){
          console.log('getCodeByText:请传入字符串或字符串数组!');
          return result;
        }
        texts = [texts];
      }
      _.forEach(texts, function(text){
        result.push(getCodeByText(text));
      });
      return result;
    };

    //根据编码找到语音对象(数组)
    var getAudiosByCodes = function(codes, language){
      var audios = [];
      //没有设置语言?
      if(!language){
        language = getLanguage();
      }
			//语言参数正确?
			if(!_audios[language]){
        console.log('getAudiosByCodes:语言参数错误!',language);
        return null;
      }
      //校验编码
      if(!_.isArray(codes)){
        if(!_.isNum(codes)){
          console.log('getAudiosByCodes:参数错误,只接受数字编码!');
          return null;
        }else{
          codes = [codes];
        }
      }
      //获取audio
      _.forEach(codes, function(code){
        audios.push(_audios[language][code]);
      });

      return audios;
    };

    //加载声音文件
    var loadAudios = function() {
      var audioScope = {
				localFileSystem: window.cordova && window.cordova.file ? window.cordova.file : {},
				server: SERVER_SETTINGS,
				local: localSettings,
				token: AuthService.authToken()
      };
      _audios = {};
      //加载语言
      _.forEach(languageConfig, function(language){
        if(!language || !language.code || !language.path){
          return;
        }
        //$Mustache计算
        audioScope.language = language;

        var thisLanguage = _audios[language.code] = [];
        //加载语音定义
        _.forEach(audioConfig, function(audioType){
          _.forEach(audioType, function(audio){
            if(!audio.text || !audio.code){
              return;
            }
            //已经存在,不覆盖
            if(thisLanguage[audio.code]){
              console.log('语音文件重复,语言:'+language.name + ',语音:'+audio.text + '('+audio.code+')');
              return;
            }
            //加载语音文件
            //$Mustache计算
            audioScope.audio = audio;
            try {
              var filepath = $Mustache.render(language.path, audioScope);
              if ($ionicPlatform.is('Android') && language.androidPath) {
                filepath = $Mustache.render(language.androidPath, audioScope);
              }else if ($ionicPlatform.is('Ios') && language.iosPath) {
                filepath = $Mustache.render(language.iosPath, audioScope);
              }else if ($ionicPlatform.is('Window') && language.winPath) {
                filepath = $Mustache.render(language.winPath, audioScope);
              }
							var thisMedia = $cordovaMedia.newMedia(filepath);
              thisLanguage[audio.code] = {
                media: thisMedia,
                code: audio.code,
                text: audio.text,
                playing: false //播放状态
              };
            } catch (e) {
              console.log('加载语音文件出错:',e);
            }
          });
        });
      });
    };

    //根据编码播放声音文件,异步调用
    function AudioPlayer(codes){
      this.q = $q.defer();
      this.index = 0;
      this.total = 0;
      this.codes = codes;
      this.audios = null;
      this.playAudios = null;
      this.isStop = false;
    }
    AudioPlayer.prototype.waterPlay = function(playAudios) {
      var thisAudio = playAudios[0];
      var player = this;
      this.q.notify({loaded: player.index, total: player.total});
      //是否中断播放?
      if(player.isStop){
        //console.log('is stop!',thisAudio);
        player.q.reject('播放中断!');
        return;
      }
      //播放语音
      if (thisAudio && thisAudio.media) {
        //console.log(thisAudio.text);
        //如果在播放,需先停止播放
        if(thisAudio.playing){
          //console.log('stop play!',thisAudio.text);
          //thisAudio.media.stop();
          thisAudio.media.seekTo(0);
        }
        thisAudio.playing = true;
        thisAudio.media.play()
        .then(function(data){
          //console.log('ok',data,thisAudio.text);
        },function(err){
          //console.log('err',err,thisAudio.text);
        },function(notify){
          //console.log('notify',notify.toString(),thisAudio.text);
          if(notify.status === DATALIST.MEDIA_STATUS.MEDIA_STOPPED){
						thisAudio.playing = false;
						if(playAudios.length <= 1){
			        //结束播放
			        player.isStop = true;
			        player.q.resolve(this.codes);
						}else{
	            player.waterPlay(playAudios.slice(1));
	          }
					}
        })
        .finally(function (data) {
          //console.log('finally',data,thisAudio.text);
          // thisAudio.playing = false;
          // player.waterPlay(playAudios.slice(1));
        });
      }else if(playAudios.length > 1){
        //跳过错误语音文件
        player.index++;
        player.waterPlay(playAudios.slice(1));
      }else {
        //结束播放
        player.isStop = true;
        player.q.resolve(this.codes);
      }
    };
    AudioPlayer.prototype.play = function() {
      var player = this;
      this.q = $q.defer();
      //获取语音文件
      this.audios = getAudiosByCodes(this.codes);
      if(!this.audios){
        this.q.reject('无法获取语音文件');
      }else{
		    //附件中断方法到promise
		    this.q.promise.stop = function(){
		      player.stop();
		    };
		    //递归播放语音
		    this.playAudios = this.audios;
		    this.total = this.audios.length;
		    this.waterPlay(this.audios);
			}

      return this.q.promise;
    };
    AudioPlayer.prototype.stop = function() {
      this.isStop = true;
    };
    AudioPlayer.prototype.isPlaying = function() {
      return this.isStop !== true;
    };

    //播放声音文件
    var play = function(texts) {
			var q = $q.defer();
			//设置了不是用语音提示?
			if(isUnUseAudio()){
				q.reject('已设置不使用语音提示功能');
			}else{
	      //未加载语音设置?
	      if(_.isEmpty(_audios)){
	        loadAudios();
	      }
	      //获取编码
	      var codes = getCodesByTexts(texts);
	      //播放声音(返回Promise)
	      //return playByCodes(codes);
	      if(_player && _player.isPlaying()){
	        //console.log('stop player');
	        _player.stop();
	      }
	      _player = new AudioPlayer(codes);
	      _player.play().then(function(ok){
					q.resolve(ok);
				},function(err){
					q.reject(err);
				},function(notify){
					q.notify(notify);
				});
			}

			return q.promise;
    };

    //播放提示音
    var beep = function(){
      return play(['di']);
    };
    //播放警告
    var warning = function(){
      return play(['dong']);
    };
    //播放错误音
    var error = function(){
      return play(['bong']);
    };
    //播放消息提示音
    var infomation = function(){
      return play(['dingdong']);
    };
    //播放金额,区分:万,千,百,点,角,分
    var getChineseMoney = function (money) {
      money = money.toString();
      var strOutput = '';
      var strUnit = '仟佰拾亿仟佰拾万仟佰拾元角分';
      money += '00';
      var intPos = money.indexOf('.');
      if (intPos >= 0) {
        money = money.substring(0, intPos) + money.substr(intPos + 1, 2);
      }
      strUnit = strUnit.substr(strUnit.length - money.length);
      for (var i=0; i < money.length; i++) {
        strOutput += '零壹贰叁肆伍陆柒捌玖'.substr(money.substr(i,1),1) + strUnit.substr(i,1);
      }
      return strOutput.replace(/零角零分$/, '整').replace(/零[仟佰拾]/g, '零').replace(/零{2,}/g, '零').replace(/零([亿|万])/g, '$1').replace(/零+元/, '元').replace(/亿零{0,3}万/, '亿').replace(/^元/, '零元');
    };
    var genChineseMoney = function(money){
      var chineseMoney = getChineseMoney(money.toString());
      return string2Array(chineseMoney);
    };
    var playMoney = function(money){
      var chineseMoney = getChineseMoney(money.toString());
      var texts = string2Array(chineseMoney);
      return play(texts);
    };
    //播放十进制数字,区分:万,千,百,十
    var getChineseNumber = function (num) {
      num = num.toString();
      var strOutput = '';
      var strUnit = '仟佰拾亿仟佰拾万仟佰拾元';
      strUnit = strUnit.substr(strUnit.length - num.length);
      for (var i=0; i < num.length; i++) {
        strOutput += '零壹贰叁肆伍陆柒捌玖'.substr(num.substr(i,1),1) + strUnit.substr(i,1);
      }
      return strOutput.replace(/零[仟佰拾]/g, '零').replace(/零{2,}/g, '零').replace(/零([亿|万])/g, '$1').replace(/零+元/, '元').replace(/亿零{0,3}万/, '亿').replace(/^元/, '零元').replace(/元/, '');
    };
    var genChineseNumber = function(num){
      var roundNumber = Math.round(num);
      var chineseNumber = getChineseNumber(roundNumber.toString());
      return string2Array(chineseNumber);
    };
    var playChineseNumber = function(num){
      if(num > 0 && num < 10){
        return play(num.toString());
      }else{
        return play(genChineseNumber(num));
      }
    };
    //播放十进制数字,不区分,逐个数字播放
    var genNumber = function(num){
      return string2Array(num.toString());
    };
    var playNumber = function(num){
      if(num > 0 && num < 10){
        return play(num.toString());
      }else{
        return play(genNumber(num));
      }
    };
    //播放日期,区分:年月日
    var genChineseDate = function (date) {
      var texts = [];
      //年份
      texts = texts.concat(string2Array(date.getFullYear().toString()));
      texts.push('年');
      //月份
      texts.push((date.getMonth()+1).toString());
      texts.push('月');
      //月份
      var chineseNumber = getChineseNumber(date.getDate().toString());
      texts = texts.concat(string2Array(chineseNumber));
      texts.push('日');
      return texts;
    };
    var playDate = function(date){
      return play(genChineseDate(date));
    };

    //准备播放素材
    //传入一个或多个参数,接受类型:日期,数字,字符串,数组
    //返回对象,包含play()方法
    var preparePlay = function(){
      //未加载语音设置?
      if(_.isEmpty(_audios)){
        loadAudios();
      }

      //定义返回对象
      function Audio(texts){
        this.texts = texts;
      }
      Audio.prototype.play = function() {
        return play(this.texts);
      };

      //分析参数
      var texts = [],i;
      for (i =0 ; i < arguments.length; i++){ // 获取参数内容。
        var argum = arguments[i];
        //是字符串?
        if(_.isString(argum)){
          texts = texts.concat(string2Array(argum));
        }else //是数组?校验是否全部内容为字符串
        if(_.isArray(argum)){
          var hasNoStr = false;
          for (var j =0 ; j < argum.length; j++){
            if(!_.isString( argum[j] )){
              hasNoStr = true;
              break;
            }
          }
          if(hasNoStr){
            console.log('preparePlay:只接受全部项目为字符串的数组参数.该参数已忽略');
            continue;
          }
          texts = texts.concat(string2Array(argum));
        }else //日期类型?
        if(_.isDate(argum)){
          texts = texts.concat(genChineseDate(argum));
        }else //数字类型?
        if(_.isNumber(argum)){
          //如果包含小数点,则认为需要按照金额的方式播报
          var isMoney = argum.toString().indexOf('.') >= 0;
          if(isMoney){
            texts = texts.concat(genChineseMoney(argum));
          }else{
            texts = texts.concat(genChineseNumber(argum));
          }
        }else {
          console.log('preparePlay:发现不受支持的参数.该参数已忽略');
        }
      }

      //返回对象
      return new Audio(texts);
    };


    //返回服务对象
		return {
      loadAudios: loadAudios,
      play: play,
      beep: beep,
      warning: warning,
      error: error,
      infomation: infomation,
      genChineseMoney: genChineseMoney,
      genChineseNumber: genChineseNumber,
      genNumber: genNumber,
      genChineseDate: genChineseDate,
      playMoney: playMoney,
      playChineseNumber: playChineseNumber,
      playNumber: playNumber,
      playDate: playDate,
      preparePlay: preparePlay,
    };
	}

);
