angular.module('bagu')
/********
缓存楼房,格式:{"buildingID":building,....}
********/
.service('$Building', function($rootScope, $q, $http, $Cache, $Product, _, EVENTS) {
  /****对象定义****/
  //表名
  var TABLENAME = 'buildings';
  var TABLENAME_ROOM = 'rooms';
  //数据实体对象
  var DATA = null;
  //服务对象
  var service = {
  };
  //是否有新数据?
  var hasNewData = false;

  /****工具函数****/

  //获取缓存数据
  function loadCacheData(){
    //缓存对象的引用
    //DATA = $Cache.get(TABLENAME);
    DATA = $Cache.get(TABLENAME,function(cachetable){
      DATA = $Cache.get(TABLENAME);
    });
    //缓存清空回调
    // $Cache.registerCacheClearCallback(function(){
    //   DATA = $Cache.get(TABLENAME);
    // });
    return DATA;
  }
  //保存数据到缓存
  function save2Cache(){
    //保存结果到缓存
    $Cache.saveTable(TABLENAME);
  }

  //修复默认收费项目中的_id,收费类型,单位等信息
  function fixDefaultProducts(building){
    _.forEach(building.defaultproducts, function(product){
      product._id = product.product;
    });
    $Product.fixOrderProducts(building.defaultproducts,null,function(){
      //写入缓存
      save2Cache();
    });
    return building;
  }

  //保存楼房数据到缓存,将_id转换成属性名称,方便后面使用
  function saveBuildings2Cache(buildinglist){
    //清除原有楼房数据
    DATA = $Cache.clearTable(TABLENAME);
    //_id转换
    if(buildinglist && _.isArray(buildinglist)){
      _.forEach(buildinglist,function(building){
        if(building && building._id){
          DATA[building._id] = fixDefaultProducts(building);
        }
      });
    }
    //写入缓存
    save2Cache();
  }

  /****服务方法****/
  //从缓存获取指定楼房
  //building:楼房
  //返回值:
  //building: object
  service.getCacheBuilding = function(building){
    var buildingId = building._id ? building._id : building;
    return  DATA[buildingId];
  };
  //获取指定楼房列表
  //fromServer:是否强制从服务器刷新数据
  //返回值:
  //{
  //  "buildingID":building,
  //  ...
  //}
  service.getBuildings = function(fromServer){
    fromServer = fromServer || false;
    //返回promise
    return $q(function(resolve, reject) {
      if(fromServer !== true && !_.isEmpty(DATA)){
        //返回缓存数据
        return resolve(DATA);
      }else{
        //无缓存数据或强制刷新,则从远程取
        $http.get('/buildings')
        .success(function (response) {
          //保存到缓存
          saveBuildings2Cache(response);
          //回调
          resolve(DATA);
        })
        .error(function (err){
          reject(err);
        });
      }
    });
  };

  //获取指定楼房数据
  //buildingId:楼房对象_id
  //fromServer:是否强制从服务器刷新数据
  service.getBuilding = function(buildingId,fromServer){
    fromServer = fromServer || false;
    //返回promise
    return $q(function(resolve, reject) {
      if(!buildingId){
        return reject('楼房参数错误!');
      }
      //存在于缓存中?
      var cacheBuilding = DATA[buildingId];
      if(fromServer !== true && cacheBuilding){
        //返回
        return resolve(cacheBuilding);
      }else{
        //无缓存数据或强制刷新,则从远程取
        $http.get('/building/'+buildingId)
        .success(function (response) {
          //保存到缓存
          DATA[buildingId] = fixDefaultProducts(response);
          save2Cache();
          //回调
          resolve(DATA[buildingId]);
        })
        .error(function (err){
          reject(err);
        });
      }
    });
  };

  //修改指定楼房信息
  //buildingId:楼房对象_id
  //data:修改内容
  service.editBuilding = function(buildingId,buildingData){
    //返回promise
    return $q(function(resolve, reject) {
      if(!buildingId){
        return reject('楼房参数错误!');
      }
      var url = '/building/'+ buildingId ;
      $http.put(url, buildingData)
      .success(function(response, status, headers, config){
        //将修改内容保存到缓存
        fixDefaultProducts(_.assign(DATA[buildingId],response));
        save2Cache();
        //有可能修改总表设置,此处重新获取抄表记录
        
        //回调
        resolve(DATA[buildingId]);
      })
      .error(function(response, status, headers, config){
        reject(response, status, headers, config);
      });
    });
  };

  //删除指定楼房信息
  //buildingId:楼房对象_id
  service.deleteBuilding = function(buildingId,buildingData){
    //返回promise
    return $q(function(resolve, reject) {
      if(!buildingId){
        return reject('楼房参数错误!');
      }
      var url = '/building/'+ buildingId ;
      $http.delete(url)
      .success(function(response, status, headers, config){
        //删除缓存
        delete DATA[buildingId];
        save2Cache();
        //回调
        resolve(response, status, headers, config);
      })
      .error(function(response, status, headers, config){
        reject(response, status, headers, config);
      });
    });
  };

  //新增楼房信息
  //buildingData:新房间内容
  service.addBuilding = function(buildingData){
    //返回promise
    return $q(function(resolve, reject) {
      $http.post('/init/building',buildingData)
      .success(function (response, status, headers, config) {
        var newbuilding = response.building ? response.building : response;
        var rooms = response.rooms ? response.rooms : null;
        var buildingId = newbuilding._id;
        //将修改内容保存到缓存
        DATA[buildingId] = fixDefaultProducts(newbuilding);
        //保存房间列表
        var roomsdb = $Cache.get(TABLENAME_ROOM);
        roomsdb[buildingId] = {};
        if(rooms && _.isArray(rooms)){
          _.forEach(rooms,function(room){
            if(room && room._id){
              roomsdb[buildingId][room._id] = room;
            }
          });
        }
        //写入缓存
        save2Cache();
        //回调
        resolve({
          building: newbuilding,
          rooms: rooms
        });
      }).error(function (response, status, headers, config){
        reject(response, status, headers, config);
      });
    });
  };


  //加载缓存
  if(!DATA){
    loadCacheData();
  }

  return service;
});
