angular.module('bagu')

//Http返回状态拦截器
.factory('responseInterceptor', function($rootScope, $q, EVENTS) {
    return {
      responseError: function(response) {
        if(response.status === 401){
          $rootScope.$broadcast(EVENTS.AUTH.NOTAUTHENTICATED, response);
        } else
        //如果是后台静默连接,则不触发事件(401错误除外)
        if (!response.config.headers || response.config.headers.isbackground !== true) {
          //无法连接网络？
          if (response.status === 0) {
            response.data = {
              message: '无法连接网络！'
            };
          }

          $rootScope.$broadcast({
            0: EVENTS.NET.NORESPONSE,
            //401: EVENTS.AUTH.NOTAUTHENTICATED,
            403: EVENTS.AUTH.NOTAUTHORIZED
          }[response.status], response);
        }

        return $q.reject(response);
      }
    };
  })
  //Url拦截器
  .factory('requestInterceptor', function($rootScope, $q, $Local, SERVER_SETTINGS) {
    return {
      // 拦截请求
      'request': function(config) {
        // 判断Url是否为简写(已'/'开头)
        if (config.url && config.url.substring(0, 1) === '/') {
          var localData = $Local.get();
          //如果缓存被清空,则使用系统设置值
          var serverUrl = localData.CUR_SERVER ? localData.CUR_SERVER : SERVER_SETTINGS.CUR_SERVER;
          config.url = serverUrl + config.url;
        }
        //设置请求超时时间
        config.timeout = 30000;
        //console.log(config.url);
        return config;
      }
    };
  })
  //注册拦截器
  .config(function($httpProvider) {
    $httpProvider.interceptors.push('responseInterceptor');
    $httpProvider.interceptors.push('requestInterceptor');
    $httpProvider.defaults.headers.common.Client = ionic.Platform.platform() + ' ' + ionic.Platform.version();
  })
  //验证服务
  .service('AuthService', function($rootScope, $q, $http, $ionicHistory, $Window, $Cache, LOCAL, EVENTS) {
    var LOCAL_TOKEN_KEY = LOCAL.TOKEN_KEY;
    var user = {};
    var isAuthenticated = false;
    var roles = '';
    var authToken;

    function loadCacheUser() {
      var tokenstr = window.localStorage.getItem(LOCAL_TOKEN_KEY);
      if (tokenstr && tokenstr !== '') {
        return angular.fromJson(tokenstr);
      } else {
        return null;
      }
    }

    function useCredentials(token) {
      isAuthenticated = true;
      authToken = token;

      // Set the token as header for your requests!
      $http.defaults.headers.common.Authorization = 'Bearer ' + token;
    }

    function loadUserCredentials() {
      var cacheuser = loadCacheUser();
      if (cacheuser && cacheuser.requestToken) {
        user = cacheuser;
        useCredentials(user.requestToken);
      }
    }

    function storeUserCredentials(user) {
      //是否不同用户,如果是,则需要清除所有缓存表
      var cacheuser = loadCacheUser();
      if (!cacheuser || cacheuser._id !== user._id) {
        $Cache.clear();
        $rootScope.$broadcast(EVENTS.CACHE.CLEAR);
      }
      window.localStorage.setItem(LOCAL_TOKEN_KEY, angular.toJson(user));
      useCredentials(user.requestToken);
    }

    function destroyUserCredentials() {
      authToken = undefined;
      user = {};
      isAuthenticated = false;
      $http.defaults.headers.common.Authorization = undefined;
      //window.localStorage.removeItem(LOCAL_TOKEN_KEY);
    }

    function clearHistory() {
      //清除页面浏览缓存和历史记录
      $ionicHistory.clearCache();
      $ionicHistory.clearHistory();
    }

    var verifycode = function(mobile, pw) {
      return $q(function(resolve, reject) {
        var url = '/verifycode';
        $http.post(url, {
            mobile: mobile
          })
          .success(function(response) {
            resolve(response);
          }).error(function(err) {
            reject(err);
          });
      });
    };


    var register = function(mobile, verifycode, password, confirm) {
      return $q(function(resolve, reject) {
        $Window.showLoading();
        $http.post('/register', {
            verifycode: verifycode,
            mobile: mobile,
            password: password,
            confirm: confirm
          })
          .success(function(response) {
            user = response;
            authToken = response.requestToken;
            storeUserCredentials(user);
            //清除页面浏览缓存和历史记录
            clearHistory();
            resolve(user);
          }).error(function(err) {
            reject(err);
          })
          .finally(function() {
            $Window.hideLoading();
          });
      });
    };

    var login = function(mobile, pw, isadmin) {
      var loginurl = isadmin && isadmin === true ? '/login/admin' : '/login/app';
      return $q(function(resolve, reject) {
        $http.post(loginurl, {
            mobile: mobile,
            password: pw
          })
          .success(function(response) {
            user = response;
            authToken = response.requestToken;
            storeUserCredentials(user);
            resolve(user);
          }).error(function(err) {
            reject(err);
          });
      });
    };

    var logout = function() {
      destroyUserCredentials();
      //清除页面浏览缓存和历史记录
      clearHistory();
    };

    var isAuthorized = function(authorizedRoles) {
      if (!angular.isArray(authorizedRoles)) {
        authorizedRoles = [authorizedRoles];
      }
      return (isAuthenticated && authorizedRoles.indexOf(roles) !== -1);
    };

    loadUserCredentials();

    return {
      verifycode: verifycode,
      register: register,
      login: login,
      logout: logout,
      isAuthorized: isAuthorized,
      isAuthenticated: function() {
        return isAuthenticated;
      },
      user: function() {
        return user;
      },
      authToken: function(){
        return authToken;
      },
      roles: function() {
        return roles;
      }
    };
  })
  //注册lodash
  .factory('_', function($window) {
    var _ = $window._;
    delete $window._;

    //在此处增加lodash自定义方法
    //example:
    //_.customerfunction = function(param){
    //   do something;
    // }

    return (_);
  })
  //注册Mustache
  .factory('$Mustache', function($window) {
    var Mustache = $window.Mustache;
    delete $window.Mustache;

    //在此处增加自定义方法
    //example:
    //Mustache.customerfunction = function(param){
    //   do something;
    // }

    return (Mustache);
  });
