angular.module('bagu')
/********
缓存常住人员信息
********/
.service('$Customer', function($q, $http, $Cache, _) {
  /****对象定义****/
  //表名
  var TABLENAME = 'customer';
  //数据实体对象
  var DATA = null;
  //服务对象
  var service = {
  };

  /****工具函数****/

  //获取缓存数据
  function loadCacheData(){
    //缓存对象的引用
    DATA = $Cache.get(TABLENAME,function(cachetable){
      DATA = $Cache.get(TABLENAME);
    });
    return DATA;
  }
  //保存数据到缓存
  function save2Cache(){
    //保存结果到缓存
    $Cache.saveTable(TABLENAME);
  }
  //保存数据到缓存,将_id转换成属性名称,方便后面使用
  function saveCustomers2Cache(customerlist){
    //_id转换
    if(customerlist && _.isArray(customerlist)){
      $Cache.add(TABLENAME,customerlist);
      //写入缓存
      save2Cache();
    }
  }

  /****服务方法****/
  //获取指定人员的常住人员
  //room:房间对象
  //fromServer:是否强制从服务器刷新数据
  service.getRoomCustomers = function(room,fromServer){
    fromServer = fromServer || false;
    //返回promise
    return $q(function(resolve, reject) {
      if(!room || !room._id || !room.rent || !room.rent.contract || !room.rent.contract.customers){
        return reject('参数错误!');
      }
      //新的对象数组
      var customers = [];
      //有缓存?
      var isInCache = 0;
      if(!fromServer){
        _.forEach(room.rent.contract.customers,function(customer){
          var customerId = customer._id || customer;
          if(DATA[customerId]){
            customers.push(DATA[customerId]);
            isInCache++;
          }
        });
      }
      //未指定从服务器获取,并且所有租客信息都有缓存
      if(fromServer !== true && isInCache > 0 && isInCache === room.rent.contract.customers.length){
        room.rent.contract.customers = customers;
        //返回room对象
        return resolve(room.rent.contract.customers);
      }else{
        var roomId = room._id;
        //无缓存数据或强制刷新,则从远程取
        $http.get('/customers/'+roomId)
        .success(function (response) {
          //更新room对象
          customers = [];
          _.forEach(room.rent.contract.customers,function(customer){
            var customerId = customer._id || customer;
            customers.push(_.find(response, { '_id': customerId }));
          });
          room.rent.contract.customers = customers;
          //保存到缓存
          saveCustomers2Cache(response);
          //回调
          resolve(room.rent.contract.customers);
        })
        .error(function (err){
          reject(err);
        });
      }
    });
  };

  //获取指定人员数据
  //customerId:对象_id
  //fromServer:是否强制从服务器刷新数据
  service.getCustomer = function(customerId,fromServer){
    fromServer = fromServer || false;
    //返回promise
    return $q(function(resolve, reject) {
      if(!customerId){
        return reject('参数错误!');
      }
      //存在于缓存中?
      var cacheCustomer =  DATA[customerId];
      if(fromServer !== true && cacheCustomer){
        //返回内存副本
        return resolve(_.cloneDeep(cacheCustomer));
      }else{
        //无缓存数据或强制刷新,则从远程取
        $http.get('/customer/'+customerId)
        .success(function (response) {
          cacheCustomer = response;
          //保存到缓存
          DATA[customerId] = cacheCustomer;
          save2Cache();
          //回调,返回内存副本
          resolve(_.cloneDeep(cacheCustomer));
        })
        .error(function (err){
          reject(err);
        });
      }
    });
  };

  //新增指定人员信息,并绑定房间
  //room:对象或_id
  //data:新增对象
  service.newCustomer = function(room,customerData){
    //返回promise
    return $q(function(resolve, reject) {
      if(!room){
        return reject('参数错误!');
      }
      //租客默认用户名和密码为手机号码
      if(!customerData.username || customerData.username.length === 0){
        customerData.username = customerData.mobile;
        customerData.password = customerData.mobile;
      }

      var roomId = room._id || room;
      var url = '/room/'+ roomId +'/customer';
      $http.post(url, customerData)
      .success(function(response, status, headers, config){
        var newCustomer = response;
        //将修改内容保存到缓存
        DATA[newCustomer._id] = newCustomer;
        save2Cache();
        //修改房间对象
        // if(room && room.rent && room.rent.contract && _.isArray(room.rent.contract.customers)){
        //   room.rent.contract.customers.push(newCustomer);
        // }
        //回调
        resolve(newCustomer, status, headers, config);
      })
      .error(function(response, status, headers, config){
        reject(response, status, headers, config);
      });
    });
  };

  //修改指定人员信息
  //customerId:对象_id
  //data:修改内容
  service.editCustomer = function(customer,customerData){
    //返回promise
    return $q(function(resolve, reject) {
      var customerId = customer._id ? customer._id :customer;
      if(!customerId){
        return reject('参数错误!');
      }
      var url = '/customer/'+ customerId ;
      $http.put(url, customerData)
      .success(function(response, status, headers, config){
        //将修改内容保存到缓存
        DATA[customerId] = angular.extend(DATA[customerId], customerData);
        save2Cache();
        //回调
        resolve(response, status, headers, config);
      })
      .error(function(response, status, headers, config){
        reject(response, status, headers, config);
      });
    });
  };

  //删除指定人员信息
  //customerId:对象_id
  //data:修改内容
  service.deleteCustomer = function(customer,room){
    //返回promise
    return $q(function(resolve, reject) {
      var customerId = customer._id ? customer._id :customer;
        var roomId = room._id ? room._id :room;
      if(!customerId){
        return reject('参数错误!');
      }
      var url = '/room/' + roomId + '/customer/'+ customerId ;
      $http.delete(url)
      .success(function(response, status, headers, config){
        //将修改内容保存到缓存
        delete DATA[customerId];
        save2Cache();
        //回调
        resolve(response, status, headers, config);
      })
      .error(function(response, status, headers, config){
        reject(response, status, headers, config);
      });
    });
  };

  //加载缓存
  if(!DATA){
    loadCacheData();
  }

  return service;
});
