angular.module('bagu')
  /********
  1,缓存中只存储未缴费租金单,缴费历史实时从远程获取
  ********/
  .service('$Invoice', function($q, $http, $Window, $Date, $Cache, $Room, _, DATALIST) {
    /****对象定义****/
    //表名
    var TABLENAME_INVOICE = 'lastinvoices';
    var TABLENAME_ROOM = 'rooms';
    //数据实体对象
    var DATA = null;
    //服务对象
    var service = {};

    /****工具函数****/
    //附加计算字段至租金单对象
    function fixInvoice(invoice) {
      invoice.begindate = invoice.begindate ? new Date(invoice.begindate) : undefined;
      invoice.enddate = invoice.enddate ? new Date(invoice.enddate) : undefined;
      invoice.guessPayDate = invoice.guessPayDate ? new Date(invoice.guessPayDate) : undefined;
      //如果猜测交费日期小于今天,则显示为预计今天交费
      // if(invoice.guessPayDate < new Date()){
      //   invoice.guessPayDate =new Date();
      // }

      if (invoice.invoicestatus < DATALIST.INVOICE_STATUS.FINANCE) {
        invoice.statusstr = '未收费';
      } else {
        invoice.statusstr = DATALIST.getEnumStr(DATALIST.INVOICE_STATUS
          .enum, invoice.invoicestatus);
      }
      invoice.typestr = DATALIST.getEnumStr(DATALIST.INVOICE_TYPE
        .enum, invoice.invoicetype);
      //退房时,显示退费
      if (invoice.invoicetype === DATALIST.INVOICE_TYPE.BACK) {
        invoice.statusstr = invoice.statusstr.replace('收费', '退费');
      }
      if (invoice.invoicestatus === DATALIST.INVOICE_STATUS.FINANCE && invoice.financelog) {
        invoice.paytype = DATALIST.getEnumStr(DATALIST.FINANCE_PAYTYPE.enum, invoice.financelog.paytype);
      } else {
        invoice.paytype = '';
      }
      //增加电话号码
      invoice.mobile = invoice.customers && invoice.customers.length > 0 ? invoice.customers[
        0].mobile : '';
      _.forEach(invoice.items, function(item) {
        //修复明细时间
        item.begindate = item.begindate ? new Date(item.begindate) : undefined;
        item.enddate = item.enddate ? new Date(item.enddate) : undefined;
        //修复明细项目信息
        item.info = '';
        switch (item.itemtype) {
          case DATALIST.INVOICE_ITEM_TYPE.NORMAL:
          case DATALIST.INVOICE_ITEM_TYPE.DEDUCTION:
            switch (item.feetype) {
              case DATALIST.PRODUCT_FEETYPE.ONCE:
                item.info = '一次性收费';
                break;
              case DATALIST.PRODUCT_FEETYPE.MONTHLY:
                item.info = item.price + '元/月';
                break;
              case DATALIST.PRODUCT_FEETYPE.AMOUNT:
              case DATALIST.PRODUCT_FEETYPE.STEP:
                item.info = item.price + '元/' + item.unit;
                if(item.isminvalue){
                  item.info += '(保底)';
                }
                break;
              case DATALIST.PRODUCT_FEETYPE.AREA:
                item.info = item.price + '元/㎡';
                break;
              case DATALIST.PRODUCT_FEETYPE.PERSON:
                item.info = item.price + '元/人';
                break;
              default:
            }
            if(item.money < 0){
              item.info += '退费';
            }
            break;
          case DATALIST.INVOICE_ITEM_TYPE.ADDITION:
            item.info = item.money < 0 ? '一次性退费' : '一次性收费';
            break;
          case DATALIST.INVOICE_ITEM_TYPE.PLEDGE:
            item.info = (item.money < 0 ? '退' : '') + '押金';
            break;
          default:

        }

      });

      return invoice;
    }
    //获取缓存数据
    function loadCacheData() {
      DATA = $Cache.get(TABLENAME_INVOICE, function(cachetable) {
        DATA = $Cache.get(TABLENAME_INVOICE);
      });
      //修正数据
      _.forEach(DATA, function(building) {
        _.forEach(building, function(roominvoice) {
          fixInvoice(roominvoice);
        });
      });
      return DATA;
    }
    //保存数据到缓存
    function save2Cache() {
      //保存结果到缓存
      $Cache.saveTable(TABLENAME_INVOICE);
    }
    //保存楼房的租金单数据到缓存,将_id转换成属性名称,方便后面使用
    function saveBuildingLastInvoices2Cache(buildingId, buildingInvoices) {
      //延迟加载缓存
      if (!DATA) {
        loadCacheData();
      }
      if (buildingInvoices && _.isArray(buildingInvoices)) {
        DATA[buildingId] = {};
        //循环保存每一个房间的租金单
        _.forEach(buildingInvoices, function(invoice) {
          DATA[buildingId][invoice.room] = fixInvoice(invoice);
        });
        //写入缓存
        save2Cache();
      }
    }
    //保存单个租金单数据到缓存,将_id转换成属性名称,方便后面使用
    function saveRoomLastInvoice2Cache(invoice) {
      if (invoice) {
        //延迟加载缓存
        if (!DATA) {
          loadCacheData();
        }
        if (_.isArray(invoice) && invoice.length > 0) {
          invoice = invoice[0];
        }
        var buildingId = invoice.building && invoice.building._id ? invoice.building._id : invoice.building;
        var roomId = invoice.room && invoice.room._id ? invoice.room._id : invoice.room;
        DATA[buildingId] = DATA[buildingId] || {};
        DATA[buildingId][roomId] = DATA[buildingId][roomId] || {};
        //保存房间的租金单
        _.assign(DATA[buildingId][roomId], fixInvoice(invoice));
        //如果新租金单不包含缴费记录,则为新增租金单.需删除缓存中的缴费记录
        if(!invoice.financelog && DATA[buildingId][roomId].financelog){
          DATA[buildingId][roomId].financelog = undefined;
        }
        //写入缓存
        save2Cache();
      }
    }
    /****服务方法****/
    //获取指定楼房的租金单列表
    //building:楼房
    //option: 过滤条件{all:true/false,nofee:true/false,noprint:true/false}
    //fromServer:是否强制从服务器刷新数据
    //返回值:
    // {
    //  roomid:[invoice:{items:[]}]
    // }
    var getBuildingLastInvoice = function(building, fromServer) {
      //延迟加载缓存
      if (!DATA) {
        loadCacheData();
      }
      //返回promise
      return $q(function(resolve, reject) {
        if (!building) {
          return reject('楼房参数错误!');
        }
        var buildingId = building._id ? building._id : building;
        DATA[buildingId] = DATA[buildingId] || {}; //获取楼房下房间数量,如果抄表记录数与房间数不相等,则从服务器获取数据
        var roomsdb = $Cache.get(TABLENAME_ROOM);

        var rooms = 0;
        _.forEach(roomsdb[buildingId], function(room) {
          if(!_.contains( [DATALIST.ROOM_STATUS.OWN,DATALIST.ROOM_STATUS.EMPTY,DATALIST.ROOM_STATUS.NULL], room.roomstatus)){
            rooms ++;
          }
        });
        var invoices = DATA[buildingId] ? Object.getOwnPropertyNames(DATA[buildingId]).length : 0;
        //查找房间数与租金单数不一致的问题
        // var roomcodes = _.map( roomsdb[buildingId], function(room){
        //   if(!_.contains( [DATALIST.ROOM_STATUS.OWN,DATALIST.ROOM_STATUS.EMPTY,DATALIST.ROOM_STATUS.NULL], room.roomstatus)){
        //     return room.code;
        //   }else{
        //     return null;
        //   }
        //   if(!room.code){
        //     console.log(room);
        //   }
        // });
        // var invoicecodes = _.map( DATA[buildingId], 'code');
        // _.forEach(roomcodes, function(roomcode){
        //   if(roomcode && !_.contains( invoicecodes, roomcode)){
        //     console.log(roomcode);
        //   }
        // });
        // if (fromServer !== true && !_.isEmpty(DATA[buildingId]) && rooms === invoices) {
        if (fromServer !== true && rooms === invoices) {
          //返回缓存数据
          return resolve(DATA[buildingId]);
        } else {
          $Window.showLoading();
          //无缓存数据或强制刷新,则从远程取
          $http.get('/invoices/building/' + buildingId)
            .success(function(response) {
              //保存到缓存
              saveBuildingLastInvoices2Cache(buildingId, response);
              //回调
              resolve(DATA[buildingId]);
            })
            .error(function(err) {
              reject(err);
            })
            .finally(function() {
              $Window.hideLoading();
            });
        }
      });
    };

    //获取指定租金单
    //building:楼房
    //room:房间
    //fromServer:是否强制从服务器刷新数据
    //返回值:
    // {
    //  roomid:[invoice:{items:[]}]
    // }
    var getRoomInvoice = function(building, room, invoice, fromServer) {
      //延迟加载缓存
      if (!DATA) {
        loadCacheData();
      }
      //返回promise
      return $q(function(resolve, reject) {
        if (!building) {
          return reject('楼房参数错误!');
        }
        var buildingId = building._id ? building._id : building;
        var roomId = room._id ? room._id : room;
        var invoiceId = invoice._id ? invoice._id : invoice;
        DATA[buildingId] = DATA[buildingId] || {};

        if (fromServer !== true && !_.isEmpty(DATA[buildingId][roomId]) && DATA[buildingId][roomId]._id === invoiceId) {
          //返回缓存数据
          return resolve(DATA[buildingId][roomId]);
        } else {
          $Window.showLoading();
          //无缓存数据或强制刷新,则从远程取
          $http.get('/invoice/' + invoiceId)
            .success(function(response) {
              //回调
              resolve(fixInvoice(response));
            })
            .error(function(err) {
              reject(err);
            })
            .finally(function() {
              $Window.hideLoading();
            });
        }
      });
    };

    //获取指定房间的最后租金单
    //building:楼房
    //room:房间
    //fromServer:是否强制从服务器刷新数据
    //返回值:
    // {
    //  roomid:[invoice:{items:[]}]
    // }
    var getRoomLastInvoice = function(building, room, fromServer) {
      //延迟加载缓存
      if (!DATA) {
        loadCacheData();
      }
      //返回promise
      return $q(function(resolve, reject) {
        if (!building) {
          return reject('楼房参数错误!');
        }
        var buildingId = building._id ? building._id : building;
        var roomId = room._id ? room._id : room;
        DATA[buildingId] = DATA[buildingId] || {};

        if (fromServer !== true && !_.isEmpty(DATA[buildingId][roomId])) {
          //返回缓存数据
          return resolve(DATA[buildingId][roomId]);
        } else {
          $Window.showLoading();
          //无缓存数据或强制刷新,则从远程取
          $http.get('/invoices/roomlast/' + roomId)
            .success(function(response) {
              //保存到缓存
              saveRoomLastInvoice2Cache(response);
              //回调
              resolve(DATA[buildingId][roomId]);
            })
            .error(function(err) {
              reject(err);
            })
            .finally(function() {
              $Window.hideLoading();
            });
        }
      });
    };

    //获取指定楼房的月度租金单列表
    //building:楼房
    //year:年度
    //month:月份
    var getBuildingMonthlyInvoices = function(building,year,month) {
      //返回promise
      return $q(function(resolve, reject) {
        var buildingId = building._id ? building._id : building;
        $Window.showLoading(
          // {
          // template: '正在加载...'
          // }
        );
        //无缓存数据或强制刷新,则从远程取
        $http.get('/invoices/building/' + buildingId + '/' + year.toString() + '/' + month.toString())
          .success(function(response) {
            _.forEach(response, function(invoice) {
              fixInvoice(invoice);
            });
            //为每个租金单生成明细项目简述(用于pos打印机打印租金单详情)
  					_.forEach(response, function(invoice){
              //详细收费项目
  						var itemstr = '';
  						_.forEach(invoice.items, function(item){
  							itemstr += item.name.replace('金','').replace('费','');
  							if(item.itemtype === DATALIST.INVOICE_ITEM_TYPE.PLEDGE){
  								itemstr += '(押)';
  							}
  							itemstr += item.money + ',';
  						});
  						if(itemstr.length > 0){
  							itemstr = itemstr.slice(0,itemstr.length-1);
  						}
  						invoice.itemstr = itemstr;
              //租金单类型:只显示签合同和退房
              if(invoice.invoicetype === DATALIST.INVOICE_TYPE.MONTHLY){
                invoice.typestr = '';
              }
  					});
            //回调
            resolve(_.sortBy(response, function(item){
              var floorstr = '000'+item.floor;
              return floorstr.slice(-3)+item.code;
            }));
          })
          .error(function(err) {
            reject(err);
          })
          .finally(function() {
            $Window.hideLoading();
          });
      });
    };

    //获取指定房间租金单列表
    //room:房间
    var getRoomInvoices = function(room) {
      //返回promise
      return $q(function(resolve, reject) {
        var roomId = room._id ? room._id : room;
        $Window.showLoading(
          // {
          // template: '正在加载...'
          // }
        );
        //无缓存数据或强制刷新,则从远程取
        $http.get('/invoices/room/' + roomId)
          .success(function(response) {
            _.forEach(response, function(invoice) {
              fixInvoice(invoice);
            });
            //回调
            resolve(response);
          })
          .error(function(err) {
            reject(err);
          })
          .finally(function() {
            $Window.hideLoading();
          });
      });
    };

    //过滤租金单(已打印,未打印,已收费,未收费)
    /*filter:[status,status,...]
     */
    var filterBuildingLastInvoices = function(building, filter, fromServer, page) {
      if (!_.isArray(filter)) {
        filter = [filter];
      }
      //默认从缓存取数据
      fromServer = fromServer || false;
      //返回promise
      return $q(function(resolve, reject) {
        //无缓存数据或强制刷新,则从远程取
        getBuildingLastInvoice(building, fromServer).then(function(buildingInvoices) {
          var result = [];
          _.forEach(buildingInvoices, function(invoice) {
            //全部
            if (_.isUndefined(filter) || _.isEmpty(filter)) {
              this.push(invoice);
            } else if (invoice && !_.isUndefined(invoice.invoicestatus) && !_.isUndefined(filter)) {
              if (_.contains(filter, invoice.invoicestatus)) {
                this.push(invoice);
              }
            }
          }, result);
          result = _.sortBy(result, function(item) {
            return item.code;
          });
          if (page && page > 0) {
            result = result.slice((page - 1) * 5, page * 5);
            // _.initial(result, function(item,index) {
            //   return index > page*5 || index < (page-1)*5;
            // });
            resolve(result);
          } else {
            resolve(result);
          }
        }, function(err) {
          reject(err);
        });
      });
    };

    //楼房最后租金单分类统计(已打印,未打印,已收费,未收费)
    /*filters:{
      filtername1: [status,status,...],
      filtername2: [status,status,...],
      ...
    }
    */
    var getBuildingLastInvoicesCount = function(building, filters, fromServer) {
      //默认从缓存取数据
      fromServer = fromServer || false;
      //返回promise
      return $q(function(resolve, reject) {
        //无缓存数据或强制刷新,则从远程取
        getBuildingLastInvoice(building, fromServer).then(function(buildingInvoices) {
          var result = {};
          //设置返回对象
          _.forEach(filters, function(filter, key) {
            result[key] = {
              count: 0,
              money: 0
            };
          });
          //遍历统计
          _.forEach(buildingInvoices, function(invoice) {
            _.forEach(filters, function(filter, key) {
              if (!_.isArray(filter)) {
                filter = [filter];
              }
              if (filter.length === 0 || _.contains(filter, invoice.invoicestatus)) {
                //console.log(invoice);
                result[key].count++;
                result[key].money += invoice.money;
              }
            });
          });
          resolve(result);
        }, function(err) {
          reject(err);
        });
      });
    };

    //获取未缴费租金单
    service.getBuildingLastInvoice = getBuildingLastInvoice;

    //过滤租金单
    service.filterBuildingLastInvoices = filterBuildingLastInvoices;

    //分类统计租金单
    service.getBuildingLastInvoicesCount = getBuildingLastInvoicesCount;

    //获取指定楼房的月度租金单列表
    service.getBuildingMonthlyInvoices = getBuildingMonthlyInvoices;
    //获取房间指定租金单
    service.getRoomInvoice = getRoomInvoice;

    //设置房间最后一个租金单
    service.saveRoomLastInvoice2Cache = saveRoomLastInvoice2Cache;

    //获取房间最后一个租金单
    service.getRoomLastInvoice = getRoomLastInvoice;

    //获取房间租金单历史
    service.getRoomInvoices = getRoomInvoices;

    //修改租金单内容
    //invoice: 租金单对象
    service.updateInvoice = function(invoice) {
      return $q(function(resolve, reject) {
        if (!invoice || !invoice._id) {
          return reject('参数错误!');
        }
        var invoiceId = invoice._id;
        var invoiceData = {
          begindate: invoice.begindate,
          enddate: invoice.enddate,
          detailmoney: invoice.detailmoney,
          coupon: invoice.coupon,
          money: invoice.money,
          items: [],
          description: invoice.description
        };
        _.forEach(invoice.items, function(item) {
          invoiceData.items.push({
            _id: item._id,
            name: item.name,
            begindate: item.begindate,
            enddate: item.enddate,
            price: item.price,
            money: item.money,
            value: item.value,
            feetype: item.feetype,
            itemtype: item.itemtype
          });
        });
        //远程提交新数据
        $http.put('/invoice/' + invoiceId, invoiceData)
          .success(function(response) {
            var invoiceId = response._id;
            var buildingId = response.building._id || response.building;
            var roomId = response.room;
            //无需缓存楼房设置信息
            if (response.building && response.building._id) {
              response.building = response.building._id;
            }
            //延迟加载缓存
            if (!DATA) {
              loadCacheData();
            }
            //修改缓存租金单对象
            DATA[buildingId] = DATA[buildingId] || {};
            DATA[buildingId][roomId] = DATA[buildingId][roomId] || {};
            _.assign(DATA[buildingId][roomId], response);
            fixInvoice(DATA[buildingId][roomId]);
            //刷新最后超数据
            save2Cache();
            //回调
            resolve(DATA[buildingId][roomId]);
          })
          .error(function(err) {
            reject(err);
          });
      });
    };

    //修改租金单为已收费
    //invoice: 租金单对象或_id
    //feedata: 收费数据,可包含租金单的修改和收费方式等
    service.updateInvoiceHasFee = function(invoice, feedata) {
      return $q(function(resolve, reject) {
        if (!invoice) {
          return reject('参数错误!');
        }
        var invoiceId = invoice._id ? invoice._id : invoice;
        //无缓存数据或强制刷新,则从远程取
        $http.post('/invoice/' + invoiceId + '/fee', feedata)
          .success(function(response) {
            /******更新租金单状态*******/
            var invoice = response.invoice;
            var invoiceId = invoice._id;
            var buildingId = invoice.building;
            var roomId = invoice.room;
            //延迟加载缓存
            if (!DATA) {
              loadCacheData();
            }
            //修改缓存租金单对象
            DATA[buildingId] = DATA[buildingId] || {};
            DATA[buildingId][roomId] = DATA[buildingId][roomId] || {};
            _.assign(DATA[buildingId][roomId], {
              invoicestatus: invoice.invoicestatus,
              financelog: invoice.financelog
            });
            fixInvoice(DATA[buildingId][roomId]);
            //刷新按量计费项目的最后抄表记录(表底数)
            // $RoomValue.updateRoomValue(buildingId, roomId).then(function (roomvalues) {
            // });
            //保存至缓存
            save2Cache();
            /******更新房间状态*******/
            var roomstatus = response.roomstatus;
            var roomsdb = $Cache.get(TABLENAME_ROOM);
            if (roomsdb[buildingId] && roomsdb[buildingId][roomId]) {
              var cacheRoom = roomsdb[buildingId][roomId];
              cacheRoom.roomstatus = roomstatus;
              $Room.fixRoom(cacheRoom);
              //保存结果到缓存
              $Cache.saveTable(TABLENAME_ROOM);
            }
            //回调
            resolve(invoice);
          })
          .error(function(err) {
            reject(err);
          });
      });
    };

    //作废租金单
    //invoice: 租金单对象或_id
    service.deleteInvoice = function(invoice) {
      return $q(function(resolve, reject) {
        if (!invoice) {
          return reject('参数错误!');
        }
        var invoiceId = invoice._id ? invoice._id : invoice;
        //远程调用
        $http.delete('/invoice/' + invoiceId)
          .success(function(response) {
            var delinvoice = response.invoice;
            var invoiceId = delinvoice._id;
            var buildingId = delinvoice.building;
            var roomId = delinvoice.room;
            /******更新房间状态*******/
            var roomstatus = response.roomstatus;
            var roomsdb = $Cache.get(TABLENAME_ROOM);
            if (roomsdb[buildingId] && roomsdb[buildingId][roomId]) {
              var cacheRoom = roomsdb[buildingId][roomId];
              cacheRoom.roomstatus = roomstatus;
              $Room.fixRoom(cacheRoom);
              //保存结果到缓存
              $Cache.saveTable(TABLENAME_ROOM);
            }
            //更新房间最后缴费记录
            getRoomLastInvoice(buildingId, roomId, true).then(function(lastinvoice) {
              //延迟加载缓存
              if (!DATA) {
                loadCacheData();
              }
              //修改缓存租金单对象
              DATA[buildingId] = DATA[buildingId] || {};
              DATA[buildingId][roomId] = DATA[buildingId][roomId] || {};
              _.assign(DATA[buildingId][roomId], lastinvoice);
              fixInvoice(DATA[buildingId][roomId]);
              //刷新按量计费项目的最后抄表记录(表底数)
              // $RoomValue.updateRoomValue(buildingId, roomId).then(function (roomvalues) {
              // });
              //保存至缓存
              save2Cache();
              //回调(已删除的租金单)
              resolve(delinvoice);
            }, function(err) {
              reject(err);
            });
          })
          .error(function(err) {
            reject(err);
          });
      });
    };

    //彻底删除租金单
    //invoice: 租金单对象或_id
    service.removeInvoice = function(invoice) {
      return $q(function(resolve, reject) {
        if (!invoice) {
          return reject('参数错误!');
        }
        var invoiceId = invoice._id ? invoice._id : invoice;
        var buildingId = invoice.building,
          roomId = invoice.room;
        //远程调用
        $http.post('/invoice/' + invoiceId + '/remove')
          .success(function(response) {
            //回调(已删除的租金单)
            resolve(invoice);
          })
          .error(function(err) {
            reject(err);
          });
      });
    };

    //是否签合同单据
    service.isRentInvoice = function(invoice) {
      return invoice && _.isNumber(invoice.invoicetype) && invoice.invoicetype === DATALIST.INVOICE_TYPE
        .RENT;
    };
    //是否月度租金单
    service.isMonthlyInvoice = function(invoice) {
      return invoice && _.isNumber(invoice.invoicetype) && invoice.invoicetype === DATALIST.INVOICE_TYPE
        .MONTHLY;
    };
    //是否一次性收费
    service.isOnceInvoice = function(invoice) {
      return invoice && _.isNumber(invoice.invoicetype) && invoice.invoicetype === DATALIST.INVOICE_TYPE
        .ONCE;
    };
    //是否退房单据
    service.isBackInvoice = function(invoice) {
      return invoice && _.isNumber(invoice.invoicetype) && invoice.invoicetype === DATALIST.INVOICE_TYPE
        .BACK;
    };
    //是否其他单据
    service.isOtherInvoice = function(invoice) {
      return invoice && _.isNumber(invoice.invoicetype) && invoice.invoicetype === DATALIST.INVOICE_TYPE
        .OTHER;
    };

    //是否一次习惯计费项目
    service.isOnceFeeItem = function(item) {
      return item && _.isNumber(item.feetype) && item.feetype === DATALIST.PRODUCT_FEETYPE
        .ONCE;
    };
    //是否按月计费项目
    service.isMonthlyFeeItem = function(item) {
      return item && item.feetype && item.feetype === DATALIST.PRODUCT_FEETYPE
        .MONTHLY;
    };
    //是否按量计费项目
    service.isAmountFeeItem = function(item) {
      return item && item.feetype && (item.feetype === DATALIST.PRODUCT_FEETYPE
        .AMOUNT || item.feetype === DATALIST.PRODUCT_FEETYPE.STEP);
    };
    //是否按面积计费项目
    service.isAreaFeeItem = function(item) {
      return item && item.feetype && item.feetype === DATALIST.PRODUCT_FEETYPE
        .AREA;
    };
    //是否按人数计费项目
    service.isPersonFeeItem = function(item) {
      return item && item.feetype && item.feetype === DATALIST.PRODUCT_FEETYPE
        .PERSON;
    };

    //是否押金项目
    service.isPledgeInvoiceItem = function(item) {
      return item && item.itemtype && item.itemtype === DATALIST.INVOICE_ITEM_TYPE.PLEDGE;
    };

    //是否未收费单据?
    service.isUnFinanceInvoice = function(invoice) {
      return invoice && invoice.invoicestatus < DATALIST.INVOICE_STATUS.FINANCE;
    };

    //是否已收费单据?
    service.isFinanceInvoice = function(invoice) {
      return invoice && invoice.invoicestatus === DATALIST.INVOICE_STATUS.FINANCE;
    };

    //是否作废单据?
    service.isDeleteInvoice = function(invoice) {
      return invoice && invoice.invoicestatus === DATALIST.INVOICE_STATUS.DELETE;
    };

    //实时加载数据
    loadCacheData();

    return service;
  });
