angular.module('bagu')

/***百度地图定位服务***/
.service('MapPosition', [
  '$rootScope',
  '$http',
  '$ionicModal',
  '$timeout',
  '$ionicPlatform',
  'IONIC_BACK_PRIORITY',
  function($rootScope, $http, $ionicModal, $timeout, $ionicPlatform, _, IONIC_BACK_PRIORITY) {

    /***创建模态弹窗**
      options: 初始设置
        save: 定位成功并保存的回调函数(position)
        close: 取消定位的回调函数
    */
    function getMapPosition(options){
      var scope = $rootScope.$new(true);
      var mapModalView = null;
      //设置位置数据回调函数
      function setLocationData(address,lng,lat){
        if(!scope.$$phase) {
          scope.$apply(function () {
            scope.address = address;
            scope.longitude = lng;
            scope.latitude =  lat;
          });
        } else {
          scope.address = address;
          scope.longitude = lng;
          scope.latitude =  lat;
        }
      }

      angular.extend(scope, {
        address: null,
        longitude: null,
        latitude: null,
        close: undefined,
        save: undefined,
        templateUrl: null
      }, options || {});

      //校验设置
      if(!scope.templateUrl || !scope.save){
        return null;
      }
      //初始化地图参数
      scope.mapOptions = {
        center: {
          longitude: scope.longitude,
          latitude: scope.latitude
        },
        zoom: 17,
        city: 'ShenZhen',
        markers: [{
          longitude: scope.longitude,
          latitude: scope.latitude,
          title: '定位',
          content: '点击并拖动红色定位标示',
          enableMessage: false,
          enableDragging: true,
          raiseOnDrag: true,
          onDragend: function(e, rc){
            console.log('onDragend',rc.address);
            setLocationData(rc.address,e.point.lng,e.point.lat);
          }
        }]
      };

      //将GPS坐标转换为百度坐标
      function convertToBaiduMapPoint(point){
        setLocationData('getCurrentPosition ok !'+ point.coords.latitude.toString() + ' ' + point.coords.longitude.toString(),0,0);
        //http://api.map.baidu.com/geoconv/v1/?ak=E4805d16520de693a3fe707cdc962045&output=json
        $http.get('http://api.map.baidu.com/geoconv/v1/',{
          params:{ //get方法传参数,必须带上params
            coords:point.coords.longitude+','+point.coords.latitude,
            ak:'MPebIjRfRPqmj07AFOVUI3fS',
            output:'json'
          }
        })
        .success(function(response) {
          if(response.status === 0 && response.result && response.result.length > 0){
            var tranpoint = response.result[0];
            scope.mapOptions.center.longitude = tranpoint.x;
            scope.mapOptions.center.latitude = tranpoint.y;
            scope.mapOptions.markers[0].longitude = tranpoint.x;
            scope.mapOptions.markers[0].latitude = tranpoint.y;
            setLocationData('gps ok!'+tranpoint.x+tranpoint.y,tranpoint.x,tranpoint.y);
          }
        })
        .error(function(data, status, headers, config){
          setLocationData('gps error!',0,0);
        });
      }

      //获取当前位置
      navigator.geolocation.getCurrentPosition(convertToBaiduMapPoint,function(error){
        console.log(error);
        setLocationData(error.message,0,0);
      }); //定位

      //关闭地图
      scope.closeMap = function(){
        if(_.isFunction(scope.close)) scope.close();
        scope.mapModalView.remove();
      };

      //保存当前位置
      scope.saveMap = function(){
        if(_.isFunction(scope.save)) scope.save({
          address :scope.address,
          lng: scope.longitude,
          lat: scope.latitude
        });
        scope.mapModalView.remove();
      };

      scope.$on('$destroy', function() {
        scope.mapModalView.remove();
      });

      //创建模态窗口
      $ionicModal.fromTemplateUrl(scope.templateUrl, {
        scope: scope
      }).then(function(modal) {
        scope.mapModalView = modal;
        scope.mapModalView.show();
      });
    }
    return {
      show: getMapPosition
    };
  }

]);
