angular.module('bagu')
  /********
  缓存房间,格式:{"buildingID":{room:[pricelog],room,...}}
  ********/
  .service('$PriceLog', function($q, $http, $Cache, _) {
    /****对象定义****/
    //表名
    var TABLENAME = 'pricechangelog';
    //数据实体对象
    var DATA = null;
    //服务对象
    var service = {};

    /****工具函数****/

    //获取缓存数据
    function loadCacheData() {
      //缓存对象的引用
      DATA = $Cache.get(TABLENAME, function(cachetable) {
        DATA = $Cache.get(TABLENAME);
      });
      return DATA;
    }
    //保存数据到缓存
    function save2Cache() {
      //保存结果到缓存
      $Cache.saveTable(TABLENAME);
    }
    //保存单个楼房数据到缓存,将_id转换成属性名称,方便后面使用
    function saveBuildingPriceLogs2Cache(priceloglist) {
      var pls_g_b = _.groupBy(priceloglist, function(pricelog) {
        return pricelog.building;
      });
      _.forEach(pls_g_b, function(b_logs, buildingId) {
        //清除原有楼房的房间数据
        DATA[buildingId] = {};
        var pls_g_r = _.groupBy(b_logs, function(pricelog) {
          return pricelog.room;
        });
        _.forEach(pls_g_r, function(r_logs, roomId) {
          //清除原有房间数据
          DATA[buildingId][roomId] = _.reduce(r_logs, function(result, pricelog) {
            result.push({
              _id: pricelog._id,
              begindate: pricelog.begindate,
              price: pricelog.price
            });
            return result;
          }, []);
        });
      });
      //写入缓存
      save2Cache();
    }
    //保存单个房间数据到缓存
    function saveRoomPriceLogs2Cache(priceloglist) {
      var pls_g_b = _.groupBy(priceloglist, function(pricelog) {
        return pricelog.building;
      });
      _.forEach(pls_g_b, function(b_logs, buildingId) {
        //找到楼房数据
        DATA[buildingId] = DATA[buildingId] || {};
        var pls_g_r = _.groupBy(b_logs, function(pricelog) {
          return pricelog.room;
        });
        _.forEach(pls_g_r, function(r_logs, roomId) {
          //清除原有房间数据
          DATA[buildingId][roomId] = _.reduce(r_logs, function(result, pricelog) {
            result.push({
              _id: pricelog._id,
              begindate: pricelog.begindate,
              price: pricelog.price
            });
            return result;
          }, []);
        });
      });
      //写入缓存
      save2Cache();
    }

    /****服务方法****/
    //获取指定楼房的房间列表
    //building:楼房(_id,或对象)
    //fromServer:是否强制从服务器刷新数据
    //返回值:
    // {
    //  "roomId":room
    //  ....
    // }
    service.getBuildingPriceLogs = function(building, fromServer) {
      fromServer = fromServer || false;
      //返回promise
      return $q(function(resolve, reject) {
        if (!building) {
          return reject('楼房参数错误!');
        }
        var buildingId = building._id ? building._id : building;

        DATA[buildingId] = DATA[buildingId] || {};

        if (fromServer !== true && !_.isEmpty(DATA[buildingId])) {
          //返回缓存数据
          return resolve(DATA[buildingId]);
        } else {
          //无缓存数据或强制刷新,则从远程取
          $http.get('/building/' + buildingId + '/pricechangelog')
            .success(function(response) {
              //保存到缓存
              saveBuildingPriceLogs2Cache(response);
              //回调
              resolve(DATA[buildingId]);
            })
            .error(function(err) {
              reject(err);
            });
        }
      });
    };

    //获取指定房间数据
    //roomId:房间对象_id
    //fromServer:是否强制从服务器刷新数据
    service.getRoomPriceLog = function(roomId, fromServer) {
      fromServer = fromServer || false;
      //返回promise
      return $q(function(resolve, reject) {
        if (!roomId) {
          return reject('房间参数错误!');
        }
        //存在于缓存中?(遍历每个building)
        var cachePriceLog = null;
        _.forEach(DATA, function(building) {
          if (building[roomId]) {
            cachePriceLog = building[roomId];
            return false;
          }
        });
        //从缓存获取数据
        if (fromServer !== true && !_.isEmpty(cachePriceLog)) {
          //返回内存副本
          return resolve(cachePriceLog);
        } else {
          //无缓存数据或强制刷新,则从远程取
          $http.get('/room/' + roomId + '/pricechangelog')
            .success(function(response) {
              var cachePriceLogs = response;

              //保存到缓存
              saveRoomPriceLogs2Cache(cachePriceLogs);
              // _.forEach(cachePriceLogs, function(pricelog){
              //   DATA[pricelog.building] = DATA[pricelog.building] || {};
              //   DATA[pricelog.building][pricelog.room] = DATA[pricelog.building][pricelog.room] || [];
              //   DATA[pricelog.building][pricelog.room].push(pricelog);
              // });
              save2Cache();
              //回调
              resolve(response);
            })
            .error(function(err) {
              reject(err);
            });
        }
      });
    };

    //增加调租记录
    service.addPriceLog = function(room, pricedata) {
      //返回promise
      return $q(function(resolve, reject) {
        if (!room) {
          return reject('参数错误!');
        }
        var roomId = room._id ? room._id : room;
        var url = '/room/' + roomId + '/changeprice';
        $http.post(url, pricedata)
          .success(function(response, status, headers, config) {
            var pricelog = response;
            //保存调租记录
            DATA[pricelog.building] = DATA[pricelog.building] || {};
            DATA[pricelog.building][pricelog.room] = DATA[pricelog.building][pricelog.room] || [];
            var cachelog = _.find(DATA[pricelog.building][pricelog.room], function(item) {
              return item.begindate.toString() === pricelog.begindate.toString();
            });
            if (cachelog) {
              cachelog.price = pricelog.price;
            } else {
              DATA[pricelog.building][pricelog.room].push(pricelog);
            }
            save2Cache();
            //回调
            resolve(response, status, headers, config);
          })
          .error(function(response, status, headers, config) {
            reject(response, status, headers, config);
          });
      });
    };

    //删除调租记录
    service.deletePriceLog = function(pricelog) {
      //返回promise
      return $q(function(resolve, reject) {
        if (!pricelog) {
          return reject('参数错误!');
        }
        var pricelogId = pricelog._id ? pricelog._id : pricelog;
        var url = '/pricechangelog/' + pricelogId;
        $http.delete(url)
          .success(function(response, status, headers, config) {
            //修改缓存
            var pricelog = response;
            DATA[pricelog.building] = DATA[pricelog.building] || {};
            DATA[pricelog.building][pricelog.room] = DATA[pricelog.building][pricelog.room] || [];
            var index = _.findIndex(DATA[pricelog.building][pricelog.room], function(item) {
              return item._id.toString() === pricelog._id.toString();
            });
            if (index >= 0) {
              DATA[pricelog.building][pricelog.room].splice(index, 1);
            }
            save2Cache();
            //回调
            resolve(response, status, headers, config);
          })
          .error(function(response, status, headers, config) {
            reject(response, status, headers, config);
          });
      });
    };

    //加载缓存
    if (!DATA) {
      loadCacheData();
    }

    return service;
  });
