angular.module('bagu')
  /********
  缓存产品信息
  ********/
  .service('$Product', function ($q, $http, $Cache, _, DATALIST, $Date) {
    /****对象定义****/
    //表名
    var TABLENAME = 'productclasses';
    //数据实体对象
    var DATA = null;
    //服务对象
    var service = {};

    /****工具函数****/
    //获取缓存数据
    function loadCacheData() {
      //缓存对象的引用
        DATA = $Cache.get(TABLENAME,function(cachetable){
          DATA = $Cache.get(TABLENAME);
        });
      return DATA;
    }
    //保存数据到缓存
    function save2Cache() {
      //保存结果到缓存
      $Cache.saveTable(TABLENAME);
    }
    //保存数据到缓存,将_id转换成属性名称,方便后面使用
    function saveProducts2Cache(productclasses) {
      if (productclasses && _.isArray(productclasses)) {
        DATA = $Cache.set(TABLENAME, productclasses);
        //写入缓存
        save2Cache();
      }
    }
    //从缓存中获取指定_id的收费类型
    function getCacheType(typeId) {
      var thistype = null;
      _.forEach(DATA, function(_class){
        _.forEach(_class.types, function(type){
          if(type._id === typeId){
            thistype = type;
            return false;
          }
        });
        if(thistype) {
          return false;
        }
      });
      return thistype;
    }
    //从缓存中获取指定_id的收费项目
    function getCacheProduct(productId) {
      var thisproduct = null;
      _.forEach(DATA, function(_class){
        _.forEach(_class.types, function(type){
          _.forEach(type.products, function(product){
            if(product._id === productId){
              thisproduct = product;
              return false;
            }
          });
          if(thisproduct) {
            return false;
          }
        });
        if(thisproduct) {
          return false;
        }
      });
      return thisproduct;
    }

    /****服务方法****/
    //获取产品数据
    //fromServer:是否强制从服务器刷新数据
    service.getProducts = function (fromServer) {
      fromServer = fromServer || false;
      //返回promise
      return $q(function (resolve, reject) {
        //有缓存?
        if (fromServer !== true && !_.isEmpty(DATA)) {
          //返回
          return resolve(DATA);
        } else {
          //无缓存数据或强制刷新,则从远程取
          $http.get('/products/product/listwithclass')
            .success(function (response) {
              //保存到缓存
              saveProducts2Cache(response);
              //回调
              resolve(response);
            })
            .error(function (err) {
              reject(err);
            });
        }
      });
    };

    //获取按量计费的产品列表
    //fromServer:是否强制从服务器刷新数据
    service.getValueProducts = function (fromServer) {
      fromServer = fromServer || false;
      //返回promise
      return $q(function (resolve, reject) {
        service.getProducts(fromServer).then(function(productclasss){
          //整理按量计费项目
          var valueproducts = [];
          _.forEach(productclasss, function (classes) {
            _.forEach(classes.types, function (producttype) {
              _.forEach(producttype.products, function (product) {
                if (product && (product.feetype === DATALIST.PRODUCT_FEETYPE.AMOUNT || product.feetype === DATALIST.PRODUCT_FEETYPE.STEP)) {
                  valueproducts.push(product);
                }
              });
            });
          });
          return resolve(valueproducts);
        },function(err){
          return reject(err);
        });
      });
    };

    //获取初始化默认费用列表
    //fromServer:是否强制从服务器刷新数据
    service.getDefaultProducts = function (fromServer) {
      fromServer = fromServer || false;
      //返回promise
      return $q(function (resolve, reject) {
        service.getProducts(fromServer).then(function(productclasss){
          //整理按量计费项目
          var initproducts = [];
          _.forEach(productclasss, function (classes) {
            _.forEach(classes.types, function (producttype) {
              _.forEach(producttype.products, function (product) {
                if (product && product.defaultusage && product.defaultusage.init === true) {
                 initproducts.push(product);
                }
              });
            });
          });
          return resolve (initproducts);
        },function(err){
          return reject(err);
        });
      });
    };

    //获取缓存产品数据
    //productId:对象_id
    service.getCacheProduct = getCacheProduct;

    //获取产品数据
    //productId:对象_id
    //fromServer:是否强制从服务器刷新数据
    service.getProduct = function (productId, fromServer) {
      fromServer = fromServer || false;
      //返回promise
      return $q(function (resolve, reject) {
        if (!productId) {
          return reject('参数错误!');
        }
        //存在于缓存中?
        var cacheProduct = null;
        _.forEach(DATA, function (classes) {
          _.forEach(classes.types, function (producttype) {
            _.forEach(producttype.products, function (product) {
              if (product && product._id === productId) {
                cacheProduct = product;
                return false;
              }
            });
            if (cacheProduct) return false;
          });
          if (cacheProduct) return false;
        });
        if (fromServer !== true && cacheProduct) {
          //返回
          return resolve(cacheProduct);
        } else {
          //无缓存数据或强制刷新,则从远程取
          $http.get('/products/product/' + productId)
            .success(function (response) {
              cacheProduct = response;
              //保存到缓存
              if (cacheProduct.type && cacheProduct.type.productclass &&
                DATA[cacheProduct.type.productclass] && DATA[
                  cacheProduct.type.productclass][cacheProduct.type]
              ) {
                DATA[cacheProduct.type.productclass][cacheProduct.type]
                  [cacheProduct._id] = cacheProduct;
              }
              save2Cache();
              //回调,返回内存副本
              resolve(cacheProduct);
            })
            .error(function (err) {
              reject(err);
            });
        }
      });
    };

    //订购产品
    //room:对象或_id
    //productData:新增对象
    service.addRoomProduct = function (room, productData) {
      //返回promise
      return $q(function (resolve, reject) {
        if (!room || !productData) {
          return reject('参数错误!');
        }
        if (!room.rent) {
          return reject('不能为未出租的房间订购产品!');
        }
        var roomId = room._id;
        var url = '/room/' + roomId + '/product';
        $http.post(url, productData)
          .success(function (response, status, headers, config) {
            var newProduct = response;
            service.fixOrderProducts([newProduct]);
            //修改缓存
            if (DATA[room.building] && DATA[room.building][room._id] &&
              DATA[room.building][room._id].rent) {
              var cacheRoom = DATA[room.building][room._id];
              cacheRoom.rent.contract.products.push(newProduct);
            }
            save2Cache();
            //回调
            resolve(response, status, headers, config);
          })
          .error(function (response, status, headers, config) {
            reject(response, status, headers, config);
          });
      });
    };

    //取消订购产品
    //room:对象或
    //productrecord:对象或_id
    service.cancelRoomProduct = function (room, productData) {
      //返回promise
      return $q(function (resolve, reject) {
        if(!productData.record){
          productData.record = productData.productrecord;
        }
        if (!room || !room._id || !productData || !productData.record) {
          return reject('参数错误!');
        }
        if (!room.rent) {
          return reject('未出租的房间无订购产品!');
        }
        var roomId = room._id,
          productrecordId = productData.record._id ? productData.record._id : productData.record;
        var url = '/products/record/' + productrecordId + '/cancel';
        $http.post(url, {backmoney: productData.backmoney})
          .success(function (response, status, headers, config) {
            //修改缓存
            if (DATA[room.building] && DATA[room.building][room._id] &&
              DATA[room.building][room._id].rent) {
              var cacheRoom = DATA[room.building][room._id];
              _.remove(cacheRoom.rent.contract.products, function (rc) {
                return rc.productrecord === productrecordId;
              });
            }
            save2Cache();
            //回调
            resolve(response, status, headers, config);
          })
          .error(function (response, status, headers, config) {
            reject(response, status, headers, config);
          });
      });
    };

    //自动计算产品到期时间
    service.calcProductEndTime = function(begintime,timelimit) {
      var regexp_limit = /^(\d{1}Y)?(\d{1,2}M)?(\d{1,3}D)?$/,
      	regexp_limitY = /^(\d{1}Y)?/,
      	regexp_limitM = /(\d{1,2}M)?/,
      	regexp_limitD = /(\d{1,3}D)?$/;
    	var endtime = new Date(begintime);
    	//是否一次性消费
    	if(!timelimit || timelimit === '0' || timelimit === ''){
    		return endtime;
    	} else{ //使用正则表达式取值
    		var limit_Y = 0, limit_M = 0, limit_D = 0;

    		var resY = regexp_limitY.exec(timelimit),
    			resM = regexp_limitM.exec(timelimit),
    			resD = regexp_limitD.exec(timelimit);

    		if(resY){
    			limit_Y = parseInt(resY[0]);
    		}
    		if(resM){
    			limit_M = parseInt(resM[0]);
    		}
    		if(resD){
    			limit_D = parseInt(resD[0]);
    		}
    		if(!isNaN(limit_Y)){
    			endtime = $Date.addYears(endtime, limit_Y);
    		}
    		if(!isNaN(limit_M)){
    			endtime = $Date.addMonths(endtime, limit_M);
    		}
    		if(!isNaN(limit_D)){
    			endtime = $Date.addDays(endtime, limit_Y);
    		}
    	}

    	return new Date(endtime);
    };

    //自动修正订购产品数据:到期时间,feetype,unit
    service.fixOrderProducts = function(products,onFixFun,done) {
      //处理订购记录,同时获取产品列表
      function fix(productclasses,orderproducts){
        _.forEach(productclasses, function(productclass){
          _.forEach(productclass.types,function(type){
            _.forEach(type.products,function(product){
              //判断是否为已订购产品
              if(orderproducts){
                var orderproduct = _.find(orderproducts, function(item) {
                    return item._id === product._id;
                });
                if(orderproduct){
                  if(!orderproduct.name){
                    orderproduct.name = product.name;
                  }
                  //已订购记录,冗余feetype,unit属性
                  orderproduct.feetype = product.feetype;
                  orderproduct.unit = product.unit;
                  //回调函数
                  if(onFixFun && _.isFunction( onFixFun )){
                    onFixFun(product,orderproduct);
                  }
                }
              }
            });
          });
        });
      }
      //计算各产品的到期时间
      _.forEach(products,function(product){
        //设置合约中订购产品记录的到期时间
        if(_.isDate(product.endtime)){
          product.expireday = parseInt((new Date(product.endtime) - new Date())/1000/60/60/24);
        }
      });
      service.getProducts(false).then(function(productclasses){
        //处理订购记录
        fix(productclasses,products);
        //回调执行完成方法
        if(done && _.isFunction( done )){
          done(products);
        }
      });
    };

    //新增收费项目
    service.newProduct = function(product) {
      //返回promise
      return $q(function (resolve, reject) {
        var url = '/products/product';
        $http.post(url, product)
          .success(function (response, status, headers, config) {
            var newProduct = response;
            //更新缓存
            if(!newProduct.type){
              newProduct.type = 'custom';
            }
            var thistype = getCacheType(newProduct.type);
            if(!thistype){
              DATA.custom = DATA.custom || {};
              DATA.custom.types = DATA.custom.types || [];
              thistype = _.find(DATA.custom.types, function(type){
                return type._id === newProduct.type;
              });
              if(!thistype){
                thistype = {
                  '_id':'custom',
                  'name':'custom',
                  'productclass':'custom',
                  'products':[]
                };
                DATA.custom.types.push(thistype);
              }
            }
            thistype.products.push(newProduct);
            save2Cache();
            //回调
            resolve(newProduct, status, headers, config);
          })
          .error(function (response, status, headers, config) {
            reject(response, status, headers, config);
          });
      });
    };

    //修改收费项目
    service.editProduct = function(product) {
      //返回promise
      return $q(function (resolve, reject) {
        if (!product || !product._id) {
          return reject('参数错误!');
        }
        var productId = product._id;
        var url = '/products/product/' + productId;
        //修正Product的type,如果是custom,则删除type字段
        if(product.type === 'custom'){
          delete product.type;
        }
        $http.put(url, product)
          .success(function (response, status, headers, config) {
            var newProduct = response;
            //修改缓存
            var thisproduct = getCacheProduct(productId);
            thisproduct = _.assign( thisproduct, newProduct );
            save2Cache();
            //回调
            resolve(newProduct, status, headers, config);
          })
          .error(function (response, status, headers, config) {
            reject(response, status, headers, config);
          });
      });
    };

    //删除收费项目
    service.deleteProduct = function(product) {
      //返回promise
      return $q(function (resolve, reject) {
        if (!product) {
          return reject('参数错误!');
        }
        var productId = product._id ? product._id : product;
        var url = '/products/product/' + productId;
        $http.delete(url, null)
          .success(function (response, status, headers, config) {
            var deleteProduct = response;
            //修改缓存
            if(!deleteProduct.type){
              deleteProduct.type = 'custom';
            }
            var thistype = getCacheType(deleteProduct.type);
            _.remove(thistype.products,function(product){
              return product._id === productId;
            });
            save2Cache();
            // //修改缓存
            // if (DATA[room.building] && DATA[room.building][room._id] &&
            //   DATA[room.building][room._id].rent) {
            //   var cacheRoom = DATA[room.building][room._id];
            //   cacheRoom.rent.contract.products.push(newProduct);
            // }
            // save2Cache();
            // //修改订购记录
            // room.rent.contract.products.push(newProduct);
            //回调
            resolve(response, status, headers, config);
          })
          .error(function (response, status, headers, config) {
            reject(response, status, headers, config);
          });
      });
    };

    //加载缓存
    if (!DATA) {
      loadCacheData();
    }

    return service;
  });
