angular.module('bagu')
  /********
  缓存房间,格式:{"buildingID":{room:[pricelog],room,...}}
  ********/
  .service('$Provider', function($q, $http, $Cache, _) {
    /****对象定义****/
    //表名
    var TABLENAME = 'providers';
    //服务商的产品数据
    var TABLENAME_PRODUCT = 'providerproducts';
    //数据实体对象
    var ProviderData = null;
    var ProductData = null;
    //服务对象
    var service = {};

    /****工具函数****/

    //获取缓存数据
    function loadCacheData() {
      //缓存对象的引用
      ProviderData = $Cache.get(TABLENAME, function(cachetable) {
        ProviderData = $Cache.get(TABLENAME);
      });
      //缓存对象的引用
      ProductData = $Cache.get(TABLENAME_PRODUCT, function(cachetable) {
        ProductData = $Cache.get(TABLENAME_PRODUCT);
      });
      return {
        ProviderData: ProviderData,
        ProductData: ProductData
      };
    }
    //保存数据到缓存
    function save2Cache() {
      //保存结果到缓存
      $Cache.saveTable(TABLENAME);
      $Cache.saveTable(TABLENAME_PRODUCT);
    }
    //保存单个服务商数据到缓存,将_id转换成属性名称,方便后面使用
    function saveProviders2Cache(providers) {
      _.forEach(providers, function(provider) {
        //清除原有房间数据
        ProviderData[provider._id] = provider;
      });
      //写入缓存
      save2Cache();
    }
    //保存单个服务商数据到缓存,将_id转换成属性名称,方便后面使用
    function saveProducts2Cache(providerId,products) {
      //覆盖原有数据
      ProviderData[providerId] = products;
      //写入缓存
      save2Cache();
    }

    /****服务方法****/
    //获取服务商列表
    //fromServer:是否强制从服务器刷新数据
    //返回值:
    // [provider]
    service.getProviders = function(fromServer) {
      fromServer = fromServer || false;
      //返回promise
      return $q(function(resolve, reject) {
        // TODO: 未来增加按楼房单独获取服务商,
        //目前服务商少,一并显示
        if (fromServer !== true && !_.isEmpty(ProviderData)) {
          //返回缓存数据
          return resolve(ProviderData);
        } else {
          //无缓存数据或强制刷新,则从远程取
          $http.get('/providers')
            .success(function(response) {
              //保存到缓存
              saveProviders2Cache(response);
              //回调
              resolve(ProviderData);
            })
            .error(function(err) {
              reject(err);
            });
        }
      });
    };

    //获取第三方服务商的产品
    //providerId:服务商对象_id
    //fromServer:是否强制从服务器刷新数据
    service.getProviderProducts = function(providerId, fromServer) {
      fromServer = fromServer || false;
      //返回promise
      return $q(function(resolve, reject) {
        if (!providerId) {
          return reject('服务商参数错误!');
        }
        //存在于缓存中?(遍历每个building)
        var cacheProviders = ProviderData[providerId];
        //从缓存获取数据
        if (fromServer !== true && !_.isEmpty(cacheProviders)) {
          //返回内存副本
          return resolve(cacheProviders);
        } else {
          //无缓存数据或强制刷新,则从远程取
          $http.get('/provider/' + providerId + '/products')
            .success(function(response) {
              //保存到缓存
              saveProducts2Cache(providerId, response);
              save2Cache();
              //回调
              resolve(response);
            })
            .error(function(err) {
              reject(err);
            });
        }
      });
    };

    //加载缓存
    if (!ProviderData || !ProductData) {
      loadCacheData();
    }

    return service;
  });
