angular.module('bagu')

.service('$Rent', function ($q, $http, $Date, $Cache, $Room, $Invoice, _, DATALIST) {
  /****对象定义****/
  //表名
  var t_rooms = $Cache.get('rooms');
  var t_lastinvoices = $Cache.get('lastinvoices');

  //生成Post回调函数
  function qPostFun(url, postdata) {
    return $q(function (resolve, reject) {
      postdata = postdata || {};
      $http.post(url, postdata)
        .success(function (response) {
          resolve(response);
        }).error(function (err) {
          reject(err);
        });
    });
  }
  //生成Put回调函数
  function qPutFun(url, postdata) {
    return $q(function (resolve, reject) {
      postdata = postdata || {};
      $http.put(url, postdata)
        .success(function (response) {
          resolve(response);
        }).error(function (err) {
          reject(err);
        });
    });
  }
  //生成Get回调函数
  function qGetFun(url) {
    return $q(function (resolve, reject) {
      $http.get(url)
        .success(function (response) {
          resolve(response);
        }).error(function (err) {
          reject(err);
        });
    });
  }
  //生成Delete回调函数
  function qDeleteFun(url) {
    return $q(function (resolve, reject) {
      $http.delete(url)
        .success(function (response) {
          resolve(response);
        }).error(function (err) {
          reject(err);
        });
    });
  }

  //对外出租
  var enableRent = function (roomId) {
    console.log('对外出租', roomId);
    return qPostFun('/rent/enableRent/' + roomId.toString());
  };

  //设为自住房
  var disableRent = function (roomId) {
    console.log('设为自住房', roomId);
    return qPostFun('/rent/disableRent/' + roomId.toString());
  };

  //推广房源
  var popularize = function (roomId) {
    console.log('推广房源', roomId);
    return qPostFun('/rent/popularize/' + roomId.toString());
  };

  //收费
  var invoice = function (room){
    console.log('收费', room);
  };
  //续约
  var autoRenew = function (room, buildingId) {
    console.log('自动续约', room);
    var roomId = room && room._id ? room._id : room;
    return qPostFun('/rent/renew/' + roomId + '/auto').then(function(response){
      if(t_rooms[buildingId] && t_rooms[buildingId][roomId]){
        var cacheroom = t_rooms[buildingId][roomId];
        cacheroom.roomstatus = response.roomstatus; //更新状态
        _.assign(cacheroom.rent, response.rent);
        $Room.fixRoom(cacheroom);
        $Cache.saveTable('rooms');
      }
      return response;
    },function(err){
      return $q.reject(err);
    });
  };
  //续约
  var renew = function (room, data, buildingId) {
    console.log('手动续约', room);
    var roomId = room && room._id ? room._id : room;
    return qPostFun('/rent/renew/' + roomId, data).then(function(response){
      if(t_rooms[buildingId] && t_rooms[buildingId][roomId]){
        var cacheroom = t_rooms[buildingId][roomId];
        cacheroom.roomstatus = response.roomstatus; //更新状态
        _.assign(cacheroom.rent, response.rent);
        $Room.fixRoom(cacheroom);
        $Cache.saveTable('rooms');
      }
      return response;
    },function(err){
      return $q.reject(err);
    });
  };
  //预订
  var order = function (room, data,buildingId) {
    console.log('预订', room);
    var roomId = room && room._id ? room._id : room;
    return qPostFun('/rent/order/' + roomId, data);
  };

  //修改预订信息
  var editOrder = function (roomId, postdata) {
    console.log('修改预订信息', postdata);
    return qPutFun('/rent/order/' + roomId.toString(), postdata);
  };

  //取消预订
  var cancelOrder = function (room) {
    console.log('取消预订', room);
    var roomId = room && room._id ? room._id : room;
    return qDeleteFun('/rent/order/' + roomId);
  };

  //出租(签合同)
  var contract = function (roomId, postdata) {
    console.log('出租(签合同)', postdata);
    return qPostFun('/rent/contract/' + roomId.toString(), postdata);
  };

  //修改合同
  var editContract = function (roomId, postdata) {
    console.log('修改合同', postdata);
    return qPutFun('/rent/contract/' + roomId.toString(), postdata);
  };

  //预约退房
  var preBack = function (buildingId, roomId, postdata) {
    console.log('预约退房', roomId);

    return qPostFun('/rent/preback/' + roomId.toString(),postdata).then(function(room){
      if(t_rooms[buildingId] && t_rooms[buildingId][roomId]){
        var cacheroom = t_rooms[buildingId][roomId];
        cacheroom.popularize = room.popularize; //设为推广状态
        cacheroom.roomstatus = room.roomstatus; //更新状态
        cacheroom.rent.back = cacheroom.rent.back || {};
        cacheroom.rent.back.backdate = postdata.backdate;
        cacheroom.rent.back.backstatus = DATALIST.RENT_BACK_STATUS.WILLBACK; //预约退房状态
        $Cache.saveTable('rooms');
      }
      return room;
    },function(err){
      return $q.reject(err);
    });
  };

  //取消预约退房
  var cancelPreBack = function (buildingId, roomId) {
    console.log('取消预约退房', roomId);

    return qDeleteFun('/rent/preback/' + roomId.toString()).then(function(room){
      if(t_rooms[buildingId] && t_rooms[buildingId][roomId]){
        var cacheroom = t_rooms[buildingId][roomId];
        cacheroom.popularize = room.popularize; //修改推广状态
        cacheroom.roomstatus = room.roomstatus; //修改房间状态
        cacheroom.rent.rentstatus = room.rent.rentstatus;
        cacheroom.rent.back = cacheroom.rent.back || {};
        cacheroom.rent.back.backstatus = DATALIST.RENT_BACK_STATUS.NOBACK; //预约退房状态
        $Room.fixRoom(cacheroom);
        $Cache.saveTable('rooms');
      }
      return room;
    },function(err){
      return $q.reject(err);
    });
  };


  //获取退房信息
  //isRun: 租客逃租
	// deduction:{ //费用扣除情况
	//  total: {type: Number, min: 0}, //退房需要扣除的总金额
	//  items:[{ //需要扣除的产品费用,如租金,水电费等
	// 	 product: { //订购产品,
	// 	 productrecord: { //产品订购记录,
	// 	 name:{ //产品名称: 如:4M1年600元,杂费,管理费,房租,水费,电费,
	// 	 begindate: { //计费周期的起始时间,
	// 	 enddate: { //计费周期的结束时间,
	// 	 value: { //按量计费时,存储抄表数据
	// 		 init: { //期初读数,
	// 		 current:{ //当前读数,
	// 		 amount:{ //用量
	// 	 },
	// 	 price:{ //单价,
	// 	 unit:{ //单位,
	// 	 money:{ //金额,
	//  }],
	//  deduother:[{
	// 	 name:{ //其他费用名称,如:卫生费,房屋修复费,
	// 	 money:{ //金额
	//  }],
	//  dedumemo: {type: String} //退房扣除费用的说明
	// },
  var getBackInfo = function (roomId, isRun) {
    isRun = isRun || false;
    console.log('预约退房', roomId, isRun);
    return qGetFun('/rent/back/' + roomId.toString()).then(function(backinfo){
      //调整项目,设置退房金额为0,增加临时收费项目:未办手续
      if(backinfo.backmoney !== 0 && isRun){
        //设置退租状态为逃租
        backinfo.backstatus = DATALIST.RENT_BACK_STATUS.RUN;
        if(backinfo.backmoney > 0){
          backinfo.description = '租客逃租,损失' + backinfo.backmoney.toString() + '元';
        }else{
          backinfo.description = '租客逃租,少退' + (backinfo.backmoney*-1).toString() + '元';
        }
        backinfo.deduction.deduother.push({
          name: '未办手续',
          money: backinfo.backmoney * -1
        });
        backinfo.backmoney = 0;
      }
      return backinfo;
    },function(err){
      return $q.reject(err);
    });
  };

  //退房
  var back = function (buildingId, roomId,postdata) {
    console.log('退房', roomId,postdata);
    return qPostFun('/rent/back/' + roomId.toString(),postdata).then(function(response){
      if(t_rooms[buildingId] && t_rooms[buildingId][roomId]){
        var room = t_rooms[buildingId][roomId];
        //room.prerent = undefined; //考虑已预订情况,此处应删除
        room.rent = undefined;
        room.popularize = 1;
        room.roomstatus = room.prerent ? DATALIST.ROOM_STATUS.ORDER : DATALIST.ROOM_STATUS.EMPTY;
        $Room.fixRoom(room);
        $Cache.saveTable('rooms');
      }
      return response;
    },function(err){
      return $q.reject(err);
    });
  };

  //收租
  var newInvoice = function (buildingId,roomId) {
    console.log('收租', roomId);

    return qGetFun('/rent/' + roomId.toString() + '/newinvoice').then(function(response){
      return response;
    },function(err){
      return $q.reject(err);
    });
  };

  //催缴
  var sendRentNotice = function (data) {
    console.log('催缴', data);
  };

  //转让
  var transfer = function (data) {
    console.log('转让', data);
  };

  //删除房间
  var remove = function (data) {
    console.log('删除房间', data);
  };

  //获取出租历史
  var list = function (data) {
    console.log('获取出租历史', data);
  };

  //发送短信
  var sendSms = function (data) {
    console.log('发送短信', data);
  };


  /**
   * 计算费用,服务端与客户端的计费方法应该保持一致
   param:
    feetype: 计费方式，0,一次性，1，按月固定计费，2，按量计费，3，按量阶梯计费, 4,按房间面积计费, 5,按入住人数计费
    price: 单价
    begintime: 开始时间
    endtime: 结束时间
    value: 数值,根据feetype不同有不同含义
      0:忽略; 1:忽略; 2:用量; 3:用量; 4:面积; 5:人数
   return: 应收金额
   */
  var calcProductMoney = function (feetype, price, begintime, endtime,
    value) {
    value = value || 0;
    var result = 0.0;
    switch (feetype) {
    case DATALIST.PRODUCT_FEETYPE.ONCE:
      result = price;
      break;
    case DATALIST.PRODUCT_FEETYPE.MONTHLY:
      //计算月份数,不足整月按天数计算(一月按30天计算)
      var monthday = $Date.monthdayDiff(begintime, new Date(endtime));
      result = monthday.month * price + monthday.day * price / monthday.maxdays;
      break;
    case DATALIST.PRODUCT_FEETYPE.AMOUNT:
    case DATALIST.PRODUCT_FEETYPE.STEP: //按量阶梯计费暂不考虑
      result = price * value;
      break;
    case DATALIST.PRODUCT_FEETYPE.AREA:
    case DATALIST.PRODUCT_FEETYPE.PERSON:
      result = price * value;
      break;
    }
    return Math.round(result);
  };

  return {
    enableRent: enableRent,
    disableRent: disableRent,
    popularize: popularize,
    order: order,
    editOrder: editOrder,
    cancelOrder: cancelOrder,
    contract: contract,
    editContract: editContract,
    preBack: preBack,
    cancelPreBack: cancelPreBack,
    getBackInfo: getBackInfo,
    back: back,
    newInvoice: newInvoice,
    sendRentNotice: sendRentNotice,
    transfer: transfer,
    autoRenew: autoRenew,
    renew: renew,
    remove: remove,
    list: list,
    sendSms: sendSms,
    calcProductMoney: calcProductMoney
  };
});
