angular.module('bagu')

/***
Remote File Service
远程文件服务
***/
.service('$Rfs', function ($q, $rootScope, $http, $ionicModal, $timeout, $ionicPlatform, _, $Mustache,
		SERVER_SETTINGS, AuthService, $cordovaFile, $cordovaFileTransfer, $cordovaWebFileUpload) {

		//模板定义
		/* 采用mustache模块.除传入的socpe外,还会自动附上:
		    localFileSystem(本地目录系统,包含常用目录)
		    server(server设置对象)
		    module(模块名称)
		    fileId(目标文件,文件名或文件序号)
		    token(当前登录令牌)
		  */
		var URI_TEMPLATE = {
			//身份证
			credentials: {
				//下载文件远程路径
				downuri: '{{{server.CUR_SERVER}}}/customer/{{customer._id}}/cp/{{{fileId}}}?access_token={{{token}}}',
				//上传文件远程路径
				upuri: '{{{server.CUR_SERVER}}}/customer/{{customer._id}}/cp/{{{fileId}}}',
				//目标本地缓存文件夹
				dir: '{{{localFileSystem.dataDirectory}}}',
				//本地缓存文件路径和名称
				path: '{{{module}}}/{{{fileId}}}',
			},
			//房间相片
			room: {
				//下载文件远程路径
				downuri: '{{{server.CUR_SERVER}}}/room/{{room._id}}/cp/{{{fileId}}}?access_token={{{token}}}',
				//上传文件远程路径
				upuri: '{{{server.CUR_SERVER}}}/room/{{room._id}}/cp/{{{fileId}}}',
				//目标本地缓存文件夹
				dir: '{{{localFileSystem.dataDirectory}}}',
				//本地缓存文件路径和名称
				path: '{{{module}}}/{{{fileId}}}',
			},
			//初始化Excel文件
			init: {
				//下载文件远程路径
				downuri: '{{{server.CUR_SERVER}}}/init/{{building._id}}/execl/{{{fileId}}}?access_token={{{token}}}',
				//上传文件远程路径
				upuri: '{{{server.CUR_SERVER}}}/init/{{building._id}}/excel',
				//目标本地缓存文件夹
				dir: '{{{localFileSystem.dataDirectory}}}',
				//本地缓存文件路径和名称
				path: '{{{module}}}/{{{fileId}}}',
			}
		};

		//修正计算文件目录和文件名的Mustache环境
		//module: 模块名称
		//fileId: 文件Id
		//scope: 自动计算文件路径和文件名的环境
		var fixFileScope = function (module, fileId, scope) {
			return _.assign({
				module: module,
				localFileSystem: window.cordova && window.cordova.file ? window.cordova.file : {},
				server: SERVER_SETTINGS,
				fileId: fileId,
				token: AuthService.authToken()
			}, scope);
		};

		//获取模块设置
		//module: 模块名称
		//fileId: 文件Id
		//scope: 自动计算文件路径和文件名的环境
		var getModuleSettings = function (module, fileId, scope) {
			var result = {
				downUri: null,
				encodeDownUri: null,
				upUri: null,
				encodeUpUri: null,
				dir: null,
				filename: null
			};
			if (!URI_TEMPLATE[module] || !URI_TEMPLATE[module].dir || !URI_TEMPLATE[module].path) {
				throw new Error('模块未配置或配置错误,请联系管理员!');
			}
			//准备计算文件名的环境
			var fileScope = fixFileScope(module, fileId, scope);
			//远程服务器文件地址
			result.downUri = $Mustache.render(URI_TEMPLATE[module].downuri, fileScope);
			result.encodeDownUri = encodeURI(result.downUri);
			result.upUri = $Mustache.render(URI_TEMPLATE[module].upuri, fileScope);
			result.encodeUpUri = encodeURI(result.upUri);
			//本地缓存文件地址
			result.dir = $Mustache.render(URI_TEMPLATE[module].dir, fileScope);
			result.filename = $Mustache.render(URI_TEMPLATE[module].path, fileScope);

			return result;
		};

		//从服务器下载文件
		//uri: 远程文件地址
		//dir: 本地缓存目录
		//filename: 本地缓存文件名
		var downloadFile = function (uri, dir, filename, scope) {
			return $q(function (resolve, reject, notify) {
				$cordovaFile.createFile(dir, filename, true).then(function (file) {
					$cordovaFileTransfer.download(uri, filename, {
						headers: {
							'Authorization': 'Bearer ' + AuthService.authToken()
						}
					}).then(
						function (entry) {
							console.log('Successful download...');
							console.log('download complete: ' + entry.toURL());
							return resolve(entry);
						},
						function (error) {
							console.log('download error source ' + error.source);
							console.log('download error target ' + error.target);
							console.log('upload error code' + error.code);
							return reject(error);
						},
						function (progress) {
							return notify(progress);
						}
					);
				}, function (err) {
					return reject(err);
				});
			});
		};

		//获取文件
		//module: 模块名称
		//fileId: 文件Id
		//fromServer: 是否强制从服务器获取文件,默认优先从本地获取
		var getFile = function (module, fileId, scope, fromServer) {
			fromServer = fromServer || false;
			return $q(function (resolve, reject) {
				//获取并计算模块设置
				var msettings = null;
				try {
					msettings = getModuleSettings(module, fileId, scope);
				} catch (e) {
					return reject('模块未配置或配置错误,请联系管理员!');
				}

				//Web环境?直接返回远程文件访问地址:带access_token参数
				if (!window.cordova || !window.cordova.file || 1 === 1) {
					msettings.downUri = msettings.downUri;
					return resolve(msettings.downUri);
				} else {
					// TODO: 缓存到本地文件目前有问题,文件下载完之后,页面中无法显示.
					//延后排插该问题.
					var doGetFile = function () {
						//强制从远程服务器获取文件
						if (fromServer) {
							downloadFile(msettings.encodeDownUri, msettings.dir, msettings.filename).then(function (file) {
								return resolve(file.nativeURL);
							}, function (err) {
								return reject(err);
							});
						} else {
							//是本地文件?
							$cordovaFile.checkFile(fileId).then(function (localfile) {
								//是否过期?
								//显示图片
								return resolve(fileId);
							}, function (err) {
								//缓存文件是否存在?
								$cordovaFile.checkFile(msettings.dir, msettings.filename).then(function (oldfile) {
									//是否过期?
									//显示图片
									return resolve(oldfile.nativeURL);
								}, function (err) {
									//文件不存在则
									downloadFile(msettings.encodeDownUri, msettings.dir, msettings.filename).then(function (file) {
										return resolve(file.nativeURL); //fullPath //nativeURL
									}, function (err) {
										return reject(err);
									});
								});
							});
						}
					};
					//检查目录:CacheDirectory/module/
					$cordovaFile.checkDir(msettings.dir, module, false).then(function (dirEntry) {
						return doGetFile();
					}, function (err) {
						//创建本地缓存目录
						$cordovaFile.createDir(msettings.dir, module, false).then(function (dirEntry) {
							return doGetFile();
						}, function (err) {
							return reject(err);
						});
					});
				}
			});
		};

		//上传文件
		//module: 模块名称
		//fileId: 文件Id
		//file:本地文件地址
		var uploadFile = function (module, fileId, file, scope) {
			var q = $q.defer();
			var filename = file;
			//WEB环境? WEB环境下,file传入值为input组件获取的第一个file对象
			if (_.isObject(filename)) {
				filename = file.name;
			}
			//获取并计算模块设置
			var msettings = null;
			try {
				msettings = getModuleSettings(module, fileId, scope);
			} catch (e) {
				return q.reject('模块未配置或配置错误,请联系管理员!');
			}
			var options = {
				fileKey: 'file',
				fileName: filename,
				mimeType: 'image/jpeg',
				headers: {
					'Authorization': AuthService && AuthService.authToken() ? 'Bearer ' + AuthService.authToken() : undefined
				}
			};
			var ft;
			//是否纯WEB环境下的Input文件对象?
			if (_.isObject(file)) {
				ft = $cordovaWebFileUpload.upload(msettings.encodeUpUri, file, options);
			} else {
				ft = $cordovaFileTransfer.upload(msettings.encodeUpUri, file, options);
			}
			q.promise.abort = function () {
				ft.abort();
			};
			ft.then(function (data) {
				//上传成功
				var response = null;
				if (data && data.response && _.isString(data.response)) {
					response = angular.fromJson(data.response);
				} else {
					response = data && data.response ? data.response : null;
				}
				q.resolve(response);
			}, function (err) {
				q.reject(err);
			}, function (progress) {
				q.notify(progress);
			});
			return q.promise;
		};

		return {
			getFile: getFile,
			uploadFile: uploadFile,
			getModuleSettings: getModuleSettings
		};
	}

);
