angular.module('bagu')
  /********
  当前抄表缓存,格式:{
    "buildingID":{
      "productId":{
        "_ids":[roomId,roomId,roomId],
        "roomId":{
          _id: room._id,
          code: room.code,
          floor: room.floor,
          rent: room.rent,
          lastvalue: 0,
          guessvalue: 0,
          lastreadtime: new Date()
        }
      }
    }
  }
  ********/
  .service('$RoomValue', function($q, $http, $Cache, $Date, $Str, $Building, $Room, $Invoice, DATALIST, LOCAL, _) {
    /****对象定义****/
    //表名
    var TABLENAME = 'roomvalues';
    var TABLENAME_VALUELOGS = 'valuelogs';
    var TABLENAME_ROOMS = 'rooms';
    var TABLENAME_LASTINVOICES = 'lastinvoices';
    var INDEXNAME = '_ids';
    //数据实体对象
    var DATA = null;
    //服务对象
    var service = {};
    //定义服务全局变量,用户房间抄表界面返回抄表列表页时,列表页能同步切换为当前抄表收费项目
    var curProduct = null;
    var orderOptions = [{
      text: '按楼层分组', //例如: 101,102,103, 201,202,203 ....
      value: 0,
      //排序方法,返回Ids列表
      doOrder: function(buildingId, valuelist){
        var sortlist = _.sortBy( valuelist, function(item){
          var floor = item.floor || 0;
          return $Str.padLeft(floor.toString(),4) + item.code;
        });
        var _ids = [];
        if (sortlist && _.isArray(sortlist)) {
          _.forEach(sortlist, function(roomvalue) {
            if (roomvalue && roomvalue._id) {
              //判断是否总表
              if(roomvalue._id === buildingId){
                if(_.isNumber(roomvalue.masterindex)){
                  _ids.push(roomvalue.masterindex);
                }
              }else{
                _ids.push(roomvalue._id);
              }
            }
          });
        }
        return _ids;
      }
    },{
      text: '按房号分组', //例如: 101,201,301, 102,202,302 ....
      value: 1,
      //排序方法,返回Ids列表
      doOrder: function(buildingId, valuelist){
        var sortlist = _.sortBy( valuelist, function( item){
          return $Str.getCodeNum(item.code) * 1000 + item.floor;
        });
        var _ids = [];
        if (sortlist && _.isArray(sortlist)) {
          _.forEach(sortlist, function(roomvalue) {
            if (roomvalue && roomvalue._id) {
              //判断是否总表
              if(roomvalue._id === buildingId){
                if(_.isNumber(roomvalue.masterindex)){
                  _ids.push(roomvalue.masterindex);
                }
              }else{
                _ids.push(roomvalue._id);
              }
            }
          });
        }
        return _ids;
      }
    }];

    /****工具函数****/
    //根据定义生成索引缓存
    function genIndex(buildingId,productId,valuelist, order){
      var result = {
        value: undefined,
        indexs: {}
      };
      _.forEach(orderOptions, function(option){
        if(option && option.doOrder && _.isFunction(option.doOrder)){
          result.indexs[option.value] = option.doOrder(buildingId,valuelist);
        }else{
          result.indexs[option.value] = _.map(valuelist, function(item){
            if(item._id === buildingId){
              if(_.isNumber(item.masterindex)){
                return item.masterindex;
              }
            }else{
              return item._id;
            }
          });
        }
      });
      var defaultorder = _.isUndefined(order) ? orderOptions[0].value : order;
      if(!_.find(orderOptions, function(item){
        return item.value === defaultorder;
      })){
        defaultorder = orderOptions[0].value;
      }
      result.value = defaultorder;

      return result;
    }
    //获取指定索引
    function getIndex(buildingId,productId,order){
      var result = [];
      if(!DATA[buildingId] || !DATA[buildingId][productId]){
        return result;
      }
      var indexObject = DATA[buildingId][productId][INDEXNAME];
      //如果未生成,则立即生成
      if(!indexObject){
        indexObject = genIndex(buildingId,productId,DATA[buildingId][productId],order);
      }
      if(indexObject && indexObject.indexs){
        //是否未传入的排序方式参数或者排序方式不存在?
        if(_.isUndefined(order)){
          //取上一次的排序方式
          order = indexObject.value;
        }
        if(!indexObject.indexs[order]){
          result = indexObject.indexs[orderOptions[0].value];
        }else{
          result = indexObject.indexs[order];
        }
      }
      return result;
    }

    //修正抄表数据(补充抄表缓存中的读数)
    function fixRoomValues(product, roomvalues){
      //获取缓存抄表数据
      var valuelogcachedb = service.getValueLogCacheDB();
      if(valuelogcachedb && valuelogcachedb.opdata){
        _.forEach(roomvalues, function(item){
          if(!item._id){
            return;
          }
          item.cacheValue = undefined;
          var roomlog = _.findLast(valuelogcachedb.opdata, function(log){
            return log.room === item._id && log.product === product;
          });
          if(roomlog){
            item.cacheValue = roomlog.value;
          }
        });
      }
      return roomvalues;
    }
    function fixRoomValue(product, roomvalue){
      //获取缓存抄表数据
      var valuelogcachedb = service.getValueLogCacheDB();
      if(valuelogcachedb && valuelogcachedb.opdata && roomvalue._id){
        roomvalue.cacheValue = undefined;
        var roomlog = _.findLast(valuelogcachedb.opdata, function(log){
          return log.room === roomvalue._id && log.product === product;
        });
        if(roomlog){
          roomvalue.cacheValue = roomlog.value;
        }
      }
      return roomvalue;
    }

    //获取缓存数据
    function loadCacheData() {
      //缓存对象的引用
      DATA = $Cache.get(TABLENAME, function(cachetable) {
        DATA = cachetable;
      });
      return DATA;
    }
    //保存数据到缓存
    function save2Cache() {
      //保存结果到缓存
      $Cache.saveTable(TABLENAME);
    }
    //保存楼房产品的抄表数据到缓存,将_id转换成属性名称,方便后面使用
    function saveRoomValues2Cache(building, product, valuelist) {
      var buildingId = building._id ? building._id : building;
      var productId = product._id ? product._id : product;
      //清除原有数据
      DATA[buildingId] = DATA[buildingId] || {};
      DATA[buildingId][productId] = {};
      //_id转换
      //var _ids = [];
      if (valuelist && _.isArray(valuelist)) {
        _.forEach(valuelist, function(roomvalue) {
          if (roomvalue && roomvalue._id) {
            //判断是否总表
            if(roomvalue._id === buildingId){
              if(_.isNumber(roomvalue.masterindex)){
                DATA[buildingId][productId][roomvalue.masterindex] = roomvalue;
                //_ids.push(roomvalue.masterindex);
              }
            }else{
              DATA[buildingId][productId][roomvalue._id] = roomvalue;
              //_ids.push(roomvalue._id);
            }
          }
        });
      }
      //保存索引
      DATA[buildingId][productId][INDEXNAME] = genIndex(buildingId,productId,valuelist);
      //写入缓存
      save2Cache();
    }
    //保存房间产品的抄表数据到缓存,将_id转换成属性名称,方便后面使用
    function saveRoomLastValue2Cache(building, product, room, lastvalue) {
      var buildingId = building._id ? building._id : building;
      var productId = product._id ? product._id : product;
      var roomId = room._id ? room._id : room;
      //找到缓存记录并更新
      DATA[buildingId] = DATA[buildingId] || {};
      DATA[buildingId][productId] = DATA[buildingId][productId] || {};
      DATA[buildingId][productId][roomId] = lastvalue;
      //写入缓存
      save2Cache();
    }
    //取抄表数据列表,不包含索引
    function getRoomValuesNoIdsFromCache(building, product) {
      var buildingId = building._id ? building._id : building;
      var productId = product._id ? product._id : product;
      var result = _.reduce(DATA[buildingId][productId], function(result,
        roomvalue, key) {
        if (key !== INDEXNAME) {
          result.push(roomvalue);
        }
        return result;
      }, []);
      return fixRoomValues(product, result);
    }

    /****服务方法****/

    //获取指定楼房和产品的抄表数据
    //building:楼房(_id,或对象)
    //product:产品(_id,或对象,需为)
    //fromServer:是否强制从服务器刷新数据
    //返回值:
    // {
    //  roomId:room
    //  ....
    // }
    var getRoomValues = function(building, product, fromServer) {
      fromServer = fromServer || false;
      //返回promise
      return $q(function(resolve, reject) {
        if (!building) {
          return reject('楼房参数错误!');
        }
        if (!product) {
          return reject('产品参数错误!');
        }
        var buildingId = building._id ? building._id : building;
        var productId = product._id ? product._id : product;

        DATA[buildingId] = DATA[buildingId] || {};
        DATA[buildingId][productId] = DATA[buildingId][productId] || {};

        //获取楼房下房间数量,如果抄表记录数与房间数不相等,则从服务器获取数据
        var roomsdb = $Cache.get(TABLENAME_ROOMS);
        var rooms = roomsdb[buildingId] ? Object.getOwnPropertyNames(roomsdb[buildingId]).length : 0;
        var values = 0;
        _.forEach(DATA[buildingId][productId], function(item){
          if(item._id && buildingId !== item._id){
            values++;
          }
        });


        if (fromServer !== true && !_.isEmpty(DATA[buildingId][productId]) && rooms === values) {
          //返回缓存数据
          return resolve(getRoomValuesNoIdsFromCache(buildingId,
            productId));
        } else {
          //无缓存数据或强制刷新,则从远程取
          $http.get('/valuelogs/roomlist/' + buildingId + '/' + productId)
            .success(function(response) {
              //保存到缓存
              saveRoomValues2Cache(buildingId, productId, response);
              //回调
              resolve(fixRoomValues(productId,DATA[buildingId][productId]));
            })
            .error(function(err) {
              reject(err);
            });
        }
      });
    };

    service.getRoomValues = getRoomValues;

    //是否未抄表房间
    var isUnReadValueRoom = function(building, roomvalue) {
      if (!building._id) {
        building = $Building.getCacheBuilding(building);
      }
      var beforevalueday = building && building.settings.before ? building.settings.before : 0;
      var checkdays = 30 - beforevalueday;
      return (!roomvalue.readtime && $Date.dayDiff(new Date(roomvalue.lastreadtime), new Date()) > checkdays) || (roomvalue.readtime && $Date.dayDiff(new Date(roomvalue.readtime), new Date()) > checkdays);
    };
    service.isUnReadValueRoom = isUnReadValueRoom;

    //获取楼房需要抄表的房间数
    var getBuildingUnReadValueCount = function(building, products, fromServer) {
      //默认从缓存取数据
      fromServer = fromServer || false;
      var promise = [];
      var result = {};
      if (!building._id) {
        building = $Building.getCacheBuilding(building);
      }
      _.forEach(products, function(product) {
        var productId = product._id ? product._id : product;
        promise.push(getRoomValues(building, productId, fromServer).then(function(valuelogs) {
          var count = 0,
            result = {};
          _.forEach(valuelogs, function(item) {
            //上次抄表日期超过一个月,则判断为需要抄表
            if (isUnReadValueRoom(building, item)) {
              count++;
            }
          });
          result[productId] = {
            count: count,
            valuelogs: valuelogs
          };
          return result;
        }, function(err) {
          return $q.reject(err);
        }));
      });
      //返回promise
      return $q(function(resolve, reject) {
        $q.all(promise).then(function(results) {
          var result = {};
          //设置返回对象
          _.forEach(results, function(item) {
            _.assign(result, item);
          });
          resolve(result);
        }, function(err) {
          reject(err);
        });
      });
    };

    service.getBuildingUnReadValueCount = getBuildingUnReadValueCount;


    //获取指定房间的抄表数据(最新一笔)
    //building:楼房对象或_id
    //product:产品对象或_id
    //room:房间对象或_id
    service.getRoomValue = function(building, product, room, fromServer) {
      fromServer = fromServer || false;
      //返回promise
      return $q(function(resolve, reject) {
        if (!building) {
          return reject('楼房参数错误!');
        }
        if (!product) {
          return reject('产品参数错误!');
        }
        if (!room) {
          return reject('房间参数错误!');
        }
        var buildingId = building._id ? building._id : building;
        var productId = product._id ? product._id : product;
        var roomId = room._id ? room._id : room;
        if (fromServer !== true && !_.isEmpty(DATA[buildingId]) && !_.isEmpty(
            DATA[buildingId][productId]) && !_.isEmpty(DATA[buildingId]
            [productId][roomId])) {
          //返回缓存数据
          return resolve(fixRoomValue(productId, DATA[buildingId][productId][roomId]));
        } else {
          //无缓存数据或强制刷新,则从远程取
          $http.get('/valuelogs/last/' + roomId + '/' +
              productId)
            .success(function(response) {
              if (!response) {
                return reject('无当前费用的抄表记录!');
              } else {
                var lastvaluelog = response[productId];
                //保存到缓存
                saveRoomLastValue2Cache(buildingId, productId, roomId, lastvaluelog);
                //回调
                return resolve(fixRoomValue(productId, DATA[buildingId][productId][roomId]));
              }
            })
            .error(function(err) {
              reject(err);
            });
        }
      });
    };

    ///valuelogs/list/:roomId/:productId
    //获取指定房间的抄表数据(所有历史记录)
    //product:产品对象或_id
    //room:房间对象或_id
    service.getValuelogHistorys = function(product, building, room) {
      //返回promise
      return $q(function(resolve, reject) {
        if (!product) {
          return reject('产品参数错误!');
        }
        if (!room) {
          return reject('房间参数错误!');
        }
        var productId = product._id ? product._id : product;
        var buildingId = building._id ? building._id : building;
        var roomId = room._id ? room._id : room;
        var url;
        //是否总表?
        if(!_.isNaN(Number(roomId))){
          url = '/valuelogs/master/'+buildingId+'/'+roomId+'/' + productId;
        }else{
          url = '/valuelogs/list/' + roomId + '/' + productId;
        }
        //无缓存数据或强制刷新,则从远程取
        $http.get(url)
          .success(function(valuelogs) {
            //回调
            return resolve(valuelogs);
          })
          .error(function(err) {
            reject(err);
          });
      });
    };


    //更新房间的最后抄表数据(所有按量收费产品)
    //room:房间对象或_id
    service.updateRoomValue = function(building, room) {
      //返回promise
      return $q(function(resolve, reject) {
        if (!building) {
          return reject('楼房参数错误!');
        }
        if (!room) {
          return reject('房间参数错误!');
        }
        var buildingId = building._id ? building._id : building;
        var roomId = room._id ? room._id : room;
        //从远程取
        $http.get('/valuelogs/last/' + roomId)
          .success(function(response) {
            if (!response) {
              return reject('刷新抄表记录出错!');
            } else {
              _.forEach(response, function(lastvaluelog, productId) {
                //保存到缓存
                saveRoomLastValue2Cache(buildingId, productId, roomId, lastvaluelog);
              });
              //回调
              return resolve(response);
            }
          })
          .error(function(err) {
            reject(err);
          });
      });
    };

    //获取下一个房间抄表数据(仅从缓存取)
    //building:楼房对象或_id
    //product:产品对象或_id
    //room:房间对象或_id
    service.getNextRoomValue = function(building, product, room, step) {
      if (!building || !product || !room) {
        return null;
      }
      var buildingId = building._id ? building._id : building;
      var productId = product._id ? product._id : product;
      var roomId = room._id ? room._id : room,
        nextid = null;
      var idsId = roomId === buildingId ? room.masterindex : roomId;
      step = step || 1; //默认下一个
      //索引
      var ids = getIndex(buildingId,productId); //DATA[buildingId][productId][INDEXNAME];
      //当前ID
      var index = _.findIndex(ids, function(item) {
        return item === idsId;
      });
      index += step;
      if (index < 0 || index > ids.length - 1) {
        nextid = step > 0 ? ids[0] : ids[ids.length - 1];
      } else {
        nextid = ids[index];
      }
      return fixRoomValue(productId, DATA[buildingId][productId][nextid]);
    };

    /*******************
    离线抄表功能
    TODO
      此处未考虑抄表记录先后问题,
      未来需要按照房间和产品进行单独保存,同一房间+产品的需要按照抄表顺序,不同房间不需要按照顺序.
    *******************/
    //获取抄表记录缓存表
    var getValueLogCacheDB = function() {
      var valuelogsdb = $Cache.get(TABLENAME_VALUELOGS);
      if (!valuelogsdb.opdata) {
        valuelogsdb.opdata = [];
      }
      return valuelogsdb;
    };
    service.getValueLogCacheDB = getValueLogCacheDB;
    //获取抄表记录缓存表
    var getCacheValueLogs = function() {
      var valuelogsdb = getValueLogCacheDB();
      return valuelogsdb.opdata;
    };
    service.getCacheValueLogs = getCacheValueLogs;
    //根据重试次数排序,优先处理重试次数少的
    var sortValueLogCacheDB = function() {
      var cachedb = getValueLogCacheDB();
      cachedb.opdata = _.sortBy(cachedb.opdata, 'retry');
    };
    service.sortValueLogCacheDB = sortValueLogCacheDB;
    //设置优先重试(缓存抄表记录)
    var resetValueLogCacheRetry = function(cachelog) {
      cachelog.retry = 0;
      //重新按照重试次数排序
      sortValueLogCacheDB();
      //保存缓存
      $Cache.saveTable(TABLENAME_VALUELOGS);
    };
    service.resetValueLogCacheRetry = resetValueLogCacheRetry;
    //设置重试次数(缓存抄表记录)
    var incValueLogCacheRetry = function(buildingId, productId, roomId, readtime) {
      var itemretry = 0;
      //参数校验
      if (!_.isString(buildingId) || !_.isString(productId) || !_.isString(roomId) || !$Date.isValidDate(readtime)) {
        return false;
      }
      var readdate = readtime.getFullYear().toString() + readtime.getMonth().toString() + readtime.getDate().toString();
      //设置重试次数
      var cachedb = getValueLogCacheDB();
      _.forEach(cachedb.opdata, function(item) {
        var itemtime = new Date(item.readtime);
        var itemdate = itemtime.getFullYear().toString() + itemtime.getMonth().toString() + itemtime.getDate().toString();
        if (item.building === buildingId && item.product === productId && item.room === roomId && itemdate === readdate) {
          itemretry = item.retry = item.retry ? item.retry + 1 : 1;
          return false;
        }
      });
      //重新按照重试次数排序
      //cachedb.opdata = _.sortBy( cachedb.opdata, ['retry','readtime']);
      sortValueLogCacheDB();
      //保存缓存
      $Cache.saveTable(TABLENAME_VALUELOGS);
      //返回
      return itemretry;
    };
    service.incValueLogCacheRetry = incValueLogCacheRetry;
    //删除缓存抄表记录
    service.deleteValueLogCacheObj = function(cachelog) {
      var valuelogsdb = getValueLogCacheDB();
      if (!valuelogsdb.opdata) {
        valuelogsdb.opdata = [];
      }
      //清除操作记录
      _.remove(valuelogsdb.opdata, function(item) {
        return item === cachelog;
      });
      //保存缓存
      $Cache.saveTable(TABLENAME_VALUELOGS);
      //返回
      return true;
    };
    //删除缓存抄表记录
    service.deleteValueLogCache = function(buildingId, productId, roomId, readtime, value) {
      //参数校验
      if (!_.isString(buildingId) || !_.isString(productId) || _.isUndefined(roomId) || !$Date.isValidDate(readtime) || !_.isNumber(value)) {
        return false;
      }
      var valuelogsdb = getValueLogCacheDB();
      if (!valuelogsdb.opdata) {
        valuelogsdb.opdata = [];
      }
      var readdate = readtime.getFullYear().toString() + readtime.getMonth().toString() + readtime.getDate().toString();
      //清除操作记录
      _.remove(valuelogsdb.opdata, function(item) {
        var itemtime = new Date(item.readtime);
        var itemdate = itemtime.getFullYear().toString() + itemtime.getMonth().toString() + itemtime.getDate().toString();
        return item.building === buildingId && item.product === productId && (item.room === roomId || item.masterindex === roomId) && itemdate === readdate && item.value === value;
      });
      //保存缓存
      $Cache.saveTable(TABLENAME_VALUELOGS);
      //返回
      return true;
    };

    //保存的抄表记录到缓存
    service.savedValueLog2Cache = function(buildingId, productId, roomId, readtime, value) {
      //参数校验
      if (!_.isString(buildingId) || !_.isString(productId) || _.isUndefined(roomId) || !$Date.isValidDate(readtime) || !_.isNumber(value)) {
        return 0;
      }
      var valuelogsdb = getValueLogCacheDB();
      if (!valuelogsdb.opdata) {
        valuelogsdb.opdata = [];
      }
      var readdate = readtime.getFullYear().toString() + readtime.getMonth().toString() + readtime.getDate().toString();
      //确保同一房间同一天只有一条记录
      _.remove(valuelogsdb.opdata, function(item) {
        var itemtime = new Date(item.readtime);
        var itemdate = itemtime.getFullYear().toString() + itemtime.getMonth().toString() + itemtime.getDate().toString();
        return item.building === buildingId && item.product === productId && item.room === roomId && itemdate === readdate;
      });
      var index = valuelogsdb.opdata.push({
        building: buildingId,
        product: productId,
        room: _.isString(roomId) ? roomId : buildingId,
        masterindex: _.isNumber(roomId) ? roomId : undefined,
        readtime: readtime,
        value: value,
        retry: 0
      });
      //保存缓存
      $Cache.saveTable(TABLENAME_VALUELOGS);
      //返回
      return index;
    };

    //批量获取未保存的抄表记录
    //优先处理重试次数少的(数组中已按重试次数排序)
    //一次只返回同一楼房的抄表记录
    service.getUnSavedValueLogs = function(count) {
      //默认批量上传20条抄表记录
      count = count ? count : 20;
      var valuelogsdb = getValueLogCacheDB();
      var valuelogs = [], building = null;
      if (!valuelogsdb.opdata) {
        valuelogsdb.opdata = [];
      }
      if (valuelogsdb.opdata.length > 0) {
        var unsavelog = null, i = 0;
        //小米手机上出现过无法后台同步抄表记录的问题,疑为列表中存在错误记录,此处判断一下
        while (i < valuelogsdb.opdata.length && valuelogs.length < count) {
          unsavelog = valuelogsdb.opdata[i];
          if(unsavelog && (!building || building === unsavelog.building)){
            // unsavelog.building = unsavelog.building ? unsavelog.building : unsavelog.buildingId;
            // unsavelog.product = unsavelog.product ? unsavelog.product : unsavelog.productId;
            // unsavelog.room = unsavelog.room ? unsavelog.room : unsavelog.roomId;
            valuelogs.push(unsavelog);
            if(!building){
              building = unsavelog.building;
            }
          }
          i++;
        }
      }
      return valuelogs;
    };

    //取第一个未保存的抄表记录
    //优先处理重试次数少的
    service.getUnSavedValueLog = function() {
      var valuelogsdb = getValueLogCacheDB();
      if (!valuelogsdb.opdata) {
        valuelogsdb.opdata = [];
      }
      if (valuelogsdb.opdata.length > 0) {
        var unsavelog = null, i = 0;
        //小米手机上出现过无法后台同步抄表记录的问题,疑为列表中存在错误记录,此处判断一下
        while (!unsavelog && i < valuelogsdb.opdata.length) {
          unsavelog = valuelogsdb.opdata[0];
          i++;
        }
        //设置重试次数时,已自动按照重试次数排序,此处直接取第一条即可
        return unsavelog;
      } else {
        return null;
      }
    };

    //取未保存抄表记录总数
    service.getUnSavedValueLogCount = function() {
      var valuelogsdb = getValueLogCacheDB();
      if (!valuelogsdb.opdata) {
        valuelogsdb.opdata = [];
      }
      return valuelogsdb.opdata.length;
    };

    //保存指定房间抄表记录(批量)
    //valuelogs:抄表对象
    service.SaveRoomValue = function(valuelogs, isbackground) {
      isbackground = _.isBoolean(isbackground) ? isbackground : true;
      //返回promise
      return $q(function(resolve, reject) {
        if (!_.isArray(valuelogs)) {
          return reject('参数错误!');
        }
        var url = '/valuelogs/batchCreate', postdata = valuelogs;
        $http.post(url, postdata, {
            headers: {
              isbackground: isbackground
            }
          })
          .success(function(response) {
            var results = response;
            _.forEach(results, function(result){
              var valuelog = result.valuelog;
              var buildingId,roomId,productId;
              //判断返回值
              if(!valuelog || !valuelog.building || !valuelog.room || !valuelog.product){
                return console.log('返回对象错误!',valuelog);
              }
              buildingId = valuelog.building;
              roomId = valuelog.room;
              productId = valuelog.product;
              if(result.err){
                incValueLogCacheRetry(buildingId, productId, roomId, new Date(valuelog.readtime));
              }else{
                //判断返回值
                if(!valuelog || !valuelog.building || !valuelog.room || !valuelog.product){
                  return console.log('返回对象错误!',valuelog);
                }
                buildingId = valuelog.building;
                roomId = valuelog.room !== valuelog.building ? valuelog.room : valuelog.masterindex;
                productId = valuelog.product;
                // TODO: 以后增加离线抄表功能,先保存至缓存,在同步到服务器
                //修改抄表记录到缓存
                var roomvalue = DATA[buildingId][productId][roomId];
                if(roomvalue){
                  roomvalue.readtime = valuelog.readtime;
                  roomvalue.value = valuelog.value;
                  save2Cache();
                }else{
                  return console.log('更新缓存失败!',valuelog);
                }

                //更新房间状态至未收费
                if (result.nofee) {
                  var roomsdb = $Cache.get(TABLENAME_ROOMS);
                  if (roomsdb[buildingId] && roomsdb[buildingId][roomId]) {
                    var cacheRoom = roomsdb[buildingId][roomId];
                    cacheRoom.roomstatus = DATALIST.ROOM_STATUS.NOFEE;
                    //fixRoom
                    $Room.fixRoom(cacheRoom);
                    //保存缓存
                    $Cache.saveTable(TABLENAME_ROOMS);
                  }
                }

                //更新最新的收租单
                if (result.invoice) {
                  $Invoice.saveRoomLastInvoice2Cache(result.invoice);
                }

                //清除抄表操作记录
                service.deleteValueLogCache(buildingId, productId, roomId, new Date(valuelog.readtime), valuelog.value);

              }
            });
            //调用回调函数
            resolve(results);
          })
          .error(function(response, status, headers, config) {
            //设置重试次数
            _.forEach(postdata, function(valuelog){
              var buildingId,roomId,productId;
              //判断返回值
              if(!valuelog || !valuelog.building || !valuelog.room || !valuelog.product){
                console.log('返回对象错误!',valuelog);
              }
              buildingId = valuelog.building;
              roomId = valuelog.room;
              productId = valuelog.product;
              incValueLogCacheRetry(buildingId, productId, roomId, new Date(valuelog.readtime));
            });
            return reject(response, status, headers, config);
          });
      });
    };

    //切换当前抄表收费项目
    //product:产品对象或_id
    service.setCurValueProduct = function(product){
      curProduct = product && product._id ? product._id : product;
    };
    //获取当前抄表收费项目
    //返回:product产品对象
    service.getCurValueProduct = function(){
      return curProduct;
    };

    //切换当前排序方式
    //order:排序方式
    service.setCurOrder = function(buildingId, productId, order){
      //判断是否排序方式是否已定义
      var indexObject  = _.find(orderOptions, function(item){
        return item.value === order;
      });
      if(indexObject && DATA[buildingId] && DATA[buildingId][productId] && DATA[buildingId][productId][INDEXNAME]){
        DATA[buildingId][productId][INDEXNAME].value = order;
      }
      //写入缓存
      save2Cache();
    };

    //获取当前排序方式
    service.getCurOrder = function(buildingId, productId){
      if(DATA[buildingId] && DATA[buildingId][productId] && DATA[buildingId][productId][INDEXNAME]){
        return DATA[buildingId][productId][INDEXNAME].value;
      }else{
        return orderOptions[0].value;
      }
    };

    //加载缓存
    if (!DATA) {
      loadCacheData();
    }

    return service;
  });
