angular.module('bagu')

.service('$Sms', function($q, $http, $ionicPopup, _, $Mustache) {

  /**
  * 校验正则表达式
  */
  var regexp_mobile = /^1[345678]{1}\d{9}/gim;

  //服务对象
  var service = {
  };

  /**
   * 是否正确手机号码
   */
  service.isMobile =function(mobile) {
    regexp_mobile.lastIndex = 0;
    return regexp_mobile.test(mobile);
  };

  //生成短信内容
  //smsScope:环境
  //template:短信模板
  service.genSMSContent = function(smsScope, template) {
    return $Mustache.render(template, smsScope);
  };

  //群发短信
  //building:楼房ID
  //rooms:需要发送催缴短信的房间列表
  //content: 短信模板
  service.sendRoomsSms = function(building, rooms, allcustomer, content) {
    if(!_.isArray(rooms)){
      rooms = [rooms];
    }
    var roomIds = _.map( rooms, function(item){
      return item && item._id ? item._id : item;
    });
    //返回promise
    return $q(function(resolve, reject) {
      if(!building){
        return reject('楼房参数错误!');
      }
      var buildingId = building._id ? building._id : building;
      var postdata = {
        rooms:roomIds,
        allcustomer: allcustomer,
        content: content
      };
      //无缓存数据或强制刷新,则从远程取
      $http.post('/sms/building/'+buildingId+'/rooms',postdata)
      .success(function (response) {
        //回调
        resolve(response);
      })
      .error(function (err){
        reject(err);
      });
    });
  };

  //发送催缴短信
  //building:楼房ID
  //invoices:需要发送催缴短信的租金单列表
  service.sendInvoiceSms = function(building, invoices) {
    if(!_.isArray(invoices)){
      invoices = [invoices];
    }
    //返回promise
    return $q(function(resolve, reject) {
      if(!building){
        return reject('楼房参数错误!');
      }
      var buildingId = building._id ? building._id : building;
      var postdata = {invoices:invoices};
      //无缓存数据或强制刷新,则从远程取
      $http.post('/sms/building/'+buildingId+'/invoice',postdata)
      .success(function (response) {
        //回调
        resolve(response);
      })
      .error(function (err){
        reject(err);
      });
    });
  };

  //发送调租短信
  //room:房间ID
  //pricelog:需要发送调租对象
  service.sendChangePriceSms = function(room, pricelog) {
    //返回promise
    return $q(function(resolve, reject) {
      if(!room || !pricelog){
        return reject('参数错误!');
      }
      var roomId = room._id ? room._id : room;
      var pricelogId = pricelog._id ? pricelog._id : pricelog;
      //发送请求
      $http.post('/sms/room/'+roomId+'/changeprice/'+pricelogId)
      .success(function (response) {
        //回调
        resolve(response);
      })
      .error(function (err){
        reject(err);
      });
    });
  };

  //发送短信预览
  //content:短信内容
  //mobile:预览手机号码
  service.sendPreviewSms = function(building, content, mobile) {
    //返回promise
    return $q(function(resolve, reject) {
      if(!building || !content || !service.isMobile(mobile)){
        return reject('参数错误!');
      }
      var buildingId = building._id ? building._id : building;
      //发送请求
      $http.post('/sms/building/' + buildingId + '/preview',{
        content: content,
        mobile: mobile
      })
      .success(function (response) {
        //回调
        resolve(response);
      })
      .error(function (err){
        reject(err);
      });
    });
  };


  //输出服务对象
  return service;
});
